;; Safely execute commands sent from a server to a browser.
;;
;; Take as input JSON, convert it to s-exps, and produce as output Javascript
;; that does not give away decrypted data.

(def srextest()
  "alert('srextest worked!')")

;; verify no command can hack us, by removing characters that spread
;; hacking terror
(def srex-clean-item (item)
  (withs (clean item)
	 ;;(prn "will clean item: " item " of type " (type item))
	 (if (or (is (type item) 'int)
		 (is (type item) 'char)
		 (is (type item) 'sym))
	     (= clean item)
	     (do (each c (list " " ";" "\r" "\n" "\"" "'"    ;; common injection attacks
			       "<" ">" ">>" "<<" ":" "*"
			       "!" "&" "|" "||" "&" "&&" "`" ;; shell injection attacks
			       "$" ".") ;; forbid DOM access through jQuery
		       (= clean (cut clean 0 (findsubseq c clean))))
		 ;;(prn "cleaned item from " item " to " clean " of type " (type clean))
		 ))
	 ;;(prn "produced: " clean " of type " (type clean))
	 clean))

(def srex-clean-table (commands-tab)
  (withs (rettab (table))
	 (maptable (fn (k v)
		       (withs (newk (coerce k 'sym))
			      ;; When the plugin calls JSON.parse, it
			      ;; ends up converting all keys from
			      ;; symbols to strings. Convert them back
			      ;; to symbols.
			      ;;(prn "table key: " k " of type " (type k) " changed to type " (type newk))
			      (if (is (type v) 'cons)
				  (= (rettab newk) (srex-clean-list v))
				  (is (type v) 'table)
				  (= (rettab newk) (srex-clean-table v))
				  (= (rettab newk) (srex-clean-item v))))) commands-tab)
	 rettab))

(def srex-clean-list (commands-list)
  (withs (l nil)
	 (each item commands-list
	       (if (is (type item) 'cons)
		   (push (srex-clean-list item) l)
		   (is (type item) 'table)
		   (push (srex-clean-table item) l)
		   (push (srex-clean-item item) l)))
	 (rev l)))

(def srex (fromwebserver cmd pubkey encprivkey encencpass)
  (withs (out "" env (table) clean-list (srex-clean-list (cmd "commands")))
	 (= env!handlercnt 0)
	 (= env!pubkey pubkey)
	 (= env!encprivkey encprivkey)
	 (= env!encencpass encencpass)
	 (= env!form-inputs (srex-clean-list (cmd "form-inputs")))
	 (= env!code (cmd "code"))
	 (= env!fns (table))

	 ;; variables set to a value that originating from decrypted data
	 (= env!vars-using-dec (table))
	 (= env!fns-using-dec (table))
	 ;; variables set to a value that will be encrypted
	 (= env!vars-using-enc (table))
	 (= env!fns-using-enc (table))

	 ;; verify the remote execution commands
	 (each token clean-list
	       (= out (str out (srex-cmd token env))))

	 ;; add savepubkeyto from contact-list to enc-itemlist
	 (srex-addto-enc-itemlist env)

	 ;; initalize hook variables
	 (= env!srex-befenc-all nil) ;; hook before all fields will be encrypted
	 (= env!srex-befenc (table)) ;; table of hooks before each field will be encrypted
         (= env!srex-aftdec (table)) ;; table of hooks after  each field will be encrypted
         (= env!srex-aftdec-all nil) ;; hook after all fields were decrypted
	 (= env!srex-used (table))   ;; table of internal functions that are used
	 (= env!srex-hack-attempt nil)

	 ;; verify the remote execution code
	 (= out (str out (srex-codewalk env)))
	 (= out (str out (srex-print env)))
	 (= out (str out (srex-remove-buttons env)))
	 (= out (str out (srex-enc-functions env)))
	 (= out (str out (srex-encencpass fromwebserver env!pubkey encprivkey env!encencpass env!pubuserid)))
	 (= out (str out (encsearch-js env!enc-itemlist
				       env!dec-itemlist env!autocomp-itemlist)))
	 (= out (str out (srex-enc env fromwebserver)))
	 (= out (str out (srex-submit env)))
	 (if env!dec-itemlist
	     (= out (str out (srex-dec-core fromwebserver env!encprivkey env!encencpass env!marktype env!dec-itemlist env!srex-aftdec env!srex-aftdec-all)))
	     (= out (str out (srex-process-aftdec-all env))))
	 (= out (str out (srex-verify-contacts env!verify-list)))
	 (= out (str out (srex-warn-unencrypted env)))
	 (= out (str out (srex-warn-leaving-browser env)))

	 (if env!srex-hack-attempt
	     (= out (str out "// Alert the user there was an unsuccessful attempt to hack them
error_messages.push('" srex-hack-attempt-msg "');
" (srex-disable-all-links) "
console.log('" srex-hack-attempt-msg "');
alert('" srex-hack-attempt-msg "');
"
)))
	 (= out (str out (srex-warn-process env!pubuserid)))
	 (= out (str out (srex-warnsign env)))
	 (= out (str out (srex-warnauthors)))
	 out))

(def srex-cmd (cmd env)
  (withs (out "")
	 (if (is (type cmd) 'cons)
	     (do (if (is (str (cmd 0)) "alert")
		     (= out (str out "alert('" (cmd 1) "');
"))
		     (is (str (cmd 0)) "may_publish")
		     (= env!may_publish (cmd 1))
		     (is (str (cmd 0)) "print")
		     (= env!may_print (cmd 1))
		     (is (str (cmd 0)) "submit") ;; an href, not a button; easier to style
		     (= env!submit-btn (cmd 1))
		     (is (str (cmd 0)) "encrypt")
		     (withs (encdata (cmd 1))
			    (= env!pubuserid encdata!pubuserid)
			    (= env!formbuttons encdata!formbuttons)
			    (= env!enc-itemlist encdata!enc-itemlist)
			    (= env!hash-itemlist encdata!hash-itemlist)
			    (= env!dec-itemlist encdata!dec-itemlist)
			    (= env!marktype encdata!marktype)
			    (= env!autocomp-itemlist encdata!autocomp-itemlist))
		     (is (str (cmd 0)) "decrypt")
		     (withs (decdata (cmd 1))
			    (= env!pubuserid decdata!pubuserid)
			    (= env!marktype decdata!marktype)
			    (= env!dec-itemlist decdata!dec-itemlist))
		     (is (str (cmd 0)) "verify")
		     (withs (verifydata (cmd 1))
			    (= env!verify-list verifydata!verify-list))
		     )))
	 out))

(def srex-print (env)
  (withs (out "")
	 (if env!may_print
	     (do (= out (str out "function print_page() {
  var elements = document.getElementsByClassName('noprint');
  var counter = 0;

  while (counter < elements.length) {
    elements[counter].style.visibility = 'hidden';
    counter++;
  }

  window.print();
  counter = 0;

  while (counter < elements.length) {
    elements[counter].style.visibility = 'visible';
    counter++;
  }
}
"))
		 (= out (str out "
document.getElementById('" env!may_print "').onclick = print_page;
document.getElementById('" env!may_print "').href = \"#\";
"))))
	 out))
		      
(def srex-remove-buttons (env)
  (withs (out "")
	 (= out (str out "
// Remove all buttons so you can't submit anything.
var buttonsList = document.getElementsByTagName('input');
for (var i = 0; i < buttonsList.length; i++) {
   if (buttonsList[i].type == 'button') {
      buttonsList[i].parentNode.removeChild(buttonsList[i]);
   }
}

"))
	 out))

(def srex-on-contact-list (formbuttons fndo)
  (each item formbuttons
	(if item!contact-list
	    (each contact item!contact-list
		  (fndo contact)))))

(def srex-find-in-enc-itemlist (id enc-itemlist)
  (withs (ret nil)
	 (each item enc-itemlist
	       (if (is item!id id)
		   (= ret item)))
	 ret))

(def srex-addto-enc-itemlist (env)
  (srex-on-contact-list env!formbuttons
			(fn (contact)
			    (if contact!savepubkeyto
				;; don't add this id again if it already exists
				;; but do set the existing one to force signing the pubkey
				(withs (existing (srex-find-in-enc-itemlist contact!savepubkeyto env!enc-itemlist))
				    (if (no existing)
					(push (obj id contact!savepubkeyto
						   forcesign t) ;; force signing the pubkey
					      env!enc-itemlist)
					(withs (new-enc-itemlist nil)
					       (each item env!enc-itemlist
						     (if (is item!id contact!savepubkeyto)
							 (= item!forcesign t))
						     (push item new-enc-itemlist))
					       (= env!enc-itemlist new-enc-itemlist))
					))))))

(def srex-only-searchable (enc-itemlist)
  (withs (ret t)
	 (each item enc-itemlist
	       (if (and (no item!tosearch)
			(no item!toretrieve))
		   (= ret nil)))
	 ret))

;; check if all items in enc-itemlist are forced to be signed: if all
;; of them are public keys that should be encrypted
(def srex-only-forcesign (enc-itemlist formbuttons)
  (withs (forcesigncnt 0)
	 (srex-on-contact-list formbuttons
			       (fn (contact)
				   (if contact!savepubkeyto
				       (++ forcesigncnt))))
	 (if (is (len enc-itemlist) forcesigncnt)
	     t nil)))

(def srex-iscontactpubkey (id formbuttons)
  (withs (ret nil)
	 (srex-on-contact-list formbuttons
			       (fn (contact)
				   (if (is id contact!savepubkeyto)
				       (= ret t))))
	 ret))

(def srex-iscontactsavetextarea (id formbuttons)
  (withs (ret nil)
	 (srex-on-contact-list formbuttons
			       (fn (contact)
				   (if contact!saveistextarea)
				   (= ret t)))
	 ret))

(= SHA512-len* 128)
(def srex-enc-core (enc-itemlist hash-itemlist srex-befenc encpassid formbuttons)
  (withs (ret "")

	 (= ret (str ret "
    // Encrypt each data field"))
	 (if encpassid
	     (= ret (str ret "
    // Prepare an array of keys, one per recipient, to support encrypting
    // data for multiple recipients.
    var keymanager_keys = [];
    {
       recipients_counter = 0;
       while (recipients_counter < all_recipients_total) {
          var this_id = Object.keys(all_recipients)[recipients_counter];
          var local_this_pubkey = all_recipients[this_id]['pubkey'];
          var local_keymanager = openpgp.key.readArmored(local_this_pubkey);
          keymanager_keys.push(local_keymanager.keys[0]);
          recipients_counter++;
       }
    }
")))

	 ;; Prepare any values in fields before encrypting
	 (each item enc-itemlist
	       (withs (id item!id
		       befenc (srex-befenc id))
		      (if befenc
			  (= ret (str ret "    // Server-supplied code for srex-before-encrypt
    " befenc)))))

	 ;; encrypt
	 (each item enc-itemlist
	       (withs (id item!id
		       tosearch item!tosearch       ;; encrypt to store so we can search
		       tosearchout item!tosearchout ;; will hold output of encryption
		       toretrieve item!toretrieve   ;; encrypt to perform the search
		       toretrievekeys item!toretrievekeys ;; number of word keys to generate
		       toretrieveout item!toretrieveout ;; will hold encrypted input to search query
                       iscontactpubkey (srex-iscontactpubkey id formbuttons)
                       forcesign item!forcesign
		       istextarea item!istextarea)
		      (if iscontactpubkey
			  (withs (contactsaveistextarea (srex-iscontactsavetextarea id formbuttons))
				 (= ret (str ret "
    var contact_pubkey = contact_keyinfo['pubkey'];
    document.getElementById('" id "')." (srex-accesstype contactsaveistextarea) " = contact_pubkey;"))))
		      (= ret (str ret "
    var val" id " = document.getElementById('" id "').value;"))
		      (= ret (str ret "
    if (val" id " != '') { // don't encrypt blanks, for error checking"))
		      (if tosearch
			  ;; must do more work to support encrypted search
			  (do ;; if there is no searchout field, fall back to using the id
			      (if (no tosearchout)
				  (= tosearchout id))
			      ;; don't clutter a visible field; also don't send the plaintext
			      (= ret (str ret "
        document.getElementById('" tosearchout "').value = encrypt_tosearch(val" id ", searchsalt, searchpass, searchiv, searchmperc);
        document.getElementById('" id "').value = '';")))
			  toretrieve
			  ;; must send an encrypted query
			  (do (if (no toretrievekeys)
				  (= toretrievekeys 0))
			      ;; don't clutter a visible field; also don't send the plaintext
			      (= ret (str ret "
        document.getElementById('" toretrieveout "').value = encrypt_toretrieve(val" id ", searchsalt, searchpass, searchiv, searchmperc, " toretrievekeys ");
        document.getElementById('" id "').value = '';")))
			  ;; else -- just encrypt
			  (= ret (str ret "
       " ;; if we are encrypting a contact's public key, encrypt it
	  ;; only for us
	  (if iscontactpubkey
	      (str "document.getElementById('" id "').value = encrypt_nonsearchable(val" id ", [pubkey_openpgp], " (if forcesign "1" "sign") ", [privkey_openpgp]);")
	      (str "document.getElementById('" id "').value = encrypt_nonsearchable(val" id ", keymanager_keys, sign, [privkey_openpgp]);")))))
		      (= ret (str ret "
    }
"))
		      ))
	 (= ret (str ret "
"))

	 ;; hash
	 (if hash-itemlist
	     (do (= ret (str ret "
    // Hash each item
    var inmessage = null;
    var outmessage = null;
"))

		 (each id hash-itemlist
		       ;; Don't hash a second time if the password was
		       ;; saved by the browser's password manager as a
		       ;; SHA-512 value
		       (= ret (str ret "
    inmessage = document.getElementById('" id "').value;
    if (inmessage.length == " SHA512-len* ") 
        outmessage = inmessage;
    else
        outmessage = CryptoJS.SHA512(inmessage);
    document.getElementById('" id "').value = outmessage;
")))
	       ))
	 ret))

(def srex-disable-all-links ()
  (withs (out "")
	 (= out (str out "var anchors = document.getElementsByTagName(\"a\");
for (var i = 0; i < anchors.length; i++) {
    anchors[i].onclick = function() {return false;};
    anchors[i].style.color = 'red';
    anchors[i].href = 'file:///THIS_LINK_WAS_DISABLED_TO_PROTECT_YOU';
}
"))
	 out))

(def srex-warn-process (pubuserid)
  (str "// Show the author of all messages
function authors_unique() {
   var author_counter = 0;
   var all_authors = {};

   while (author_counter < author_messages.length) {
     var author = author_messages[author_counter];
     var userid = author[1];

     all_authors[userid] = 1;
     author_counter++;
   }

   return Object.keys(all_authors).length;
}

function authors_only_us () {
   if (authors_unique() == 1) {
      var author = author_messages[0];
      var userid = author[1];
      if (userid == '" pubuserid "') {
         return true;
      }
   }
   return false;
}

if (author_messages.length > 0) {
   var author_counter = 0;

   // Create all author verification messages
   while (author_counter < author_messages.length) {
     var author = author_messages[author_counter];
     var fieldname = author[0];
     var userid = author[1];
     create_author(fieldname, userid);
     show_author(fieldname, true);
     author_counter++;
   }
}

// Alert the user on the top left of the screen if there were errors
function alert_errors () {
  if (error_messages.length > 0) {
     var error_counter = 0;
     var innerHTML = '';

     while (error_counter < error_messages.length) {
         innerHTML = innerHTML + error_messages[error_counter] + '<br>';
         error_counter++;
     }

     var div = document.createElement('div');
     div.id = 'srex-alarm';
     div.style.textAlign = 'left';
     div.style.position = 'absolute';
     div.style.left = 15;
     div.style.top = 15;
     div.style.font = '13px Verdana';
     div.style.padding = '14px';
     div.style.color = 'white';
     div.style.backgroundColor = 'red';
     div.innerHTML = innerHTML;

     document.body.appendChild(div);
  }
}
alert_errors();

"))

(def srex-warn-unencrypted (env)
  (withs (ret "")
	 ;; Don't encrypt hidden values, and never set them. Hidden
	 ;; values should be created as textareas with style
	 ;; "display:none" so they stay hidden visually but are
	 ;; guaranteed to be encrypted.
	 ;; Raise alarms for every field that won't be encrypted
	 (= ret (str ret "

// ---------- alert the user if there are any fields that won't be encrypted
var found_fields_to_not_encrypt = false;
"))
	 (each elementtype (list "textarea" "input")
	       (= ret (str ret "
// Inspect every " elementtype "
var " elementtype "List = document.getElementsByTagName('" elementtype "');
for (var i=0; i < " elementtype "List.length; i++) {
"
  (srex-warn-unencrypted-each elementtype env!enc-itemlist env!hash-itemlist env!autocomp-itemlist)
"
}
")))
	 (= ret (str ret "
if (found_fields_to_not_encrypt) {
  // Alert the user some fields won't be encrypted
  error_messages.push(\"The input fields in red won't be encrypted.\");
}

"))
         ret))

(def srex-warn-unencrypted-each (varname enc-itemlist hash-itemlist autocomp-itemlist)
  (withs (ret "" counter 0)
	 (= ret (str ret "
    if ( "))
	 (if (is varname "input")
	     (do (= ret (str ret "(" varname "List[i].type == 'text' || " varname "List[i].type == 'password')"))
		 (++ counter)))
	 (each item enc-itemlist
	       (withs (id item!id)
		      (if (> counter 0)
			  (= ret (str ret " && ")))
		      (= ret (str ret varname "List[i].name != '" id "'")))
	       (++ counter))
	 (each id hash-itemlist
	       (if (> counter 0)
		   (= ret (str ret " && ")))
	       (= ret (str ret varname "List[i].name != '" id "'"))
	       (++ counter))
	 (each item autocomp-itemlist
	       (withs (id item!searchin)
		      (if (> counter 0)
			  (= ret (str ret " && ")))
		      (= ret (str ret varname "List[i].name != '" id "'"
				  " && " varname "List[i].id != 'token-input-" id "'")))
	       (++ counter))
	 (if (> counter 0)
	     (= ret (str ret " && ")))
	 (= ret (str ret "true ) {
       console.log(" varname "List[i].name + ' will not be encrypted');
       " varname "List[i].style.backgroundColor='red';
       " varname "List[i].style.color='white';
       found_fields_to_not_encrypt = true;
    }
"))
	 ret))

(def srex-warn-leaving-browser (env)
  (withs (ret "" found-sendout nil)
	 (maptable (fn (k v)
		       (if (is k "jq-ajax")
			   (= found-sendout t))
		       ) env!fns-using-enc)
	 (if found-sendout
	     (do (= ret (str ret "// Alert the user some fields are sent out of the browser
error_messages.push(\"The input fields in red will be sent unencrypted out of the browser.\");

"))
		 (maptable (fn (k v)
			       ;;(prn (str "k: " k))
			       (if (srex-is-encrypted-data env (str k) nil)
				   (do ;;(prn (str "var " k " is encrypted"))
				       (= ret (str ret "var val" k " = document.getElementById('" k "');
val" k ".style.backgroundColor='red';
val" k ".style.color='white';

"))
				       ))
			       ) env!vars-using-enc)
		 ))
	 ret))

(def srex-warnsign (env)
  (withs (out "")
	 (if (and env!enc-itemlist
		  (no (srex-only-searchable env!enc-itemlist))
		  (no (srex-only-forcesign env!enc-itemlist env!formbuttons)))
	     (= out (str out "
// warn the user if the encrypted data won't be signed
{
  var div = document.createElement('div');
  div.id = 'srex-sign-alarm';
  div.style.textAlign = 'left';
  div.style.position = 'absolute';
  div.style.right = 15;
  div.style.top = 15;
  div.style.font = '13px Verdana';
  div.style.padding = '14px';
  div.style.color = 'white';
  div.style.backgroundColor = 'orange';
  div.style.display = 'none';
  div.innerHTML = 'The encrypted data will not be digitally signed.';
  document.body.appendChild(div);
}

function check_signing () {
  if (document.getElementById('sign') == null ||
      document.getElementById('sign').checked == false)
     document.getElementById('srex-sign-alarm').style.display = 'block';
  else
     document.getElementById('srex-sign-alarm').style.display = 'none';

  // published data are neither encrypted nor signed
  if (document.getElementById('publish') &&
      document.getElementById('publish').checked == true) {
     document.getElementById('srex-sign-alarm').style.display = 'none';
     if (document.getElementById('sign'))
        document.getElementById('sign').disabled = true;
  } else {
     if (document.getElementById('sign'))
        document.getElementById('sign').disabled = false;
  }
}

if (document.getElementById('sign')) {
   document.getElementById('sign').onclick = check_signing;
}

if (document.getElementById('publish')) {
   document.getElementById('publish').onclick = check_signing;
}

check_signing();
")))
	 out))

(def srex-warnauthors ()
  (withs (out "")
	 (= out (str out "
// warn the user author information will not be shown
{
  var div = document.createElement('div');
  div.id = 'srex-showauthors-alarm';
  div.style.textAlign = 'left';
  div.style.position = 'absolute';
  div.style.right = 15;
  div.style.top = 50;
  div.style.font = '13px Verdana';
  div.style.padding = '14px';
  div.style.color = 'white';
  div.style.backgroundColor = 'orange';
  div.style.display = 'none';
  div.innerHTML = 'Author user ids will not be shown.';
  document.body.appendChild(div);
}

function check_authors() {
  if (document.getElementById('showauthors') != null) {
     if (document.getElementById('showauthors').checked == false) {
        // If all messages have been authored by us don't show the alarm
        if (authors_only_us() == false)
           document.getElementById('srex-showauthors-alarm').style.display = 'block';
        show_all_authors(false);
     } else {
        document.getElementById('srex-showauthors-alarm').style.display = 'none';
        show_all_authors(true);
     }
  } else {
     // If all messages have been authored by us don't show the alarm
     if (authors_only_us() == false)
        document.getElementById('srex-showauthors-alarm').style.display = 'block';
     show_all_authors(false);
  }
}

if (document.getElementById('showauthors')) {
   document.getElementById('showauthors').onclick = check_authors;
}
check_authors();
"))
	 out))

(def srex-encencpass (fromwebserver pubkey encprivkey encencpass pubuserid)
  (str "var error_messages = [];
var author_messages = [];

var pubkey = " (pgp-tomultiline pubkey "             ") ";
var pubkey_openpgp = openpgp.key.readArmored(pubkey).keys[0];

// Extract the encrypted encryption password
var encencpass = " (if fromwebserver
		       "localStorage.getItem('encrypted_encryption_password')"
		       (str "'" encencpass "'")) ";

// Decrypt the private key
var encprivkey = '" encprivkey "';
var privkeyObj = CryptoJS.AES.decrypt( encprivkey, encencpass );
var privkey = privkeyObj.toString(CryptoJS.enc.Utf8);

// Prepare for decrypting each item
var privkey_openpgp = openpgp.key.readArmored(privkey).keys[0];
privkey_openpgp.decrypt(encencpass);

// Decrypt the verified keyids
var encverifiedkeyids = document.getElementById('encverifiedkeyids').value;
var verifiedkeyids = {};
verifiedkeyids = decrypt_nonsearchable('encverifiedkeyids', encverifiedkeyids, privkey_openpgp);
verifiedkeyids = JSON.parse(verifiedkeyids);

// Decrypt a non-searchable field "
;; when called from autocomplete_to_encpass,
;; there is no fieldname, so don't break if a
;; blank one is passed
"
function create_author(fieldname, userid) {
  if (document.getElementById(fieldname).type != 'hidden') {
    var div = document.createElement('div');
    div.id = fieldname + '_author_verification';
    div.style.textAlign = 'left';
    div.style.font = '11px Verdana';
    div.style.color = 'white';
    div.style.position = 'absolute';
    var field_top = document.getElementById(fieldname).getBoundingClientRect().top;
    var field_left = document.getElementById(fieldname).getBoundingClientRect().left;
    div.style.top = field_top - 20;
    div.style.left = field_left;
    div.innerHTML = 'by ' + userid;
    div.style.width = '120px';
    div.style.padding = '3px';
    div.style.backgroundColor = 'green';

    document.body.appendChild(div);
  }
}

function show_author(fieldname, show) {
  if (document.getElementById(fieldname + '_author_verification') != null) {
     if (show)
        document.getElementById(fieldname + '_author_verification').style.display = 'block';
     else
        document.getElementById(fieldname + '_author_verification').style.display = 'none';
  }
}

function show_all_authors(show) {
  var author_counter = 0;

  while (author_counter < author_messages.length) {
    var author = author_messages[author_counter];
    var fieldname = author[0];
    show_author(fieldname, show);
    author_counter++;
  }
}

function decrypt_nonsearchable(fieldname, encfieldvalue, decrypt_privkey_openpgp) {
  var field = encfieldvalue;

  if (encfieldvalue != '') {
    var messagefield = openpgp.message.readArmored( encfieldvalue );
    var decresult = {};
    var msg = messagefield;
    msg = msg.decrypt(decrypt_privkey_openpgp);
    decresult.text = msg.getText();
    if (decresult.text) {
      // success
      console.log('decrypted ' + fieldname + ' to ' + decresult.text);

      // verify all signatures were valid
      decresult.signatures = msg.verify([ decrypt_privkey_openpgp ]);
      var signatures_counter = 0;
      var signatures_total = Object.keys(decresult.signatures).length;
      console.log('There are ' + signatures_total + ' signatures for ' + fieldname);
      var signature_verification_failed = 0;
      var user_verification_failed = 0;
      while (signatures_counter < signatures_total) {
        var sigkey = Object.keys(decresult.signatures)[signatures_counter];
        var sigvalue = decresult.signatures[sigkey];
        if (sigvalue.valid == 0) {
            console.log('invalid signature found');
            error_messages.push(\"The field '\" + fieldname + \"' in red failed signature verification. \" +
                                'Odds are SOMEONE IS EAVESDROPPING ON YOU RIGHT NOW.');
            signature_verification_failed = 1;
        } else {
          // check if this signature is from a user we verified
          var keyid = sigvalue.keyid.toHex();
          console.log('will check keyid: ' + keyid);
          console.log('with verifiedkeyids: ');
          console.log(verifiedkeyids);
          var userid = verifiedkeyids[keyid];
          if (userid == null) { // also check if the keyid was ours
             var our_keyid = pubkey_openpgp.getKeyIds()[0].toHex();
             console.log(our_keyid);
             if (our_keyid == keyid) {
               console.log('the keyid was our own: ' + our_keyid);
               author_messages.push([fieldname, '" pubuserid "']);
             } else {
               console.log('the signature is from an unknown user');
               error_messages.push(\"The field '\" + fieldname + \"' in red was signed by someone you haven't verified.\");
               user_verification_failed = 1;
             }
          } else {
            author_messages.push([fieldname, userid]);
          }
        }
        signatures_counter++;
      }
      if (signatures_total == 0) {
        error_messages.push(\"The data field '\" + fieldname + \"' in red was not digitally signed. \");
      }
      if (signatures_total == 0 || signature_verification_failed == 1 || user_verification_failed) {
          if (document.getElementById(fieldname) != null) {
             document.getElementById(fieldname).style.backgroundColor='red';
             document.getElementById(fieldname).style.color='white';
          }
      }
    } else {
      console.log('failed to decrypt');
    }

    field = decresult.text;
    console.log('decrypted to: ' + field);
  }

  return field;
}

" (encrypt-nonsearchable) "
"))

(def srex-encrypt-verifiedkeyids (spacing forkeys)
  (str "
" spacing "// encrypt the verifiedkeyids
" spacing "var this_verifiedkeyids = JSON.stringify(verifiedkeyids);
" spacing "encverifiedkeyids = encrypt_nonsearchable(this_verifiedkeyids, " forkeys ", 1, [privkey_openpgp]);
" spacing "document.getElementById('encverifiedkeyids').value = encverifiedkeyids;"))

(def srex-contact-dialogids (dialogids)
  (withs (dialogid (str "dialog_" (srex-uniq)))
	 (push dialogid dialogids)))

(def srex-contact-functions (contact dialogid next btn)
  (withs (out "")
	 (= out (str out "// must " contact!action " contact for " contact!id "
var " dialogid " = document.createElement('dialog');

function " dialogid "_yes () {
  // add to verifiedkeyids
  var contact_pubkey = contact_keyinfo['pubkey'];
  var contact_pubkey_openpgp = openpgp.key.readArmored(contact_pubkey).keys[0].primaryKey;
  var keyid = contact_pubkey_openpgp.getKeyId().toHex();
  var userid = contact_keyinfo['userid'];
  if (keyid != null && userid != null) {
     // mark this key as verified
     verifiedkeyids[keyid] = userid;
  }
  " dialogid ".close();
  // go to next dialog, or submit
  " (if next
	(str next ".showModal();")
	"") "
  encrypt_before_submit_" (srex-valid-js btn) "();
}

function " dialogid "_no() {
  " dialogid ".close();
  // go to next dialog, or submit
  " (if next
	(str next ".showModal();")
	;; no need to encrypt the verifiedkeyids again
	(str "encrypt_before_submit_" (srex-valid-js btn) "();")) "
}

"))
	 out))

;; Larry and Sergey won't respect you in the morning.
;; http://old.ycombinator.com/ad.html
(def srex-contact-add (env contact dialogid)
  (withs (out "")
	 ;; note to self: add html.arc
	 (= out (str out "
  // verify contact for " contact!id "
  contact_keyinfo = " (env!fns (sym contact!fn)) "();
  " dialogid ".id = '" dialogid "';
  " dialogid ".style.borderColor = '" (hexrep (color 255 102 0)) "';
  " dialogid ".style.font = '13px Verdana';
  " dialogid ".style.padding = '14px';
  " dialogid ".style.backgroundColor = '#ffffee';
  " dialogid ".style.color = '" (hexrep (color 255 102 0)) "';
  " dialogid ".innerHTML = '<span><b>PLEASE VERIFY the public-key fingerprint of this contact:</b></span>' + '<br><br><br><br>';
  var userid = contact_keyinfo['userid'];
  " dialogid ".innerHTML = " dialogid ".innerHTML + userid + '<br><br>';
  var contact_pubkey = contact_keyinfo['pubkey'];
  if (contact_pubkey != null && contact_pubkey != '') {
    var contact_pubkey_openpgp = openpgp.key.readArmored(contact_pubkey).keys[0].primaryKey;
    var keyid = contact_pubkey_openpgp.getFingerprint();
    if (keyid != null && userid != null) {
       var readable_keyid = keyid.match(/.{1,4}/g).join(' ').toUpperCase();
       " dialogid ".innerHTML = " dialogid ".innerHTML + readable_keyid + '<br>';
    }

    var keyid_short = contact_pubkey_openpgp.getKeyId().toHex();
    if (verifiedkeyids[keyid_short]) {
      // this key was already added; don't override a previous one
      " dialogid ".innerHTML = " dialogid ".innerHTML + '<br><br>' +
\"This key was already added. Will not add it again to protect the existing one.\" + '<br><br>' +
'<table>' +
'<tr>' +
\"<td style='width:150px;'>\" +
\"<button id='" dialogid "_no_button'>OK</button>\" +
'</td>' + 
'</tr>' +
'</table>';
      document.body.appendChild(" dialogid ");
      document.getElementById('" dialogid "_no_button').onclick = " dialogid "_no;
    } else {
      // this key wasn't already added; add it

      " dialogid ".innerHTML = " dialogid ".innerHTML + '<br><br>' +
'<table>' +
'<tr>' +
\"<td style='width:150px;'>\" +
\"<button id='" dialogid "_yes_button'>Yes</button>\" +
'</td>' +
\"<td style='width:150px;'>\" +
'</td>' +
'<td>' +
\"<button id='" dialogid "_no_button'>No</button>\" +
'</td>' + 
'</tr>' + 
'</table>';
      document.body.appendChild(" dialogid ");
      document.getElementById('" dialogid "_yes_button').onclick = " dialogid "_yes;
      document.getElementById('" dialogid "_no_button').onclick = " dialogid "_no;
   }
  } else {
    // this user doesn't have a public key
    " dialogid ".innerHTML = " dialogid ".innerHTML + '<br><br>' +
\"This user doesn't have a public key. There's nothing to verify.\" + '<br><br>' +
'<table>' +
'<tr>' +
\"<td style='width:150px;'>\" +
\"<button id='" dialogid "_no_button'>OK</button>\" +
'</td>' + 
'</tr>' +
'</table>';
   document.body.appendChild(" dialogid ");
   document.getElementById('" dialogid "_no_button').onclick = " dialogid "_no;
  }
  " dialogid ".showModal();
  "))
	 out))

(def srex-enc (env fromwebserver)
  (withs (out "")

	 (each item env!autocomp-itemlist
	       (withs (url item!url enc-fieldlist item!enc-fieldlist
		       searchin item!searchin searchout item!searchout
		       toretrievekeys item!toretrievekeys
		       encpassid item!encpassid
		       eflout "" counter 0)
		      (= eflout (str eflout "["))
		      (each ef enc-fieldlist
			    (if (> counter 0)
				(= eflout (str eflout ", ")))
			    (= eflout (str eflout "'" ef "'"))
			    (++ counter))
		      (= eflout (str eflout "]"))
		      (= out (str out "jq_autocomplete_tokeninput_encrypted('" url "', " eflout  ", '" encpassid "', '#" searchin "', '" searchout "', " toretrievekeys ");
"))))

	 (each item env!formbuttons
	       (withs (form item!form btn item!btn encpassid item!encpassid
		       contact-list item!contact-list)
		      (if contact-list
			  (withs (counter 0 dialogids (list) willadd nil willdelete nil)
				 (= out (str out "var contact_keyinfo = {};
"))
				 (each contact contact-list
				       (if (is contact!action "add")
					   (= dialogids (srex-contact-dialogids dialogids))))
				 (each contact contact-list
				       (if (is contact!action "add")
					   (withs (next (if (< (+ counter 1) (len contact-list))
							    (dialogids counter) nil))
						  (= out (str out (srex-contact-functions contact (dialogids counter) next btn)))))
				       (++ counter))
				 (= out (str out "function verify_before_submit_" (srex-valid-js btn) "() {"))
				 (= counter 0)
				 (each contact contact-list
				       (if (is contact!action "delete")
					   (do (= willdelete t)
					       (= out (str out (srex-contact-delete env contact)))))
				       (++ counter))
				 (= counter 0)
				 (each contact contact-list
				       (if (is contact!action "add")
					   (do (= willadd t)
					       (= out (str out (srex-contact-add env contact (dialogids counter))))))
				       (++ counter))
				 (if willadd
				     (= out (str out "
  // Don't submit the form. Submit only if all verifications occur.
  return false;
}

"))
				     willdelete ;; but not willadd
				     (= out (str out "
  // Submit the form regardless
  " (srex-encrypt-verifiedkeyids "  " "[pubkey_openpgp]") "
  document.getElementById('" form "').submit();
  return true;
}
")))
				 ))

		      (= out (str out "function encrypt_before_submit_" (srex-valid-js btn) "() {
"))

		      (if env!srex-befenc-all
			  (= out (str out "  // Server-supplied code for srex-before-encrypt-all
  " env!srex-befenc-all "
")))

		      ;; if there are any autocomplete entries that
		      ;; contain an id and pubkey, add entries for
		      ;; them in encpass
		      (if env!autocomp-itemlist
			  (do (each item env!autocomp-itemlist
				    (withs (searchout item!searchout)
					   (= out (str out "
  // --------------------------------------------------------------------------
  // Build from autocomplete the new encpass variable
  var autocomplete_ret = autocomplete_to_encpass('" searchout "', '" encpassid "');
  if (autocomplete_ret == false) {
     // someone was eavesdropping, or someone didn't have a public key to encrypt for them
     alert_errors();
     return false;
  }
"))))
			      (= out (str out "

"))))
		      (if env!enc-itemlist
			  (do (= out (str out "  var sign = true;"))
			      ;; if processing only searchable
			      ;; encryption don't check for a signing
			      ;; message
			      (if (no (srex-only-searchable env!enc-itemlist))
				  (= out (str out "
  if (document.getElementById('sign') &&
      document.getElementById('sign').checked == true)
     sign = true;
  else
     sign = false;
")))))
		      (if (or env!enc-itemlist env!hash-itemlist)
			  (do (if env!may_publish
				  ;; the publish field can't be hidden when it was served; it must be checked by the user
				  (= out (str out "  var publish = false;
  // the publish field can't be hidden and checked when served; it must be checked by the user
  if (document.getElementById('publish') &&
      document.getElementById('publish').checked == true &&
      document.getElementById('publish').hidden == false)
      publish = true;

  if (publish) {
     document.getElementById('" form "').submit(); 
     return true; // Do not encrypt published
  }

")))

			      ;;(if env!enc-itemlist ;; do this always
			      (= out (str out "
"
  (if (no encpassid)
      "  var should_encrypt = 1;
"
      (str "
  // --------------------------------------------------------------------------
  // Detect if no recipient supports encryption, to not encrypt message
  var should_encrypt = 1;
  var recipients_counter = 0;
  var all_recipients = document.getElementById('" encpassid "').value;
  if (all_recipients == '') {
    all_recipients = {};
  } else {
    all_recipients = JSON.parse(all_recipients);
  }
  all_recipients['" env!pubuserid "'] = { 'pubkey': pubkey };
  var all_recipients_total = Object.keys(all_recipients).length;

  while (recipients_counter < all_recipients_total) {
     var this_id = Object.keys(all_recipients)[recipients_counter];
     var this_pubkey = all_recipients[this_id]['pubkey'];
     if (this_pubkey == '' || this_pubkey == null) {
         should_encrypt = 0;
     }
     recipients_counter++;
  }

"))

  "  if (should_encrypt == 1) {
"))

			      (= out (str out (srex-enc-core env!enc-itemlist env!hash-itemlist env!srex-befenc encpassid env!formbuttons)))
			      (= out (str out "  }
"))
			      (if (and encpassid env!enc-itemlist)
				  ;; don't try to encrypt unnecessarily
				  (= out (str out "
  // don't send public keys to the server
  recipients_counter = 0;
  while (recipients_counter < all_recipients_total) {
     var this_id = Object.keys(all_recipients)[recipients_counter];
     all_recipients[this_id]['pubkey'] = null;
     if (should_encrypt)
       all_recipients[this_id]['encrypted'] = true;
     else
       all_recipients[this_id]['encrypted'] = false;
     recipients_counter++;
  }

  // set the recipients
  all_recipients = JSON.stringify(all_recipients);
  document.getElementById('" encpassid "').value = all_recipients;
  " (srex-encrypt-verifiedkeyids "  " "[pubkey_openpgp]") "

  document.getElementById('" form "').submit();
")))
			      )) ;; (if (or env!enc-itemlist env!hash-itemlist)

		      (if (no (and encpassid env!enc-itemlist))
			  (= out (str out 
				      (if (srex-processing-contacts env)
					  (str "
  " (srex-encrypt-verifiedkeyids "  " "[pubkey_openpgp]"))) "
  document.getElementById('" form "').submit();
")))
		      (= out (str out "}
"))
		      ))
	 out))

(def srex-enc-autocomplete-prologue (env)
  (str "  // For every entry with an 'id' and 'pubkey' in autocomplete's searchout, add an entry in encpass
  var autocomplete = document.getElementById(searchout).value;
  autocomplete = JSON.parse(autocomplete);
  var autocomplete_counter = 0;

  // Load existing encpass
  var all_recipients = document.getElementById(encpass).value;
  if (all_recipients == '') {
    all_recipients = {};
  } else {
    all_recipients = JSON.parse(all_recipients);
  }

  all_recipients['" env!pubuserid "'] = { 'pubkey': pubkey };
"))

(def srex-enc-functions (env)
  (withs (out "")
	 (if env!autocomp-itemlist
	     (do (= (env!srex-used "jq-autocomplete-tokeninput-encrypted") t)
		 (= (env!srex-used "srex-get") t)
		 (= (env!srex-used "srex-get-wtype") t)
		 (= (env!srex-used "srex-length") t)
		 (= (env!srex-used "srex-set") t)))

	 (if env!enc-itemlist
	     (do (= (env!srex-used "srex-set") t)))

	 (if env!dec-itemlist
	     (do (= (env!srex-used "srex-get") t)))

	 (= out (str out "
"
		     (if (env!srex-used "srex-get-wtype")
			 "function srex_get_wtype(obj, accessor) {
  if (typeof(obj[accessor]) == \"function\") {
      var args = Array.prototype.splice.call(arguments, 2);
      return obj[accessor].apply(obj, args);
  } else
      return obj[accessor]; // accessing a method or indexing a string
}

")
		     (if (env!srex-used "srex-length")
			 "function srex_length(obj) {
  if (typeof(obj) == \"string\")
      return obj.length;
  else
      return Object.keys(obj).length;
}

")
		     (if (env!srex-used "srex-get")
			 "function srex_get (id, istextbox) {
  if (istextbox == true)
      return document.getElementById(id).value;
  else
      return document.getElementById(id).innerText;
}

")
		     (if (env!srex-used "srex-set")
			 "function srex_set (id, istextbox, value) {
  if (istextbox == true)
     document.getElementById(id).value = value;
  else
     document.getElementById(id).innerText = value;
}

")
		     (if (env!srex-used "srex-set-attr")
			 "function srex_set_attr (id, attribute, value) {
  document.getElementById(id).setAttribute(attribute, value);
}

")

		     (if (env!srex-used "srex-onclick")
			 "function srex_onclick (id, fn) {
  var item = document.getElementById(id);
  item.onclick = fn;
}

")

		     (if (env!srex-used "srex-onkeypress")
			 "function srex_onkeypress (id, fn) {
  var item = document.getElementById(id);
  item.onkeypress = fn;
}

")
		     (if (env!srex-used "jq-ajax")
			 "function jq_ajax (ajax) {
  $.ajax( ajax );
}

")
		     (if (env!srex-used "jq-autocomplete")
			 "function jq_autocomplete (field, hash) {
  $( function () {
    $( field ).autocomplete( hash );
  });
}

")

		     (if (env!srex-used "jq-autocomplete-tokeninput")
			 "function jq_autocomplete_tokeninput (field, source, hash) {
  $( field ).tokenInput ( source, hash );
}

")

		     (if (env!srex-used "jq-autocomplete-tokeninput-encrypted")
			 (str (deepcompare) "
function tokeninput_encrypted_getsafe(item, encfields) {
    // All fields will be copied except the fields that were decrypted
    var keyscounter = 0;
    var keystotal = Object.keys(item).length;
    var safe_item = {};
    while (keyscounter < keystotal) {
      var key = Object.keys(item)[keyscounter];
      var value = item[key];
      if (encfields != null && Array.isArray(encfields) && encfields.indexOf(key) > -1) {
         // This is a decrypted field; do not copy it
      } else
        safe_item[key] = value;
      keyscounter++;
    }
   return safe_item;
}

function autocomplete_to_encpass(searchout, encpass) {
" (srex-enc-autocomplete-prologue env) "
  // Add to encpass each entry from autocomplete that contains a 'pubkey'
  while (autocomplete_counter < autocomplete.length) {
     var toadd = autocomplete[autocomplete_counter];
     var enc_contact_pubkey = toadd['pubkey'];
     console.log('enc_contact_pubkey is ' + enc_contact_pubkey);
     console.log('toadd: ');
     console.log(toadd);
     if (enc_contact_pubkey != null && enc_contact_pubkey != false) {
       // must decrypt the contact_pubkey, because it is stored encrypted
       var contact_pubkey = decrypt_nonsearchable('enc_contact_pubkey', enc_contact_pubkey, privkey_openpgp);
       console.log('contact_pubkey');
       console.log(contact_pubkey);
       var userid = toadd['id'];
       console.log('userid: ' + userid);
       if (contact_pubkey == false) {
          // this contact doesn't have a public key
          error_messages.push(\"Can't encrypt for user \" + userid + \" because the public key for them could not be decrypted.\");
          return false;
       } else {
          var contact_pubkey_openpgp = openpgp.key.readArmored(contact_pubkey).keys[0].primaryKey;
          var keyid = contact_pubkey_openpgp.getFingerprint();
          var keyid_short = contact_pubkey_openpgp.getKeyId().toHex();
          var our_keyid_short = pubkey_openpgp.getKeyIds()[0].toHex();
          console.log('keyid: ' + keyid);
          console.log('keyid_short: ' + keyid_short);
          console.log('our_keyid_short: ' + our_keyid_short);
          if ((our_keyid_short == keyid_short) ||
              (keyid != null && userid != null && verifiedkeyids[keyid_short])) {
             // this key was verified; add it
             all_recipients[toadd['id']] = { 'pubkey': contact_pubkey };
             console.log('the key was verified');
          } else {
             // this key failed verification; raise an alert
             console.log('the key failed verification');
             console.log(keyid);
             console.log(userid);
             console.log(keyid_short);
             console.log(verifiedkeyids);
             var readable_keyid = keyid.match(/.{1,4}/g).join(' ').toUpperCase();
             error_messages.push(\"The public key '\" + readable_keyid + \"' for userid '\" + userid + \"' failed verification. \" +
                                 'Odds are SOMEONE IS EAVESDROPPING ON YOU RIGHT NOW.');
             return false;
          }
       }
     } else {
        // this contact doesn't have a public key
        // but the user has been notified, so don't complain
     }
     autocomplete_counter++;
  }

  // Set the new encpass
  all_recipients = JSON.stringify(all_recipients);
  document.getElementById(encpass).value = all_recipients;
}

function validate_recipients (searchout, encpass) {
  console.log('in validate_recipients');
  console.log(searchout);
  console.log(encpass);
" (srex-enc-autocomplete-prologue env) "
  while (autocomplete_counter < autocomplete.length) {
     var toadd = autocomplete[autocomplete_counter];
     var enc_contact_pubkey = toadd['pubkey'];
     console.log('enc_contact_pubkey is ' + enc_contact_pubkey);
     var userid = toadd['id'];
     if (enc_contact_pubkey != null && enc_contact_pubkey != false) {
       var contact_pubkey = decrypt_nonsearchable('enc_contact_pubkey', enc_contact_pubkey, privkey_openpgp);
       if (contact_pubkey == false) {
          // For whatever reason, the public key for this contact could not be decrypted
          error_messages.push(\"Can't encrypt for user \" + userid + \" because the public key for them could not be decrypted.\");
          error_messages.push(\"NONE OF THE DATA WILL BE ENCRYPTED.\");
          all_recipients[toadd['id']] = { 'pubkey': null };
       }
     } else {
       // this contact can't be encrypted
       error_messages.push(\"Can't encrypt for user \" + userid + \" because they don't support encryption.\");
       error_messages.push(\"NONE OF THE DATA WILL BE ENCRYPTED.\");
       all_recipients[toadd['id']] = { 'pubkey': null };
     }
     autocomplete_counter++;
  }

  // Set the new encpass
  all_recipients = JSON.stringify(all_recipients);
  document.getElementById(encpass).value = all_recipients;

  alert_errors();
}

function jq_autocomplete_tokeninput_encrypted (url, encfields, encpass, searchin, searchout, toretrievekeys) {
  var data = {};
  data['onSearch'] = function (inputquery) {
    var ret = 0;
    console.log('----> onSearch');
    ret = encrypt_toretrieve(inputquery, searchsalt, searchpass, searchiv, searchmperc, toretrievekeys);
    console.log('will return: ');
    console.log(ret);
    return ret;
  };
  data['prePopulate'] = JSON.parse(document.getElementById(searchout).value);
  data['onResult'] = function (initial_query, results) {
    console.log('----> onResult');
    console.log(results);
    var counter = 0;
    var l = 0;
    console.log('before l');
    console.log(results);
    if (results != null)
       l = srex_length(results);
    console.log('got len: ' + l);
    
    while (counter < l) { 
      console.log('processing ' + counter);
      var thisres = 0;
      thisres = srex_get_wtype(results, counter);
      console.log('BEFORE thisres is: ');
      console.log(thisres);
      var encfield_counter = 0;

      while (encfield_counter < encfields.length) {
        var contact_enc = 0;
        var contact_dec = 0;

        console.log('processing encfield_counter: ' + encfield_counter + ' for field: ' + encfields[encfield_counter]);

        contact_enc = srex_get_wtype(thisres, encfields[encfield_counter]);
        console.log('got contact');
        console.log(contact_enc);
        contact_dec = decrypt_fromsearch(contact_enc, searchsalt, searchpass, searchiv, searchmperc);
        console.log('got decrypted ' + encfields[encfield_counter] + ': ' + contact_dec);
        thisres[encfields[encfield_counter]] = contact_dec;
        encfield_counter++;
      }

      console.log('AFTER thisres is: ');
      console.log(thisres);
      counter += 1;
    }

    if (l == 0) { // there was no match; use the initial_query
        // if the last character in the initial_query is a space, add this entry
        if (initial_query.substring(initial_query.length-1, initial_query.length) == ' ') {
           var toadd = initial_query.substring(0, initial_query.length-1);
           $( searchin ).tokenInput('add', {id: null, name: toadd});
        }
     }

    console.log('end of onResult');
    return results;
  };
  data['propertyToSearch'] = 'name';
  data['theme'] = 'facebook';
  data['onAdd'] = function (item) {
    console.log('----> onAdd');
    var existing = srex_get(searchout, 1);
    console.log('existing was');
    console.log(existing);
    
    if (existing == '') {
      existing = [];
    } else {
      existing = JSON.parse(existing);
    }

    var safe_item = tokeninput_encrypted_getsafe(item, encfields);
    existing.push( safe_item );
    existing = JSON.stringify(existing);
    srex_set(searchout, 1, existing);
    console.log('onAdd existing is now');
    console.log(existing);
    validate_recipients(searchout, encpass);
  };
  data['onDelete'] = function (item) {
    console.log('----> onDelete');
    var existing = srex_get(searchout, 1);
    console.log('existing was');
    console.log(existing);

    console.log('deleted: ');
    console.log(item);
    if (existing == '') {
      existing = [];
    } else {
      existing = JSON.parse(existing);
    }

    var safe_item = tokeninput_encrypted_getsafe(item, encfields);

    // iterate over existing fields and remove the one that is deleted
    var new_existing = [];
    var existingcounter = 0;
    while (existingcounter < existing.length) {
       var todelete = existing[existingcounter];
       if (deepCompare(safe_item, todelete) == false)
         new_existing.push(todelete);
       existingcounter++;
    }

    new_existing = JSON.stringify(new_existing);
    srex_set(searchout, 1, new_existing);
    console.log('onDelete existing is now');
    console.log(new_existing);
    validate_recipients(searchout, encpass);
  };
  $( searchin ).tokenInput ( url, data );
}

"))

		     (if (env!srex-used "srex-cookies-get")
			 "function srex_cookies_get () {
  var pairs = document.cookie.split(\";\");
  var cookies = {};
  for (var i=0; i<pairs.length; i++){
    var pair = pairs[i].split(\"=\");
    cookies[pair[0]] = unescape(pair[1]);
  }
  return cookies;
}

")
		     (if (env!srex-used "srex-cookie-del")
			 (do (= (env!srex-used "srex-cookie-set") t)
			     "function srex_cookie_del (name) {
  srex_cookie_set(name, \"\", -1);
}

"))
		     (if (env!srex-used "srex-cookie-set")
			 "function srex_cookie_set (name, value, days) {
  if (days) {
        var date = new Date();
        date.setTime(date.getTime()+(days*24*60*60*1000));
        var expires = \"; expires=\"+date.toGMTString();
    }
  else var expires = \"\";
  document.cookie = name+\"=\"+value+expires+\"; path=/\";
}

")
		     (if (env!srex-used "srex-location-set")
			 "function srex_location_set (param, value) {
  location[param] = value;
}

")
		     ))

	 out))

(def srex-processing-contacts (env)
  (withs (ret nil)
	 (if env!formbuttons
	     (each item env!formbuttons
		   (if item!contact-list
		       (= ret t))))
	 ret))

(def srex-submit (env)
  (withs (out "")
	 (if env!formbuttons
	     (each item env!formbuttons
		   (withs (form item!form btn item!btn entersubmits item!entersubmits)
			  (= out (str out "document.getElementById('" btn "').onclick = "
				      (if (srex-processing-contacts env)
					  "verify"
					  "encrypt")
				      "_before_submit_" (srex-valid-js btn) ";
document.getElementById('" btn "').href = \"#\";
"))
			  ;;force all input (not textarea) fields to submit on enter
			  (each elementtype (list "input")
				(= out (str out "// Force every " elementtype " of form " form " to encrypt data if we press Enter
var " elementtype "List = document.forms['" form "'].getElementsByTagName('input');
for (var i=0; i < " elementtype "List.length; i++) {
  " elementtype "List[i].onkeypress = function (e) {
    if(event.keyCode == 13) {
      event.preventDefault();
      " (if entersubmits
	    (str(if (srex-processing-contacts env)
		"verify"
		"encrypt")
  "_before_submit_" (srex-valid-js btn) "();")) "
      return false;
    }
  };
}
")))			  
			  )))
	 out))

(def srex-contact-delete (env contact)
  (withs (out "")
	 (= out (str out "
  {
     var userid = document.getElementById('" contact!id "')." (srex-accesstype contact!istextarea) ";
     var keyid_counter = 0;
     var keyid_total = Object.keys(verifiedkeyids).length;
     while (keyid_counter < keyid_total) {
       var this_keyid = Object.keys(verifiedkeyids)[keyid_counter];
       var this_userid = verifiedkeyids[this_keyid];
       if (userid == this_userid) {
          delete verifiedkeyids[this_keyid];
       }
       keyid_counter++;
     }
     document.getElementById('" contact!deletedid "').value = 'yes';
  }

"))
	 out))

(def srex-accesstype (istextarea)
  (if istextarea "value" "innerText")) ;; textboxes have a value
      
(def srex-dec-core (fromwebserver encprivkey encencpass marktype dec-itemlist srex-aftdec srex-aftdec-all)
  (if dec-itemlist
      (withs (ret "")
	     (= ret (str "
// ----------------------------------- Decrypt data in the browser to view them

" (if (or (is marktype "markdown")
	  (is marktype "markdown-pretty"))
      "var markdownConverter = new Showdown.converter();
"
      (is marktype "to-html")
      "function to_html (theString) {
  return theString;
}
")
   (if (is marktype "markdown-pretty")
       "
function prettypar(theString) {
  var out = '';
  var pars = theString.split(\"\\n\");
  var length_pars = pars.length;
  var counter = 0;

  while (counter < length_pars) {
    var par = pars[counter];
    var length_par = par.length;    
    var sents = par.split('.');
    var length_sents = sents.length;
    var first = null;
    var length_first = 0;

    if (length_sents > 0) {
        first = sents[0];
        length_first = first.length;
    }
    if (first != null) {
       out = out + '<font color=#993333>' + first + '</font>';
       if (length_par > length_first) {
          out = out + '.';
       }
       out = out + par.substring(length_first + 1, length_par);
    }

    counter++;
  }

  return out;
}
")))

	     (each item dec-itemlist
		   (withs (encpass item!encpass
			   id item!id
			   istextarea item!istextarea
			   only-restsent item!only-restsent
			   newcolor item!newcolor
			   aftdec (srex-aftdec id)
			   fromsearch item!fromsearch ;; decrypt search results
			   fromsearchin item!fromsearchin ;; holds encrypted output of search query
			   fromsearchtextarea item!fromsearchtextarea)
			  (if fromsearch
			      ;; decrypt encrypted search results
			      (do ;; if there is no fromsearchin field, fall back to using the id
				  (if (no fromsearchin)
				      (do (= fromsearchin id)
					  (= fromsearchtextarea istextarea)))
				  ;; fixme: if fromsearchin isn't accessed
				  ;; with .value, things break. so fromsearchin
				  ;; must always be a hidden field
				  (= ret (str ret "
// Decrypt the encrypted search results
var enc" id " = document.getElementById('" fromsearchin "')." (srex-accesstype fromsearchtextarea) ";
var dec" id " = '';
if (enc" id " != '') {
  var enc" id "_parsed = JSON.parse( enc" id " );
  dec" id " = decrypt_fromsearch(enc" id "_parsed, searchsalt, searchpass, searchiv, searchmperc);
"
  ;; after decryption finished, set the value
  (srex-dec-core-after id istextarea only-restsent aftdec marktype "  ")
"}
")))
			      ;; else -- decrypt normally
			      (do (= ret (str ret "
// Decrypt " id "
var enc" id " = srex_get('" id "', " (if istextarea "1" "0") ");
var dec" id " = decrypt_nonsearchable('" id "', enc" id ", privkey_openpgp);
"))
				  ;; after decryption finished, set the value
				  (= ret (str ret (srex-dec-core-after id istextarea only-restsent aftdec marktype "")))
				  (= ret (str ret "
" (if newcolor
      (str "document.getElementById('" id "').style.color = '" (hexrep newcolor) "';")) "
"))
))
			  ))

  ;; all decryption finished
 (= ret (str ret (aif srex-aftdec-all (str "// Server-supplied code for srex-after-decrypt-all
" it))))

	     ret)))

;; after decryption finished, set the value
(def srex-dec-core-after (id istextarea only-restsent aftdec marktype prefix)
  (withs (ret "")
	 (= ret (str ret prefix "document.getElementById('" id "')." (srex-accesstype istextarea) " = dec" id ";
"))
	 (if aftdec
	     (= ret (str ret "// Server-supplied code for srex-after-decrypt
" aftdec "
")))
	 (if (is marktype "markdown")
	     (= ret (str ret "
" prefix "document.getElementById('" id "').innerHTML = markdownConverter.makeHtml( document.getElementById('" id "')." (srex-accesstype istextarea) " );"))
	     (and (is marktype "markdown-pretty") only-restsent)
	     (= ret (str ret "
" prefix "document.getElementById('" id "').innerHTML = prettypar( markdownConverter.makeHtml( document.getElementById('" id "')." (srex-accesstype istextarea) " ) );"))
	     (is marktype "to-html")
	     (= ret (str ret "
" prefix "document.getElementById('" id "').innerHTML = to_html ( document.getElementById('" id "')." (srex-accesstype istextarea) " );
")))
	 (if (is id "subject")
	     (= ret (str ret "
" prefix "document.title = dec" id ";")))
	 (= ret (str ret "
"))
	 ret))

(def srex-process-aftdec-all (env)
  (withs (out "")
	 (aif env!srex-aftdec-all (= out it))
	 out))

(def srex-verify-contacts (verify-list)
  (withs (out "")
	 (if verify-list
	     (do (= out (str out "// Verify contacts
"))
		 (each ver verify-list
		       ;; id istextarea showin kind
		       (= out (str out "{
  var verified = 0;
  var userid = document.getElementById('" ver!id "')." (srex-accesstype ver!istextarea) ";
  var our_keyid = pubkey_openpgp.getKeyIds()[0].toHex();
  var keyid_counter = 0;
  var keyid_total = Object.keys(verifiedkeyids).length;
  while (keyid_counter < keyid_total) {
    var this_keyid = Object.keys(verifiedkeyids)[keyid_counter];
    var this_userid = verifiedkeyids[this_keyid];
    if (this_keyid == our_keyid)
       verified = 1;
    if (userid == this_userid)
       verified = 1;
    keyid_counter++;
  }

  var targetdiv = document.getElementById('" ver!showin "');
"))
		       (= out (str out (if (is ver!kind "word")
					   (str "  var div = document.createElement('div');
  div.id = '" ver!showin "_after_verification';
  div.style.textAlign = 'left';
  div.style.font = '13px Verdana';
  div.style.color = 'white';
  if (verified) {
     div.innerHTML = 'VERIFIED';
     div.style.width = '62px';
     div.style.padding = '7px';
     div.style.backgroundColor = 'green';
  } else {
     div.innerHTML = 'NOT verified';
     div.style.width = '85px';
     div.style.padding = '7px';
     div.style.backgroundColor = 'red';
  }
  targetdiv.appendChild(div);
")
					   (is ver!kind "color")
					   (str "if (verified) {
  targetdiv.backgroundColor = 'green';
} else {
  targetdiv.backgroundColor = 'red';
}"))))
		       (= out (str out "
}"
))
		       )
	       ))
	 out))

;; a field holds decrypted data if (a) it was defined in the
;; decryption list, or (b) is a variable that uses a decrypted data
(def srex-is-decrypted-data (env field)
  (withs (ret nil usedfield (srex-rm-dquotes field))
	 ;;(prn "in srex-is-decrypted-data: " field)
	 (each item env!dec-itemlist
	       (withs (id item!id)
		      (if (is id usedfield)
			  (= ret t))))
	 (if (env!vars-using-dec usedfield)
	     (= ret t))
	 ;;(prn "out of srex-is-decrypted-data: " ret)
	 ret))

;; a field holds data that will be encrypted if (a) it was defined in
;; the encryption list, or (b) is a variable that uses data that will
;; be encrypted
(def srex-is-encrypted-data (env field (o b-too t))
  (withs (ret nil usedfield (srex-rm-dquotes field))
	 ;;(prn "in srex-is-encrypted-data: " field)
	 (each item env!enc-itemlist
	       (withs (id item!id)
		      (if (is id usedfield)
			  (= ret t))))
	 (if b-too
	     (if (env!vars-using-enc usedfield)
		 (= ret t)))
	 ;;(prn "out of srex-is-encrypted-data: " ret)
	 ret))

(def srex-rm-dquotes (f)
  (subst "" "\"" f))

(def srex-is-form-data (env field)
  ;;(prn "in srex-is-form-data: " field " in " env!form-inputs)
  ;; The field passed in could be in double quotes
  (withs (ret nil usedfield (subst "" "\"" field))
	 (each item env!form-inputs
	       (if (is item usedfield)
		   (= ret t)))
	 ;;(prn "out of srex-is-form-data: " (str ret))
	 ret))

;; ----- code verification start ---------------------------------------------

;; Protect from two types of attempts to steal decrypted data:
;; (1) setting a field that won't be encrypted with the value of a
;;     field that was decrypted
;;
;; (2) setting a field that won't be encrypted with the value of a
;;     temporary variable, while this variable was first set to the
;;     value of a field that was decrypted. Also check this
;;     recursively across several temporary variables

(def srex-vars-newcount(env)
  (+ (len env!vars-using-dec)
     (len env!fns-using-dec)
     (len env!vars-using-enc)
     (len env!fns-using-enc)))

(def srex-codewalk (env)
  (withs (out "")
	 (if env!code
	     (do
	       ;; Ideally, we want to first perform control-flow
	       ;; analysis to identify which variables end up using
	       ;; decrypted data, and then walk over all code and
	       ;; produce safe Javascript for it. In practice, we
	       ;; cheat: we continually try to generate Javascript,
	       ;; until there is no change in the number of new
	       ;; variables discovered that use decrypted data.

	       ;; This should be bad at runtime and good at makertime.
	       (withs (lastcount -1)
		      (while (no (is lastcount (srex-vars-newcount env)))
			     (= lastcount (srex-vars-newcount env))
			     (= env!uses-dec nil)
			     (= env!uses-enc nil)
			     (= env!srex-aftdec (table)) ;; because they hold lists
			     (= env!srex-befenc (table))
			     (= out (verify-go env))
			     ))
	       ))
	 (prn "// ======= form-inputs: " env!form-inputs)
	 (prn "// ======= variables using decrypted data: " env!vars-using-dec)
	 (maptable (fn (k v)
		       (prn "// v: " k)) env!vars-using-dec)
	 (prn "// ======= functions using decrypted data: " env!fns-using-dec)
	 (maptable (fn (k v)
		       (prn "// f: " k)) env!fns-using-dec)
	 (prn "// ======= variables using encrypted data: " env!vars-using-enc)
	 (maptable (fn (k v)
		       (prn "// v: " k)) env!vars-using-enc)
	 (prn "// ======= functions using encrypted data: " env!fns-using-enc)
	 (maptable (fn (k v)
		       (prn "// f: " k)) env!fns-using-enc)
	 (prn "// ======= srex functions used: " env!srex-used)
	 (maptable (fn (k v)
		       (prn "// f: " k)) env!srex-used)
	 out))

(= srex-hack-attempt-msg "THERE WAS AN ATTEMPT TO HACK YOU, but it was NOT successful.")

(mac outpend s
     `(do (repeat (* depth 2) (= out (str out " ")))
	  (= out (str out ,@s))))

(= srex-uniq-counter* 0)
(def srex-uniq () ; This gives us repeatable testcases, and valid Javascript var names
  (++ srex-uniq-counter*)
  (str "gs" srex-uniq-counter*))

(def verify-go (env)
  (withs (out nil depth 0 newparams nil	  
	  ;; (read) only reads the first element, so read everything
	  ;; in one big do
	  data (read (str "(do " env!code ")"))) ;; what separates Lisp: treating code as data
	 ;;(prn "in verify-go: " data)
	 (if (is (type data) 'cons)
	     (if (is (str (data 0)) "do")
		 (= out (str out (verify-do env data newparams depth)))
;; 		 (is (str (data 0)) "withs")
;; 		 (= out (str out (verify-withs env data newparams depth)))
		 ))
	 out))

(def verify-do (env data oldparams depth)
  ;;(prn "in verify-do: " data)
  (withs (out "" dobody nil counter 0)
	 (each d data
	       (if (> counter 0)
		   (push d dobody))
	       (++ counter))
	 (each d (rev dobody)
	       (outpend (verify-exp env d oldparams depth) ";" #\newline))
	 ;;(prn "out of verify-do: " out)
	 out))

(def srex-before-encrypt-hook (env data depth)
  (withs (out "" var (data 1) stmt nil)
	 (= stmt (verify-exp env (data 2) nil 0))
	 ;; there could be many hooks for a variable
	 (push (str stmt ";") (env!srex-befenc var))
	 ;;(prn env!srex-befenc)
	 out))

(def srex-before-encrypt-all-hook (env data depth)
  (withs (out "" stmt nil)
	 (= stmt (verify-exp env (data 1) nil 0))
	 (= env!srex-befenc-all (str stmt ";"))
	 ;;(prn env!srex-befenc-all)
	 out))

(def srex-after-decrypt-hook (env data depth)
  (withs (out "" var (data 1) stmt nil)
	 (= stmt (verify-exp env (data 2) nil 0))
	 ;; there could be many hooks for a variable
	 (push (str stmt ";") (env!srex-aftdec var))
	 ;;(prn env!srex-aftdec)
	 out))

(def srex-after-decrypt-all-hook (env data depth)
  (withs (out "" stmt nil)
	 (= stmt (verify-exp env (data 1) nil 0))
	 (= env!srex-aftdec-all (str stmt ";"))
	 ;;(prn "SREX-AFTDEC-ALL: " env!srex-aftdec-all)
	 out))

;; Javascript function and variable names can't have dashes
(def srex-valid-js (v)
  (subst "_" "-" v))

(def verify-def-body (env data oldparams depth fnname)
  ;;(prn "in verify-def-body " data)
  (withs (counter 0 out "" newparams nil)
	 (each i data
	       (if (is counter 0)
		   (= out (str out "function " (if (no (is fnname "")) fnname)))
		   (is counter 1)
		   (withs (paramcnt 0) ;; we will process function parameters
			  (= out (str out "("))
			  (each param i
				(if (> paramcnt 0)
				    (= out (str out ", ")))
				(= out (str out param))
				(push (str param) newparams)
				(++ paramcnt))
			  (= out (str out ") {")))
		   ;; what we will process isn't function parameters
		   (= out (str out #\newline (verify-exp env i (+ oldparams newparams) (+ depth 1)) ";")))
	       (++ counter))
	 (= out (str out #\newline))
	 (outpend "}")
	 ;;(prn "out from verify-def-body")
	 out))

;; no global variables are supported for now (no oldparams passed in verify-def)
(def verify-def (env data oldparams depth)
  ;;(prn "in verify-def: " (data 1) " with uses-dec: " env!uses-dec)
  ;;(prn "in verify-def: " data)
  (withs (out "" fnname nil dataleft nil newfnname nil
	  old-uses-dec nil old-uses-enc nil)
	 (= dataleft data)
	 (pop dataleft) ; pop def
	 (= fnname (car dataleft)) ; pop fnname
	 (if (no (env!fns fnname))
	     (do (= newfnname (str fnname "_" (srex-uniq)))
		 (= newfnname (srex-valid-js newfnname))
		 (= (env!fns fnname) newfnname))
	     (= newfnname (env!fns fnname)))
	 (= old-uses-dec env!uses-dec) ; save
	 (= old-uses-enc env!uses-enc) ; save
	 (= env!uses-dec nil)
	 (= env!uses-enc nil)
	 (= out (str out (verify-def-body env dataleft oldparams depth newfnname)))
	 (if env!uses-dec
	     (do ;;(prn "added " fnname)
		 (= (env!fns-using-dec fnname) t)))
	 (if env!uses-enc
	     (do ;;(prn "added " fnname)
		 (= (env!fns-using-enc fnname) t)))
	 (= env!uses-enc old-uses-enc) ; restore
	 (= env!uses-dec old-uses-dec) ; restore
	 ;;(prn env!fns)
	 ;;(prn "out of verify-def: " out)
	 out))

(def verify-withs (env data oldparams depth)
  ;;(prn "in verify-withs " data)
  (withs (out "" foundwiths nil foundwithsparams nil invar t lastvar nil
	  newparams oldparams pcnt 0)
	 (each i data
;;  	       (prn "type: " (type i))
;;  	       (prn "data: " (str i))
	       (if (no foundwiths)
		   (= foundwiths t)
		   (if (is (type i) 'cons)
		       (if (no foundwithsparams) ;; process the with params
			   (do (= foundwithsparams t)
			       (each p i
				     (++ pcnt)
				     ;;(prn "p is: " p)
				     (if invar ;; processing the 1st, 3rd, 5th, ... element
					 (withs (newvarname (verify-sym env (str p)
									(+ oldparams newparams (list (str p)))))
						(if (> pcnt 1)
						    (= out (str out #\newline)))
						(outpend (str "var " newvarname " = "))
						(= lastvar newvarname)
						(= invar nil)
						;;(push (str newvarname) newparams)
						(push (str p) newparams))
					 (no invar) ;; processing the even element
					 (do (= out (str out (verify-exp env p (+ oldparams newparams) 0) ";"))
					     (if env!uses-dec
						 (= (env!vars-using-dec lastvar) t))
					     (if env!uses-enc
						 (do ;;(prn (str "WILL ADD to vars-usinc-enc: " lastvar))
						     (= (env!vars-using-enc lastvar) t)))
					     (= invar t))
					 ))
			       ;;(outpend "" #\newline)
			       )
			   foundwithsparams ;; process the body of the with
			   (= out (str out #\newline (verify-exp env i (+ oldparams newparams) depth) ";"))
			   )
		       (atom i) ;; not a cons
		       (do (= out (str out #\newline))
			   (outpend (str (verify-exp env i (+ oldparams newparams) depth) ";")))
		       ))
	       )
	 ;;(outpend "" #\newline)
	 ;;(prn "out of verify-withs")
	 out))

(def verify-while (env data oldparams depth)
  (withs (out "" counter 0)
	 (outpend #\newline)
	 (outpend "")
	 (each i data
	       (if (is counter 0)
		   (= out (str out "while ("))
		   (is counter 1)
		   (do (= out (str out (verify-exp env i oldparams depth)))
		       (= out (str out ") { " #\newline)))
		   (= out (str out (verify-exp env i oldparams (+ depth 1)) ";" #\newline)))
	       (++ counter))
	 (outpend "}")
	 out))

; verify an expression using a blank env!uses-dec, and do not
; propagate the value of env!uses-dec further up when done, because
; it's not an assignment
(mac w/udec (env . body)
     (w/uniq oud
	     `(withs (,oud (,env 'uses-dec))
		     (= (,env 'uses-dec) nil)
		     ,@body
		     (= (,env 'uses-dec) ,oud))))
(def verify-exp (env data oldparams depth)
  (withs (out "")
	 ;;(prn "in verify-exp: " data)
	 (if (is (type data) 'cons)
	     (do ;;(prn "in verify-exp: (data 0) " (data 0))
		 (if (is (str (data 0)) "srex-before-encrypt")
		     (= out (str out (srex-before-encrypt-hook env data depth)))
		     (is (str (data 0)) "srex-before-encrypt-all")
		     (= out (str out (srex-before-encrypt-all-hook env data depth)))
		     (is (str (data 0)) "srex-after-decrypt")
		     (= out (str out (srex-after-decrypt-hook env data depth)))
		     (is (str (data 0)) "srex-after-decrypt-all")
		     (= out (str out (srex-after-decrypt-all-hook env data depth)))
		     (or (is (str (data 0)) "eval")
			 (is (str (data 0)) "setTimeout")
			 (is (str (data 0)) "setInterval"))
		     (= env!srex-hack-attempt t)
		     (is (str (data 0)) "new")
		     (= out (str out (verify-new env data oldparams depth)))
		     (is (str (data 0)) "return")
		     (= out (str out (verify-return env data oldparams depth)))
		     (is (str (data 0)) "def")
		     (= out (str out (verify-def env data oldparams depth)))
		     (is (str (data 0)) "withs")
		     (= out (str out (verify-withs env data oldparams depth)))
		     (is (str (data 0)) "do")
		     (= out (str out (verify-do env data oldparams depth)))
		     (is (str (data 0)) "while")
		     (= out (str out (verify-while env data oldparams depth)))
		     (is (str (data 0)) "each")
		     (= out (str out (verify-each env data oldparams depth)))
		     (is (str (data 0)) "maptable")
		     (= out (str out (verify-maptable env data oldparams depth)))
		     (is (str (data 0)) "=")
		     (= out (str out (verify-assign env data oldparams depth)))
		     (is (str (data 0)) "+")
		     (= out (str out (verify-addsub env data oldparams depth "+")))
		     (is (str (data 0)) "-")
		     (= out (str out (verify-addsub env data oldparams depth "-")))
		     (is (str (data 0)) "++")
		     (outpend (data 1) " += 1")
		     (is (str (data 0)) "--")
		     (outpend (data 1) " -= 1")
		     (is (str (data 0)) "*")
		     (= out (str out (verify-addsub env data oldparams depth "*")))
		     (is (str (data 0)) "/")
		     (= out (str out (verify-addsub env data oldparams depth "/")))
		     (is (str (data 0)) "if")
		     (= out (str out (verify-if env data oldparams depth)))
		     (or (is (str (data 0)) "and")
			 (is (str (data 0)) "or"))
		     (= out (str out (verify-andor env data oldparams depth)))
		     (is (str (data 0)) "no")
		     (= out (str out "(" (verify-exp env (data 1) oldparams depth) ") == false"))
		     (is (str (data 0)) "is")
		     (do (= out (str out (verify-exp env (data 1) oldparams depth)))
			 (= out (str out " == "))
			 (= out (str out (verify-exp env (data 2) oldparams depth))))
		     (or (is (str (data 0)) "<")
			 (is (str (data 0)) "<=")
			 (is (str (data 0)) ">")
			 (is (str (data 0)) ">="))
		     (= out (str out (data 1) " " (data 0) " " (data 2)))
		     (or (is (str (data 0)) "string")
			 (is (str (data 0)) "str"))
		     (= out (str out (verify-str env data oldparams depth)))
		     (is (str (data 0)) "len")
		     (do (= (env!srex-used "srex-length") t)
			 (= out (str out "srex_length(" (data 1) ")")))
		     (is (str (data 0)) "table") ;; hashtable in Arc
		     (= out (str out "{}"))
		     (is (str (data 0)) "list")  ;; list/array in Arc
		     (outpend (verify-list env data oldparams 0))
		     (is (str (data 0)) "obj")   ;; obj in Arc
		     (outpend (verify-obj env data oldparams 0))
		     (is (str (data 0)) "push")  ;; add to a list in Arc
		     (= out (str out (verify-push env data oldparams depth)))
		     (is (str (data 0)) "pop")   ;; remove from a list in Arc
		     (= out (str out (verify-pop env data oldparams depth)))
		     (is (str (data 0)) "tojson")
		     (do (= out (str out "JSON.stringify("))
			 (= out (str out (verify-exp env (data 1) oldparams depth)))
			 (= out (str out ")")))
		     (is (str (data 0)) "fromjson")
		     (do (= out (str out "JSON.parse("))
			 (= out (str out (verify-exp env (data 1) oldparams depth)))
			 (= out (str out ")")))
 		     (find (str (data 0)) oldparams)
 		     (outpend (verify-var env data oldparams 0)) ;; calling a property of a variable
		     (is (str (data 0)) "fn") ;; a function object
		     (= out (str out (verify-def-body env data oldparams (+ depth 1) "")))
		     ;; it's an ordinary function call
		     (outpend (verify-apply env data oldparams 0))
		     ))
	     (is (type data) 'char)
	     (if (is (coerce data 'int) 10) ;; #\newline
		 (= out (str "'\\n'"))
		 (is (coerce data 'int) 13) ;; #\return
		 (= out (str "'\\r'"))
		 (= out (str "'" (str data) "'")))
	     (is (type data) 'int)
	     (= out (str out data))
	     (is (type data) 'sym)
	     (= out (str out (verify-sym env data oldparams)))
	     (is (type data) 'string)
	     (do (if (srex-is-decrypted-data env (str data))
		     (= env!uses-dec t))
		 (= out (str out (str "\"" (str data) "\""))))
	     )
	 ;;(prn "out of verify-exp with: " out)
	 out))

(def verify-addsub (env data oldparams depth sign)
  (withs (out "" counter 0)
	 (= out (str out (verify-exp env (data 1) oldparams depth)))
	 (each d data
	       (if (>= counter 2)
		   (do (= out (str out " " sign " "))
		       (= out (str out (verify-exp env d oldparams depth)))))
	       (++ counter))
	 out))

(def verify-list (env data oldparams depth)
  ;;(prn "in verify-list: " data)
  (withs (out "" counter 0)
	 (outpend "[")
	 (each d data
	       (if (> counter 1)
		   (= out (str out ",")))
	       (if (> counter 0)
		   (= out (str out (verify-exp env d oldparams depth))))
	       (++ counter))
	 (= out (str out "]"))
	 ;;(prn "out of verify-list with: " out)
	 out))

(def verify-obj (env data oldparams depth)
  (withs (out "" counter 0)
	 (outpend "{")
	 (each d data
	       (if (> counter 1)
		   (if (is (mod counter 2) 1)
		       (= out (str out ", "))
		       (= out (str out ":"))))
	       (if (> counter 0)
		   (= out (str out (verify-exp env d oldparams depth))))
	       (++ counter))
	 (= out (str out "}"))
	 out))

(def verify-push (env data oldparams depth)
  ;;(prn "in verify-push: " data)
  (withs (out "" counter 0)
	 (if (is (len data) 3)
	     (outpend (str (data 2) ".push(" (verify-exp env (data 1) oldparams depth) ")"))
	     (outpend "null"))
	 ;;(prn "out of verify-push with: " out)
	 out))

(def verify-pop (env data oldparams depth)
  ;;(prn "in verify-pop: " data)
  (withs (out "" counter 0)
	 (if (is (len data) 2)
	     (= out (str out (data 1) ".splice(0,1)"))
	     (= out (str out "null")))
	 ;;(prn "out of verify-pop with: " out)
	 out))

(def verify-var (env data oldparams depth)
  ;;(prn "in verify-var: " data)
  (withs (out "" counter 0 varparams nil)
	 (if (> (len data) 1)
	     (do (each d data
		       ;;(prn "d: '" d "'")
		       (if (>= counter 2)
			   (push d varparams))
		       (++ counter))
		 ;;(= out (str out (data 0) "." (verify-exp (rev varparams) oldparams depth)))
		 (if (is (type (data 1)) 'string) ;; this is a hardcoded string
		     (= out (str out "srex_get_wtype(" (data 0) ", \"" (srex-clean-item (str (data 1))) "\""))
		     (find (str (data 1)) oldparams)
		     ;; this is a variable
		     (= out (str out "srex_get_wtype(" (data 0) ", " (data 1)))
		     ;; this could be a number
		     (= out (str out "srex_get_wtype(" (data 0) ", " (data 1))))
		 ;; either way, srex-get-wtype is used here
		 (= (env!srex-used "srex-get-wtype") t)
		 ;;(prn "will process varparms: " varparams)
		 (withs (varcounter 0)
			;; if we are calling something like: (srex-get-wtype console log "test")
			;; we should allow access to the second var ("log")
			;; but not necessarily the first ("console"); except
			;; the case of specifically the first variable "console".
			(each p (rev varparams)
			      (withs (paramstouse oldparams)
				     (if (> varcounter 0)
					 (push p paramstouse))
				     (= out (str out ", " (verify-exp env p paramstouse depth))))
			      (++ varcounter)))
		 (= out (str out ")"))
		 ;;(= out (str out (data 0) "[" (data 1) "]"))
		 )
	     (= out (str out data)))
	 ;;(prn "out of verify-var: " out)
	 out))

(def verify-new (env data oldparams depth)
  ;;(prn "in verify-new: " data)
  (withs (out "" name (data 1))
	 (if (is name "Function") ; not allowed by Chrome extension
	     (= env!srex-hack-attempt t)
	     (withs (counter 0 fnbody nil) ;; unpack into a function call
		    (each d data
			  (if (> counter 0)
			      (push d fnbody))
			  (++ counter))
		    (= out (str out "new " (verify-apply env (rev fnbody) oldparams 0)))))
	 ;;(prn "out from verify-new: " out)
	 out))

(def verify-return (env data oldparams depth)
  (withs (out "" counter 0 retbody nil)
	 (each d data
	       (if (> counter 0)
		   (push d retbody))
	       (++ counter))
	 (if (is (len retbody) 1)
	     (outpend "return " (verify-exp env (retbody 0) oldparams 0)) ;; to avoid interpreting as a function call
	     (outpend "return " (verify-exp env (rev retbody) oldparams 0)))
	 out))

(def verify-sym (env data oldparams)
  ;;(prn "in verify-sym with: " data " of type " (type data))
  (withs (out "")
	 (if (is (str data) "t")
	     (= out "1")
	     (and (no data)
		  (is (str data) "")) ; nil
	     ;;(= out "0")
	     (= out "false") ;; equal to 0, plus doesn't break when setting browser navigation to false
	     ;; Javascript variable names can't have dashes
	     (= out (srex-valid-js (str data))))
	 (if (srex-is-decrypted-data env out)
	     (= env!uses-dec t))
;; 	 (prn "verify-sym - oldparams: " oldparams)
;; 	 (prn "verify-sym - data: " data)
;; 	 (prn "verify-sym - not a declared var: " (no (find (str data) oldparams)))
;; 	 (prn "verify-sym - not t: " (no (and (is (type data) 'sym) (is (str data) "t"))))
	 (if (and data ; not nil
		  (no (find (str data) oldparams)) ; not a declared var
		  (no (and (is (type data) 'sym) (is (str data) "t")))) ; not t
	     (do ;; Don't allow direct access to the DOM or global
	         ;; framework variables
	         ;; (prn "HACK ATTEMPT --- sym: " (str data))
		 (= env!srex-hack-attempt t)))
	 ;;(prn "out of verify-sym: " out)
	 out))

(def verify-assign (env data oldparams depth)
  ;;(prn "in verify-assign")
  (withs (out "" lhs nil rhs nil
	  old-uses-dec nil lhs-uses-dec nil rhs-uses-dec nil
	  old-uses-enc nil lhs-uses-enc nil rhs-uses-enc nil)
	 ;; save any previous information about accessing decrypted data
	 (= old-uses-dec env!uses-dec)
	 (= env!uses-dec nil)
	 (= old-uses-enc env!uses-enc)
	 (= env!uses-enc nil)

	 ;; check the lhs
	 (if (is (type (data 1)) 'cons) ;; assigning a hashtable
	     (= lhs (str ((data 1) 0) "[\"" ((data 1) 1) "\"]"))
	     (= lhs (verify-sym env (data 1) oldparams)))
	 (= lhs-uses-dec env!uses-dec)
	 (= env!uses-dec nil)
	 (= lhs-uses-enc env!uses-enc)
	 (= env!uses-enc nil)

	 ;; check the rhs
	 (= rhs (verify-exp env (data 2) oldparams 0))
	 (= rhs-uses-dec env!uses-dec)
	 (= rhs-uses-enc env!uses-enc)
	 (if (and (no (is (type (data 2)) "cons"))
		  (env!vars-using-dec (data 2)))
	     (do ;;(prn "detected that " (data 2) " was using decrypted data")
		 (= rhs-uses-dec t)))

	 (if (and (no (is (type (data 2)) "cons"))
		  (env!vars-using-enc (data 2)))
	     (do ;;(prn "detected that " (data 2) " was using encrypted data")
		 (= rhs-uses-enc t)))
	     
	 (= env!uses-dec nil)
	 (= env!uses-enc nil)

	 ;; check if the rhs used a decrypted field
	 (if rhs-uses-dec
	     ;; This is conservative, because variables with the same
	     ;; name in other functions will be incorrectly determined
	     ;; to also be using decrypted data. But it's safe. The
	     ;; proper thing would be more rigorous control-flow
	     ;; analysis; we settle for an inappropriate hack.
	     (= (env!vars-using-dec lhs) t))

	 ;; check if the rhs used an encrypted field
	 (if rhs-uses-enc
	     (do ;; for card["exp_year"], mark both card and exp_year.
	       ;; this is conservative, but easier to reason about in
	       ;; srex-is-encrypted-data (b). eg. for testcase post.arc:
	       ;; card is used in data, used in ajax, used by jq-ajax, thus
	       ;; exp_year should be shown in read
	       ;; fixme: why does this lead to duplicate vars in vars-using-enc?
	       ;;        ...while using (str ((data 1) 0)) doesn't -- but doesn't
	       ;;        produce the desired result of detecting the ajax variable?
	       (if (is (type (data 1)) 'cons) ;; assigning a hashtable
		   (do ;;(prn (str "the lhs hash USED an encrypted field: " ((data 1) 0)))
		       ;;(prn (str "the lhs hash USED an encrypted field: " ((data 1) 1)))
		       (= (env!vars-using-enc ((data 1) 0)) t)
		       (= (env!vars-using-enc ((data 1) 1)) t))
		   (do ;; assigning a value
		       ;;(prn (str "the lhs value USED an encrypted field: " (data 1)))
		       (= (env!vars-using-enc (data 1)) t)
		     ))
	       ))

	 ;; better to not allow arbitrary method access, which will
	 ;; then make the following check unnecessary.
;; 	 (if (and rhs-uses-dec
;; 		  (no (srex-is-encrypted-data env lhs))
;; 		  (no (find lhs oldparams)))
;; 	     ;; tried to set a field that won't be encrypted (fixme:
;; 	     ;; should verify it's a form variable though) and isn't a
;; 	     ;; local variable to the value of a field that was
;; 	     ;; decrypted.
;; 	     (env!srex-hack-attempt t))

	 (if (no env!srex-hack-attempt)
	     (withs (rhstouse rhs)
		    ;; if rhs was a single token that was a function, use the new function name
		    (if (is (type (data 2)) 'sym)
			(aif (env!fns (data 2))
			     (= rhstouse it)))			
		    (outpend (str lhs " = " rhstouse)))
	     (do ;;(prn "HACK ATTEMPT --- 7 on " lhs " = " rhs)
	         (= env!srex-hack-attempt t)
		 ;; clear out env!uses-dec to catch more such issues in other expressions
		 ;; (= env!uses-dec nil)
		 ))

	 ;; restore previous information about the right-hand side
	 ;; accessing decrypted data. You'd think this isn't
	 ;; necessary, but it might be possible to make a valid
	 ;; assignment within an assignment that bypasses our
	 ;; checks. Better safe than sorry.
	 ;; (= env!old-uses-dec (or rhs-uses-dec env!uses-dec))
	 ;;(prn "out of verify-assign with: " out)
	 out))

(def verify-str (env data oldparams depth)
  (withs (out "" counter 0)
	 (each i data
	       (if (is counter 0) ;; str
		   nil
		   (>= counter 2)
		   (= out (str out " + " (verify-exp env i oldparams 0)))
		   (is counter 1)
		   (= out (str out (verify-exp env i oldparams depth))))
	       (++ counter))
	 out))

(def verify-andor (env data oldparams depth)
  ;;(prn " verify-andor: " data)
  (withs (out "" counter 0 op nil)
	 (if (is (str (data 0)) "and")
	     (= op " && ")
	     (is (str (data 0)) "or")
	     (= op " || "))
	 (each i data
	       ;;(prn " processing " i " with op " op " at counter " counter)
	       (do (if (> counter 1)
		       (do (= out (str out op))
			   (= out (str out (verify-exp env i oldparams depth))))
		       (> counter 0) ;; skip the and/or
		       (= out (str out (verify-exp env i oldparams depth)))
		       ;;(prn " did nothing")
		       ))
	       ;;(prn "  will go to next iteration")
	       (++ counter))
	 ;;(prn "out from verify-andor")
	 out))

(def tounderscores (s)
  (withs (out "" counter 0 l (len s))
	 (while (< counter l)
		(withs (thischar (str (s counter)))
		       (if (is (str thischar) "-")
			   (= thischar "_"))
		       (= out (str out thischar)))
		(++ counter))
	 out))

(def srex-is-btn (env i)
  (withs (ret nil)
	 (each item env!formbuttons
	       (withs (btn item!btn)
		      (if (is btn i)
			  (= ret t))))
	 ret))

(= srex-fnmaps* (table))
(each f (list "srex-set-attr" "srex-get-wtype"
	      "jq-autocomplete" "jq-autocomplete-tokeninput" "jq-ajax"
	      "srex-cookie-del" "srex-cookies-get"
	      "srex-location-set" "srex-location-get"
	      "srex-onclick" "srex-onkeypress")
      (= (srex-fnmaps* f) t))
(def verify-apply (env data oldparams depth)
  ;;(prn "in verify-apply: " data)
  (withs (out "" counter 0 name nil isset nil isget nil isgetwt nil
          issendout nil isonclick nil
	  setparam nil setvalue nil getparam nil
	  rhs-uses-dec nil rhs-uses-enc nil)
	 (each i data
	       ;;(prn (str "processing i: " i))
	       (if (> counter 1)
		   (= out (str out ", ")))
	       (if (is counter 0)
		   (do (if (env!fns i)
			   ;; this is a call to a user-supplied function
			   (do ;; (prn "will call user-supplied function " i)
			       (= name (env!fns i))
			       (= out (str out name "(")))
			   ;; this is a call to a different function
			   (is (str i) "srex-get")
			   (do (= isget t)
			       (= (env!srex-used "srex-get") t)
			       (= out (str out "srex_get(")))
			   (is (str i) "srex-set")
			   (do (= isset t)
			       (= (env!srex-used "srex-set") t)
			       (= out (str out "srex_set(")))
			   (is (str i) "srex-onclick")
			   (do (= isonclick t)
			       (= (env!srex-used "srex-onclick") t)
			       (= out (str out "srex_onclick(")))
			   (is (str i) "srex-onkeypress")
			   (do (= (env!srex-used "srex-onkeypress") t)
			       (= out (str out "srex_onkeypress(")))
			   (srex-fnmaps* (str i))
			   (do (if (is (str i) "srex-get-wtype")
				   (= isgetwt t))
			       (= (env!srex-used (str i)) t)
			       (= out (str (tounderscores (str i)) "("))
			       (if (is (str i) "jq-ajax")
				   (do (= issendout t)
				       (= (env!srex-used "jq-ajax") t)
				       ;;(prn "found AJAX")
				       ;;(prn env!using-enc)
				       ))
			       )
			   (do ;; (prn "other function")
			       (= out (str out (tounderscores (str i)) "("))
			       ))
		       )
		   (> counter 0)
		   (withs (param nil paramstouse oldparams)
			  (= rhs-uses-dec env!uses-dec) ;save
			  (= rhs-uses-enc env!uses-enc) ;save
			  (= env!uses-dec nil)
			  (= env!uses-enc nil)
			  (if (and isgetwt (or (> counter 1)
					       (and (is counter 1) (is (str i) "console"))))
			      ;; if we are calling something like: (srex-get-wtype console log "test")
			      ;; allow the first variable only if it
			      ;; is console; allow the second variable
			      ;; at all times
			      (do ;; (prn "pushed " i " to paramstouse")
				  (push (str i) paramstouse)))
			  ;;(prn "i: " i " counter " counter " isgetwt " isgetwt)
			  (= param (verify-exp env i paramstouse 0))
			  (if (and issendout (is counter 1))
			      (do ;;(prn "sending out: " i)
				  (if (env!vars-using-enc i)
				      ;; the variable passed to jq-ajax contains data
				      ;; that should be encrypted but are instead sent
				      ;; out of the browser
				      (= (env!fns-using-enc "jq-ajax") t))
				))
			  (if (and isonclick (is counter 1))
			      (do ;; (prn "isonclick: " i)
				  (if (or (srex-is-btn env i)
					  (is i "sign")
					  (is i "publish")
					  (is i "showauthors"))
				      ;; the element that has an onclick handler set
				      ;; is a button used for encryption. This is a hack attempt.
				      (do (= env!srex-hack-attempt t)))
				))
			  (if (and isset (is counter 1))
			      (= setparam param))
			  (if (and isset (is counter 3))
			      (= setvalue i))
			  (if (and isset (is counter 3)
				   (is (type i) 'cons)
				   (env!fns-using-dec (i 0)))
			      ;; if this is the parameter that was
			      ;; set, and it was a function call
			      ;; using decrypted data
			      (= env!uses-dec t))
			  (if (and isset (is counter 3)
				   (is (type i) 'cons)
				   (env!fns-using-enc (i 0)))
			      ;; if this is the parameter that was
			      ;; set, and it was a function call
			      ;; using decrypted data
			      (= env!uses-enc t))
			  (if (and isget (is counter 1))
			      (do ;; if this is an srex-get, and its first argument describing the
				  ;; input field isn't a string, conservatively assume we are being
				  ;; hacked. We could implement a proper eval here, but why bother
				  ;; helping hackers?
				  (if (no (is (type i) 'string))
				      (do
					;;(prn "HACK ATTEMPT --- 1")
					(= env!srex-hack-attempt t)))
				  (if (srex-is-encrypted-data env i)
				      (do ;;(prn (str "isget for: " i))
					  (= env!uses-enc t)))
				  (= getparam param)))
			  (= env!uses-dec (or rhs-uses-dec env!uses-dec)) ;restore
			  (= env!uses-enc (or rhs-uses-enc env!uses-enc)) ;restore
			  (if (no env!srex-hack-attempt)
			      (= out (str out param))
			      (= out "null"))
			  )
		   )
	       (++ counter))
	 (= out (str out ")"))

	 ;; check if what srex-set will send to the server is not encrypted data
	 (if (and isset
		  env!uses-dec
		  ;; note: don't also check for env!uses-enc. Need
		  ;; special code for showing in red fields that will
		  ;; have encrypted data sent away from the browser.
		  (no (srex-is-encrypted-data env setparam))
		  (srex-is-form-data env setparam)
		  (no (is (type setvalue) 'string)))
	     ;; setting the value of a field that won't be encrypted, using decrypted data
	     (do ;; (prn "HACK ATTEMPT --- 2 with data: " data)
;; 		 (prn "isset: " isset)
;; 		 (prn "env!uses-dec: " env!uses-dec)
;; 		 (prn "not encrypted data: " (no (srex-is-encrypted-data env setparam)))
;; 		 (prn "is-form-data: " (srex-is-form-data env setparam))
;; 		 (prn "not string data: " (no (is (type setvalue) 'string)))
		 (= env!srex-hack-attempt t)))

	 (if env!srex-hack-attempt
	     (= out "null"))

;; 	 (if (srex-is-decrypted-data env lhs)
;; 	     (env!srex-hack-attempt t)
;; 	     (outpend (str lhs " = " rhs ";")))

	 ;;(prn "out from verify-apply: " out)
	 out))

(def verify-each (env data oldparams depth)
  (withs (out "" counter 0 name (srex-uniq)
	  eachbody nil)
	 (withs (listcounter 0)
		;; must generate list variable
		(outpend #\newline)
		(outpend (str "var counter_" name " = 0;" #\newline))
		(outpend (str "var each_" name " = "))
		(if (is (type (data 2)) 'cons)
		    (if (is (str ((data 2) 0)) "list")
			;; this is indeed a list
			(do (= out (str out "["))
			    (each l (data 2)
				  (if (> listcounter 1)
				      (= out (str out ", ")))
				  (if (> listcounter 0)
				      (do ;;(prn "will emit item for each_")
					  (= out (str out (verify-exp env l oldparams depth)))
					  ;;(prn "DID emit item for each_"))
					  ))
				  (++ listcounter))
			    (= out (str out "]")))
			;; this is a function call
			(verify-exp env (data 2) oldparams depth))
		    (atom (data 2))
		    ;; this is a variable
		    (= out (str out (data 2)))
		    )
		(= out (str out ";" #\newline))
		(= (env!srex-used "srex-length") t)
		(outpend (str "var length_" name " = srex_length(each_" name ");" #\newline))
		)
	 (outpend (str "while (counter_" name " < length_" name ") {" #\newline))
	 (++ depth)
	 (outpend (str "var " (data 1) " = each_" name "[counter_" name "];" #\newline))
	 (withs (bodycounter 0 paramstouse oldparams)
		(each i data
		      (if (is bodycounter 1)
			  ;; consider the each iterator is a local variable
			  (push (str i) paramstouse))
		      (if (> bodycounter 2)
			  (push i eachbody))
		      (++ bodycounter))
		(each i eachbody
		      (outpend (verify-exp env i paramstouse depth) ";" #\newline)))
	 (outpend (str "counter_" name " += 1;" #\newline))
	 (-- depth)
	 (outpend (str "}"))
	 out))

(def verify-maptable (env data oldparams depth)
  ;;(prn "in verify-maptable: " data)
  (withs (out "" fnitem (data 1) key ((fnitem 1) 0) value ((fnitem 1) 1)
	  bodycounter 0 maptablebody nil)
	 (each i fnitem
	       (if (> bodycounter 1)
		   (push i maptablebody))
	       (++ bodycounter))
	 (outpend #\newline)
	 (outpend (str "for (" key " in " (data 2) ") {" #\newline))
	 (++ depth)
	 (outpend (str "var " value " = " (data 2) "[" key "];" #\newline))
	 (each i maptablebody
	       (outpend (verify-exp env i (+ oldparams (list (str key))) 0) ";" #\newline))
	 (-- depth)
	 (outpend (str "}"))
	 ;;(prn "out of verify-maptable: " out)
	 out))

(def verify-if (env data oldparams depth)
  ;;(prn "in verify-if: " data)
  (withs (out "" counter 0 els (len data))
	 (outpend #\newline)
	 (outpend "")
	 (each i data
	       ;;(prn "els: " els ", if item " counter ": " i)
	       (if (> counter 0)
		   (if (is (mod counter 2) 1)
		       (if (is (+ counter 1) els)
			   ;; processing the last element
			   (do (= out (str out (verify-exp env i oldparams (+ depth 1)) ";" #\newline))
			       (outpend (str "}" #\newline))
			   )
			   ;; processing a boolean expression
			   (do (= out (str out "if ("))
			       (= out (str out (verify-exp env i oldparams depth)))
			       (= out (str out ")"))))
		       (is (mod counter 2) 0)
		       (do (= out (str out " {" #\newline))
			   (= out (str out (verify-exp env i oldparams (+ depth 1)) ";" #\newline))
			   (if (is (+ counter 2) els)
			       (outpend (str "} else {" #\newline))
			       (< (+ counter 1) els)
			       (outpend (str "} else "))
			       (outpend (str "}")))
			   )))
	       (++ counter))
	 ;;(prn "out from verify-if")
	 out))
