;; from http://awwx.ws/tojson
(def 4hex (i)
  (let s (coerce i 'string 16)
    (string (n-of (- 4 (len s)) #\0) s)))

(def json-encoded-char (c)
  (case c
    #\newline "\\n"
    #\"       "\\\""
    #\\       "\\\\"
    (let i (coerce c 'int)
      (if (<  i 32)
           (string "\\u" (4hex i))
           c))))

(def pr-json-string (s)
  (pr #\")
  (each c s (pr:json-encoded-char c))
  (pr #\"))

;; ;; our version: too slow, but stable
;; (def json-escape (s)
;;   (withs (fstring (str tmpdir* (rand-string 8)))
;; 	 (fwrite fstring s)
;; 	 (tostring (system (str "(cat " fstring " | python json-escape.py); rm " fstring)))))

;; reuse awwx.xs pr-json-tostring
(def json-escape (s)
  (tostring (pr-json-string s)))

(def tojson-other (v)
  (if (is (type v) 'int)    v
      (is (type v) 'num)    v
      (is (type v) 'string) (json-escape v)
      (is (type v) 'sym)    (if v
				(if (is v t)
				    "true"
				    (str "\"" v "\""))
				"false")))

(def tojson-list (l (o depth 2))
  (withs (out "[" counter 0)
	 (each el l
	       (if (> counter 0)
		   (= out (str out ",")))
	       (= out (str out (if (is (type el) 'table)  (tojson-obj el (+ depth 2))
				   (is (type el) 'cons)   (tojson-list el (+ depth 2))
				   (tojson-other el))))
	       (++ counter))
	 (= out (str out "]"))
	 out))

(def tojson-obj (obj (o depth 2))
  (withs (out (str "{" #\newline) counter 0)
	 (maptable (fn (k v)
		       (if (> counter 0) (= out (str out "," #\newline)))
		       (= out (str out (tostring (repeat depth (pr " ")))))
		       (= out (str out "\"" k "\": "
				   (if (is (type v) 'table)  (tojson-obj v (+ depth 2))
				       (is (type v) 'cons)   (tojson-list v (+ depth 2))
				       (tojson-other v))))
		       (++ counter)
		       ) obj)
	 (= out (str out #\newline "}" #\newline))
	 out))

(def tojson (el (o depth 2))
  (if (is (type el) 'table) (tojson-obj el (+ depth 2))
      (is (type el) 'cons)  (tojson-list el (+ depth 2))
      (tojson-other el)))
