(def node-openpgp-install ()
  (tostring (system "npm install --save openpgp")))

(def node-openpgp-key-exists (user)
  (fexists (str appdir* "/" app-name* "/" user "/node-openpgp/pubkey")))

(def node-openpgp-savefile (path var)
  (str "        fs.writeFile('" path "', " var ", function(err) {
          if(err) {
            return console.log(err);
          }
        });"))

(def node-openpgp-genkey (user)
  (withs (fcmd (str tmpdir* (uniq)))
	 (fwrite fcmd (str "
var fs = require('fs');
var openpgp = require('openpgp');
var options = {
    numBits: 3072,
    userId: '" user "@" site-domain* "',
    passphrase: null
};

openpgp.generateKeyPair(options).then(function(keypair) {
        // success                                                                  
        var privkey = keypair.privateKeyArmored;
        var pubkey = keypair.publicKeyArmored;

        // save the pubkey
" (node-openpgp-savefile (str appdir* app-name* "/" user "/node-openpgp/pubkey") "pubkey") "

        // save the privkey
" (node-openpgp-savefile (str appdir* app-name* "/" user "/node-openpgp/privkey") "privkey") "

    }).catch(function(error) {
            // failure                                                              
        }); 
"))
	 (tostring (system (str "mkdir -p " appdir* app-name* "/" user "/node-openpgp;"
				"chmod 700 " appdir* app-name* "/" user "/node-openpgp;"
				"(cat " fcmd " | node);"
				"rm " fcmd)))))

(def node-openpgp-importkey (user)
  (withs (p (prof user))
	 (fwrite (str appdir* app-name* "/" user "/node-openpgp/one-off-imported-pubkey") p!pubkey)))

(def node-openpgp-encrypt (user msg sign)
  (withs (fin (str tmpdir* (uniq))
	  fout (str tmpdir* (uniq))
	  fcmd (str tmpdir* (uniq))
	  ret nil)
	 (fwrite fin msg)
	 (fwrite fcmd (str "
var fs = require('fs');
var data_to_encrypt = '" msg "';
var privkey = " (pgp-tomultiline (fread (str appdir* app-name* "/" user "/node-openpgp/privkey")) "              ") ";
var target_pubkey = " (pgp-tomultiline (fread (str appdir* app-name* "/" user "/node-openpgp/one-off-imported-pubkey")) "              ") ";

var openpgp = require('openpgp');
var privkey_openpgp = openpgp.key.readArmored(privkey).keys[0];
console.log('will dump privkey_openpgp');
var target_pubkey_openpgp = openpgp.key.readArmored(target_pubkey).keys[0];

var keymanager_keys = [ target_pubkey_openpgp ];
  { // from openpgp.js
    var msg, armored;
    msg = openpgp.message.fromText(data_to_encrypt);
" (if sign
      "    msg = msg.sign([privkey_openpgp]);"
      "") "
    msg = msg.encrypt(keymanager_keys);
    armored = openpgp.armor.encode(openpgp.enums.armor.message, msg.packets.write());
    //console.log('armored is: ' + armored);
    " (node-openpgp-savefile fout "armored") "
  }
"))
	 (fwrite "/tmp/nodeout" (tostring (system (str "cat " fcmd " | " (node) " 2>&1"))))
	 (= ret (fread fout))
	 ;;(tostring (system (str "rm " fout " " fcmd " " fin)))
	 ret))

;; ========================================================================
;; ==== I couldn't get this to work with either gpg and gpg2 2.0.x. I
;; think it's because 2.0.x doesn't support the %no-protection flag
;; So I gave up and used nodejs instead
;; ========================================================================
(def gpg-key-exists (user)
  (fexists (str appdir* "/" app-name* "/" user "/gnupg/pubring.gpg")))

;; how to generate entropy
;; $ apt-get install rng-tools
;; $ rngd -r /dev/urandom  # as root
;; $ gpg2 --version | head -1
;; gpg (GnuPG) 2.0.19
;; (gpg-genkey "atcQZoEUr9lEKS")

;; need gpg2 for the %no-protection flag
;; without %no-protection, openpgp.js is unable to decrypt the private key:
;; TypeError: Cannot read property 'isDecrypted' of null
(def gpg-genkey (user)
  (withs (fcmd (str tmpdir* (uniq)))
	 (fwrite fcmd (str "%echo Generating a basic OpenPGP key
Key-Type: RSA
Key-Length: 3072
Subkey-Length: 3072
Name-Real: " user "
Name-Email: " user "@" site-domain* "
Expire-Date: 0
%pubring " appdir* app-name* "/" user "/gnupg/pubring.gpg
%secring " appdir* app-name* "/" user "/gnupg/secring.gpg
%no-protection
%no-ask-passphrase
# Do a commit here, so that we can later print \"done\" :-)
%commit
%echo done
"))
	 (tostring (system (str "mkdir -p " appdir* app-name* "/" user "/gnupg;"
				"export GNUPGHOME=" appdir* app-name* "/" user "/gnupg;"
				"(cat " fcmd " | gpg2 --batch --gen-key);"
				"chmod 700 " appdir* app-name* "/" user "/gnupg;"
				"rm " fcmd)))))

(def gpg-importkey (user)
  (withs (p (prof user) fkey (str tmpdir* (uniq)))
	 (fwrite fkey p!pubkey)
	 (tostring (system (str "export GNUPGHOME=" appdir* app-name* "/" user "/gnupg;"
				"gpg2 --import " fkey " > /tmp/import.out 2>&1;"
				;;"gpg2 --import " fkey ";"
				"rm " fkey)))))

(def gpg-encrypt (user msg)
  (withs (fin (str tmpdir* (uniq))
	  fout (str tmpdir* (uniq))
	  ret nil)
	 (fwrite fin msg)
	 (tostring (system (str "export GNUPGHOME=" appdir* app-name* "/" user "/gnupg;"
				;; use '--trust-model always' to bypass the following valid,
				;; warning; the warning "caught us": we are forging a message
				;;
				;; It is NOT certain that the key belongs to the person named
				;; in the user ID.  If you *really* know what you are doing,
				;; you may answer the next question with yes.
				"gpg2 --trust-model always --sign --output " fout " --armor --encrypt --recipient " user "@" site-domain* " " fin ";"
				"rm " fin ";")))
	 (= ret (fread fout))
	 (tostring (system (str "rm " fout)))
	 ret))

(def example-failed-signature-mitm (user msg encpass sign)
  (withs (forged-message nil
          encrypted-forged-message nil
	  dataid-invalid nil)
	 ;; ======== THIS IS WHERE WE FORGE THE MESSAGE =======
	 (= forged-message "meet me under the bridge and make sure no one follows you")
	 (= encrypted-forged-message ;;(gpg-encrypt user forged-message)
	                             (node-openpgp-encrypt user forged-message sign))
	 (= dataid-invalid (add-data user "blog-entry" encpass encrypted-forged-message nil nil nil))
	 dataid-invalid))

(defop example-failed-signature req
  ;; add this test in case someone removes "node" as a needed tool in (main)
  (if (no (tool "node"))
      (page (backlink "/examples") site-name* nil
	    (encverif (get-user req))
	    (pr "Can't run this example because gpg is missing on the server.")
	    (br r8)
	    (hline))
      (example-failed-signature-intro req)))

(def example-failed-signature-intro (req)
  (page (backlink "/examples") site-name* nil
	(encverif (get-user req))
	(pr "This example simulates and detects a man-in-the-middle attack. It shows data fields that are decrypted correctly but fail signature verification are changed to have a red background.")
	(br2)
	(prbold "It will take two minutes") (pr " for the links below to load the first time you click on any one of them, because the server generates a public-key pair on the fly to prepare for the attack.")
	(br2)
	(withs (msg "Meet me at the opera!")
	       (tag (ul)
		    (tag (li)
			 (w/link (example-failed-signature-ok req msg "unsigned")
				 (pr "Decrypt an unsigned message."))
			 (br2))
		    (tag (li)
			 (w/link (example-failed-signature-ok req msg "forge")
				 (pr "Decrypt a message forged to appear it's from you (MitM)."))
			 (br2))))
	(br)
	(pr "If you visit the /demo link, you won't be able to run these two examples again without re-installing the plugin. The server will delete your cookie and lose access to your public key. The examples need the public key to know who to encrypt the message for.")
	(br2)
	(click-source "joan-example-failed-signature.arc")
	(br 8)
	(hline)))

(def example-failed-signature-ok (req msg extype)
  ;; generate public-key pair on the server to simulate attack
  (do (if (no ;;(gpg-key-exists (get-user req))
	   (node-openpgp-key-exists (get-user req))
	   )
	  (do ;;(gpg-genkey (get-user req))
	    ;;(gpg-importkey (get-user req))
	    (node-openpgp-install)
	    (node-openpgp-genkey (get-user req))
	    (node-openpgp-importkey (get-user req))
	    )
	  ;; so it shows in the server terminal
	  (system (str "echo skipped regenerating an existing key for " (get-user req) " >&2"))))
  (page (backlink "/examples") site-name* nil
	;; define what the browser extension should do
	(rex (list (list 'may_publish 'ENTER_TEXT)
		   (list 'encrypt
			 (obj pubuserid (get-user req)
			      formbuttons (list (obj form "ENTER_TEXT"
						     btn 'example-text
						     encpassid "encpass"))
			      enc-itemlist (list (obj id "body"))))))
	(pr "In this page press 'submit', and the text below will be encrypted, signed with your private key, and sent to the server.")
	(br2)
	(pr "In the next page the data will be decrypted and displayed, but the signature of the message will not match yours. It will be a fake signature generated by the server: a signature for the same user id, but with a different signature id.")
	(br 3)
	(click-source "joan-example-failed-signature.arc")
	(br2)
	(sform "ENTER_TEXT"
	       (fn (_)
		   (withs (p (prof (get-user _))
			   body (arg _ "body")
			   encpass (arg _ "encpass")
			   dataid nil
			   dataid-invalid nil)
			  (encpass-read encpass p)
			  (if (is encpass "") (= encpass nil))
			  (= dataid (add-data (get-user _) "blog-entry" encpass body nil nil nil))
			  ;; create a second, forged message, to simulate a man-in-the-middle attack
			  (if (is extype "forge")
			      (= dataid-invalid (example-failed-signature-mitm (get-user _) msg encpass t))
			      (is extype "unsigned")
			      (= dataid-invalid (example-failed-signature-mitm (get-user _) msg encpass nil)))
			  (example-text-view-encrypted (get-user _)
						       ;; dataid 
						       dataid-invalid ;; return the forged message
						       )))
	       ;; each data item has an encryption password
	       (gentag input type "hidden" id "encpass" name "encpass" value "")
	       (tag (textarea id "body" name "body" rows 3 cols 53 autofocus "")
		    (pr msg))
	       (br 3)
	       (encverif (get-user req))
	       (tag (a id "example-text")
		    (pr "submit")))
	(br 8)
	(hline)
	))
