(defop example-nodecrypt req
  (example-nodecrypt-page1 req t nil))

(def example-nodecrypt-page1 (req do-not-hack hacktype)
  (page (backlink "/examples") site-name* nil
	;; define what the browser extension should do
	(rex (list (list 'encrypt
			 (obj pubuserid (get-user req)
			      formbuttons (list (obj form "ENTER_TEXT"
						     btn 'example-text
						     encpassid "encpass"))
			      enc-itemlist (list (obj id "body"))))))
	(pr "This example shows you can sent unencrypted (new) data, but can't sent decrypted (old) data back to the server.")
	(br 3)
	(pr "There are two steps in this example:")
	(br)
	(tag (ul)
	     (tag (li)
		  (pr "In this page, you enter some text and the text is encrypted and saved on the server.")
		  (br2))
	     (tag (li)
		  (pr "In the next page, you'll see the decrypted text you entered and will have to type it in again. That page will check if what you typed in the second time matches what you typed in the first time. If it matches, an unencrypted flag is sent to the server that says the text matches.")
		  (br2)
		  (if do-not-hack
		      (pr "The checking code obeys an extra safety check enforced by the code verifier: it can't send decrypted data back to the server. If it tries, control flow analysis detects it and stops the workflow. Decrypted data can't leave the browser.")
		      (do (prbold "Sending decrypted data will fail.")
			  (pr " Safety analysis will detect the attempt to steal decrypted data and stop the workflow.")))))
	(br2)
	(click-source "joan-example-nodecrypt.arc")
	(pr " Click ")
	(tag (a href (if do-not-hack
			 "/joan-example-nodecrypt-sent-good.arc"
			 (if (is hacktype "static")
				 "/joan-example-nodecrypt-sent-bad-static.arc"
				 "/joan-example-nodecrypt-sent-bad-dynamic.arc")))
	     (pr "here"))
	(pr " to see the ")
	(if do-not-hack
	    (prbold "GOOD")
	    (prbold "BAD"))
	(pr " code written on the server that is executed on the second page by the browser extension.")
	(br2)
	(pr "To see how the same example ")
	(if do-not-hack
	    (do (prbold "DOES")
		(pr " raise an alert if the code written on the server tries to send decrypted (old) data back to the server: ")
		(tag (ul)
		     (tag (li)
			  (pr "by supplying input field names statically")
			  (pr " click ") (w/link (example-nodecrypt-page1 req nil "static") (pr "here.")))
		     (br)
		     (tag (li)
			  (pr "by constructing input field names dynamically")
			  (pr " click ") (w/link (example-nodecrypt-page1 req nil "dynamic") (pr "here.")))))
	    (do (pr "does ") (prbold "NOT")
		(pr " raise an alert if the code written on the server only sends unencrypted (new) data, but ")
		(prbold "NOT") (pr " decrypted (old) data back to the server")
		(pr " click ") (w/link (example-nodecrypt-page1 req t nil) (pr "here."))))
	(br2)
	(sform "ENTER_TEXT"
	       (fn (_)
		   (withs (p (prof (get-user _))
			   body (arg _ "body")
			   encpass (arg _ "encpass")
			   dataid nil thedata nil)
			  (encpass-read encpass p)
			  (if (is encpass "") (= encpass nil))
			  (= dataid (add-data (get-user _) "blog-entry" encpass body nil nil nil))
			  (= thedata (data (get-user _) dataid))
			  (example-nodecrypt-page2 _ thedata nil do-not-hack hacktype)))
	       ;; each data item has an encryption password
	       (gentag input type "hidden" id "encpass" name "encpass" value "")
	       (encverif (get-user req))
	       (tag (textarea id "body" name "body" rows 12 cols 53 autofocus "")
		    (pr ""))
	       (showsign nil t)
	       (br 3)
	       (tag (a id "example-text")
		    (pr "submit")))
	(br 8)
	(hline)
	))

(def example-nodecrypt-page2 (req data error do-not-hack hacktype)
  (page (backlink "/examples") site-name* nil
	(withs (dec-itemlist nil)
	       ;; must provide the password and id of the item to decrypt
	       (push (obj encpass (encpass-val data!encpass (get-user req))
			  id (str "entry_1")) dec-itemlist)
	       (rex (list ;; - every submit button should be accompanied by an 'encrypt command
		          ;; - an encrypt command also decrypts, so there should be no 'decrypt if there's an 'encrypt
		          ;;(list 'decrypt "blog-entry" nil dec-itemlist)
			  (list 'encrypt
				(obj pubuserid (get-user req)
				     formbuttons (list (obj form "ENTER_TEXT"
							    btn 'example-nodecrypt-page2
							    encpassid "encpass"))
				     enc-itemlist (list (obj id "body"))
				     dec-itemlist dec-itemlist)))
		    (if do-not-hack
			(fread "joan-example-nodecrypt-sent-good.arc")
			(if (is hacktype "static")
			    (fread "joan-example-nodecrypt-sent-bad-static.arc")
			    (fread "joan-example-nodecrypt-sent-bad-dynamic.arc"))))
	       (if error
		   (do (br2)
		       (prbold "The text entered did not match. Try again.")
		       (br2)))
	       (pr "The decrypted text is:")
	       (br2)
	       (encverif (get-user req))
	       (tabwrap
		(tag (span id (str "entry_1"))
		     (pr (topgp data!data))))
	       (br2)
	       (pr "Now enter the same text again:")
	       (sform "ENTER_TEXT"
		      (fn (_)
			  (withs (matchok (arg _ "matchok"))
				 (if (is matchok "yes")
				     (example-nodecrypt-page3 do-not-hack (arg _ "send_back_decrypted"))
				     (example-nodecrypt-page2 _ data t do-not-hack hacktype))))
		      ;; each data item has an encryption password
		      (gentag input type "hidden" id "matchok" name "matchok" value "no")
		      (gentag input type "hidden" id "encpass" name "encpass" value "")
		      (if (no do-not-hack)
			  ;; ==========================================================
			  ;; HACK ATTEMPT: Create a field that will
			  ;; hold decrypted data, in an attempt to
			  ;; steal the data and send it back to the
			  ;; server
			  ;; ==========================================================
			  (gentag input type "hidden" id "send_back_decrypted" name "send_back_decrypted" value ""))
		      (encverif (get-user req))
		      (tag (textarea id "body" name "body" rows 12 cols 53 autofocus "")
			   (pr ""))
		      (showsign nil t)
		      (br 3)
		      (tag (a id "example-nodecrypt-page2")
			   (pr "submit")))
	       (br 8)
	       (hline)
	       )))

(def example-nodecrypt-page3 (do-not-hack stolen-data)
  (page (backlink "/examples") site-name* nil
	(if do-not-hack
	    (do (prbold "Success. ")
		(pr "The text entered the second time matched."))
	    (no do-not-hack)
	    (do (pr "The text entered the second time matched.")
		(br2)
		(if (no stolen-data)
		    (do (prbold "Success. ")
			(pr "There was an attempt to hack you, but no data were stolen and sent back to the server. The data sent back were:"))
		    (do (prbold "You got hacked. ")
			(pr "The following decrypted data were sent back to the server:")))
		(br2)
		(pr stolen-data)))
	(br 8)
	(hline)))
