;; for the demo, these are test keys, not live keys
(= stripe_secret_key*      "sk_test_S3kosqfW8W0PThARh7AAR9Kh")
(= stripe_publishable_key* "pk_test_1WiRnNstW603Lrdj0Z7hm2bG")

(def safe-item (item)
  (withs (clean item)
	 (if (or (is (type item) 'int)
		 (is (type item) 'char)
		 (is (type item) 'num))
	     (= clean item)
	     (do (each c (list " " ";" "\r" "\n" "\"" "'"    ;; common injection attacks
			       "<" ">" ">>" "<<" ":" "*"
			       "!" "&" "|" "||" "&" "&&" "`" ;; shell injection attacks
			       "$"
			       )
		       (= clean (cut clean 0 (findsubseq c clean))))
		 ;;(prn "cleaned item from " item " to " clean)
		 ))
	 clean))

(def stripe-customer-create (desc token)
  (sysjson (str "curl https://api.stripe.com/v1/customers"
		" -u " stripe_secret_key* ": "
		" -d \"description=" desc "\" "
		" -d card=" (safe-item token))))

(def stripe-customer-charge (desc customer-id dollars)
  (withs (cents (* dollars 100))
	 (sysjson (str "curl https://api.stripe.com/v1/charges"
		       " -u " stripe_secret_key* ": "
		       " -d currency=usd "
		       " -d \"description=" desc "\" "
		       " -d customer=" customer-id " "
		       " -d amount=" cents))))

(defop example-payments req
  (example-payments-core req nil))

(def example-payments-core (req errmsg)
  (withs (amount 50) ;; in dollars
  (page (backlink "/examples") site-name* nil
	;; define what the browser extension should do
	(rex (list (list 'encrypt
			 (obj formbuttons (list (obj form "PAYMENT"
						     btn 'example-payment
						     encpassid "encpass"))
			      enc-itemlist (list (obj id "number")
						 (obj id "exp_month")
						 (obj id "exp_year")
						 (obj id "cvc"))
			      )))
	     (fread "joan-example-payments-sent.arc"))
	(pr "This example processes payments with Stripe. It uses Stripe's payment API to charge a credit card under test mode without using Stripe's Javascript library.")
	(br2)
	(pr "Use the fake card number: 4242 4242 4242 4242.")
	(br 3)
	(pr "Payments conflict with encryption. Credit card info must first be sent unencrypted to a foreign server (to Stripe) to authorize the transaction, calling the wrapper function jq-ajax. Since that's insecure, the code verifier flags the fields in red.")
	(br2)
	(pr "But although the data are sent unencrypted to the foreign server, they are sent encrypted to the web server.")
	(br 3)
	(click-source "joan-example-payments.arc")
	(pr " Click ")
	(tag (a href "/joan-example-payments-sent.arc")
	     (pr "here"))
	(pr " to see the code written on the server that is executed by the browser extension.")
	(br 4)
	(if errmsg
	    (do (br2)
		(center (tag (div style "background-color: red; padding: 14px;")
			     (br)
			     (prbold errmsg) (br)
			     (pr "Please try again.") (br2)))
		(br2)))
	(sform "PAYMENT"
	       (fn (_)
		   (withs (user (get-user _) p (prof user)
			   stoken (arg _ "stripeToken")
			   stripe-cus-id nil
			   res nil)
			  ;; first, create a customer
			  (= res (stripe-customer-create (str site-name* " user " user) stoken))
			  (if (or (no res) (no (res "id")))
			      (example-payments-core "The payment did not go through; could not create new customer." _)
			      (do (= stripe-cus-id (res "id"))
				  (= res nil)
				  (if p
				      (do (= p!stripe-cus-id stripe-cus-id)
					  (save-prof p)))
				  ;; then, charge the customer
			          (= res (stripe-customer-charge (str site-name* " Stripe payment example " user) stripe-cus-id amount))
				  (if (or (no res) (no (res "id")))
				      (example-payments-core "The payment did not go through; could not charge customer." _)
				      (payment-succeeded stripe-cus-id (res "id")))))))
	       (gentag input type "hidden" id "encpass" name "encpass" value "")
	       (encverif (get-user req))
	       ;; note: this field would otherwise be added automatically if you were using stripe.com.v2.js
	       (gentag input type "hidden" id "stripeToken" name "stripeToken" value "")
	       (showsign nil t)
	       (tab
		(row (nbsp) (nbsp))
		(row (nbsp) (nbsp))
		(row (pr "Card Number:")
		     (gentag input id "number" name "number" autofocus "" type "text" size 20 maxlength 16 style "width:160px"))
		(row (pr "Expires:")
		     (do (gentag input id "exp_month" name "exp_month" type "text" size 2 maxlength 2 style "width:35px" placeholder "MM")
			 (tag (span)
			      (pr " / "))
			 (gentag input id "exp_year" name "exp_year" type "text" size 2 maxlength 2 style "width:35px" placeholder "YY")
			 ))
		(row (pr "Card Code:")
		     (gentag input id "cvc" name "cvc" type "text" size 4 maxlength 3 style "width:40px" placeholder "CVC"))
;; 		(row (pr "Name on card:")
;; 		     (gentag input id "name" type "text" size 4 style "width:160px"))
		(row (br))
		(row ""
		     (tag (a id "example-payment")
			  (pr "submit"))
		     ))
	       )
	(br 3)
	(pr "You will supposedly be charged $" amount ".")
	(br 8)
	(hline))))

(def payment-succeeded (stripe-cus-id charge-id)
  (page (backlink "/examples") site-name* nil
	(pr "The payment was processed successfully for customer " stripe-cus-id " with charge id " charge-id ".")))
