(load "joan-version.arc")
(load "share_with_plugin.arc")
(load "share_with_plugin_driver.arc")

(attribute ol         type          opstring)

(def abusive-ip (ip) nil)
(= threadlife* 300) ;; don't break when searching in large documents

(= tabw* 455)
(= black  (color 30 30 30))
(= site-name* "Joan")
(= site-domain* "somedomain.com")
(= app-name* "joan")
(= tmpdir* "/tmp/")

(mac pmall back
     `(tag (td style "width:49px;vertical-align:top;")
	   (tag (table border 0 cellpadding 0 cellspacing 0 style "border-spacing:3px;")
		(tr (tag (td style "vertical-align:bottom; text-align:right; padding-bottom: 7px;")
			 ,@back)))))
(def pmalc () (tag (td style "width:26px; vertical-align:top;")))
(mac pmalr body
     `(tag (td style (str "width:" tabw* "px;vertical-align:top;"))
	   (tag (table width tabw*)
		(tr (tag (td)
			 (tag
			  (font size 2 face "Verdana")
			  ,@body))))))

(mac pagemulti-align-left(back . body)
     `(tag (span style "left:0px;")
	   (tag (table)
		(tr (pmall ,back)
		    (pmalc)
		    (pmalr ,@body)))))

(def backlink (back)
  (if back
      (tag (a href back)
	   (pr "back"))))

(def css ()
  (prn "<link rel=\"stylesheet\" type=\"text/css\" href=\"joan.css\">")
  (prn "<link rel=\"shortcut icon\" href=\"favicon.png\"></link>"))

(defop joan.css req
  (pr (str "
body {font: 13px Verdana;letter-spacing: 0; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px;}
td {font: 13px Verdana;letter-spacing: 0;}

input[type=\"text\"] 
{ 
    font: 13px Verdana;letter-spacing: 0;
    height: 24px;
    width: 150px;
    color: " (hexrep black) ";
}

textarea 
{ 
    font: 13px Verdana;letter-spacing: 0;
    color: " (hexrep black) ";
}

")))

(mac page (back title js . body)
     `(tag html
          (tag head
               (css)
               (tag title (pr ,title)))
          (tag body
               (prn)
	       (br 4)
	       (tag (span style "height:100%;min-height:100%;padding-bottom:50px;")
		    (pagemulti-align-left ,back ,@body))
	       (if (no (is ,js nil))
		   (pr ,js)))
          ))

(= wordwrapfix* "word-wrap: break-word;")
(mac tabwrap-core (width . body) ;; make sure long text without spaces wraps around
     `(tag (table border 0 cellpadding 0 cellspacing 0
		  style "width:100%;table-layout:fixed;")
	   (tr (tag (td style (str "width: " ,width "px;" wordwrapfix*))
		    ,@body))))
(mac tabwrap body
     `(tabwrap-core tabw* ,@body))

(mac tabwraplong body
     `(tabwrap-core (+ tabw* 100) ,@body))

(def hline ()
  (tag (table border 0 cellspacing 0 cellpadding 0 width tabw*)
       (tr (tag (td)
		(tag (font size 2 face "Verdana")
		     (br 3)
		     (tag (hr))
		     )))))

;; /security-extension-x does the same thing /x does in Arc: process
;; fnids. But it standardizes the url used.
(defop-raw security-extension-fnid (str req)
  (withs (p (prof (get-user req)))
	 (= p!pubkey (arg req "pubkey"))
	 (= p!encverifiedkeyids (arg req "encverifiedkeyids"))
	 (= p!encsearchinfo (arg req "encsearchinfo"))
	 (save-prof p))
  (w/stdout str 
    (aif (fns* (sym (arg req "fnid")))
         (it req)
         (pr dead-msg*))))

(= orange (color 230 90 41))
(def title (t)
     (fontcolor orange
                (tag (font size 3 face "Verdana")
                     (pr t)))
     (br2))

 ;; for this demo, always create a new cookie in the front page. The
 ;; user must have a cookie in their browser before installing the
 ;; extension. The server needs to know which user tries to enable
 ;; encryption.
(defop || req
  (page nil site-name* nil
	(title site-name*)
	(pr site-name* " is a Chrome extension for safely encrypting web form data.")
	(br2)
	(pr "Say you're submitting a web form; when you press submit, the data in the textboxes in the form are automatically encrypted. You don't have to do anything manually to encrypt the data.")
	(br2)
	(pr site-name* " is the first system that lets one write web apps that do all three of the following:")
	(tag (ol type "a")
	     (tag (li)
		  (pr "encrypt data safely in the browser, inside a browser extension without trusting the server or tricking the user, and"))
	     (br)
	     (tag (li)
		  (pr "update what to encrypt easily: the choice of data fields to encrypt is controlled by the web server, and if one day different data fields should be encrypted the user doesn't need to update the browser extension, and"))
	     (br)
	     (tag (li)
		  (pr "require no manual effort from the user: data are automatically encrypted and automatically decrypted, without the user needing to click some button that encrypts or to continually enter a decrypt password.")))
	(pr "A server attacker can't access decrypted data protected like this.")
	(br2)
	(pr "Although " site-name* " today is closer to a framework for writing web apps, that's far from what I'd like (web) programming to evolve to. It's more accurate to say " site-name* " is an exercise in writing web apps for some future version of the web, with an informal version 1 written for today's browser.")
	(br2)
	(pr "You are looking at the server part of a live demo that shows how " site-name*" works. The demo shows how you can encrypt data in these ")
	(tag (a href "/examples")
	     (pr "examples"))
	(pr " but requires you install the Chrome extension to do that. Click ")
	(tag (a href "/demo")
	     (pr "here"))
	(pr " to run the demo.")
	(br2)
	(tag (ul)
	     (tag (li)
		  (tag (a href (str "/how-it-works"))
		       (pr "How it Works")))
	     (br)
	     (tag (li)
		  (tag (a href (str "/faq"))
		       (pr "Frequently Asked Questions")))
	     (br)
	     (tag (li)
		  (tag (a href (str "/notes"))
		       (pr "Implementation Details")))
	     (br)
	     (tag (li)
		  (tag (a href (str "/" app-name* version* ".tar.gz"))
		       (pr "Download"))
		  (pr " version " version* " of the source code.")))
	(br)
	(pr "The browser extension source code has not been audited for security. I'm not sure if the best direction forward is a browser plugin or a new ")
	(tag (a href "/how-it-works#browser")
	     (pr "browser"))
	(pr ".")
	(br2)
	(br 8)
	(hline)))

(defop how-it-works req
  (page (backlink "/") site-name* nil
	(prbold "How it Works")
	(br2)
	(pr "The browser extension protects decrypted data from unauthorized access by enforcing a key restriction: decrypted data can't leave the browser. It verifies all code sent by the server does not violate this restriction before allowing the code to execute. All a web app can do with decrypted data is display them or use them in navigation logic. ")
	(br2)
	(pr "If an application wants to use decrypted data for some other reason, it has to convince the user to retype the data in.")
	(br2)
	(pr "To encrypt, data are copied from the input fields of a browser tab (from the browser web page) into the browser extension. They are encrypted there with public-key cryptography, copied back into the input fields of the browser tab, sent to the server and saved.")
	(br2)
	(pr "To decrypt, data are retrieved from the server, copied from the browser tab into the browser extension, decrypted inside the browser extension, and displayed back on the browser tab.")
	(br2)
	(pr "At no time can server code steal data saved in the browser extension, like a private key or password.")
	(br2)
	(pr "Attackers who break into the web server can't make the web app do evil things with a user's already encrypted data behind the user's back. For example, they can't make the browser extension decrypt the data and send them off to their own server. Attackers also can't make the browser execute their own evil Javascript, simply because the browser extension prohibits the browser from executing any Javascript. If they attempt foul behavior, the browser extension cracks down on it before it starts and notifies the user. The data stay safe either way, regardless of how the web app is modified.")
	(br2)
	(pr "(The user notices if some input fields will not be encrypted because they are changed to have a red background. A red alert also appears on the top left of the web page.)")
	(br2)
	(pr "What helps the browser extension refuse to execute inline Javascript or an eval() call is a modification to the Content Security Policy. The policy is changed on the fly to guarantee safety each time a web page is retrieved.")
	(br2)
	(pr "An unforeseen twist here is that data that will be encrypted may <i>need</i> to be first sent unencrypted to foreign servers. For example, in credit card payments. When a user presses submit, the credit card number should be sent unencrypted to the payment processor, but still sent encrypted to the app server so the server cannot read the number. The browser extension identifies such input fields, changes them to have a red background, and displays a red alert on the top left of the web page.")
	(br2)
	(pr "The worst thing that can happen is someone breaks into the server and steals the data. But the data are encrypted, and without the private key there are limits to what one can do with them. None of the data can be decrypted if the encryption was strong enough.")
	(br2)
	(tag (a id "the-catch"))
	(prbold "The Catch")
	(br2)
	(pr "A feature that's both powerful and yet missing for years from browsers is unlikely to arrive without casualties. In " site-name* " it comes with a catch: you can't write Javascript, but that doesn't mean you can't execute Javascript. It simply means you can't write Javascript in Javascript. You can write Javascript in another language and the browser extension verifies the code is safe, converts it back to Javascript, and executes it.")
	(br2)
	(pr "Why avoid Javascript? Because it's easier that way. Code executed in the browser must be verified to not steal decrypted data and it's harder to write a safe code verifier for Javascript (and in Javascript) than to write it for another language. I'm sure it's possible in principle but I couldn't bring myself to attempt it in practice.")
	(br2)
	(pr "The reason it's necessary to have a code verifier is to control data access: writing to input fields, reading from input fields, and sending data back to servers. Without a code verifier a hacked server could send to the client bad Javascript code that, upon pressing submit, could copy unencrypted web form data and send them to the server. It could bypass any kind of software protection added by a plugin. But because no Javascript is allowed and there is a code verifier, server code can't copy any web form data fields on its own at all. It has to call wrapper functions to access data. Because data can be accessed only through these wrapper calls, when the code verifier analyzes the code sent from the server it can guarantee data can't be stolen.")
	(br2)
	(pr "What language do you use if you don't use Javascript? The best you could do is use a Lisp, but not just because Lisp is powerful or has macros. Primarily because it's easy to parse: it treats code as a list of data. It's easy with Lisp to examine lists than to first build a parse tree for a language weighted down by syntax. It also helps to choose a Lisp that is succinct. When there are fewer types of tokens to support, you have less work to do to support them.")
	(br2)
	(pr "With " site-name* " you program in Arc, a new Lisp with succinct syntax that works well for basic web apps. The code verifier in the browser extension and the server part of this demo are both written in Arc but you can write the server code in any language.")
	(br2)
	(pr "Although you can't execute arbitrary code, some types of code are safe. Like a function that takes as input a string and splits the string on whitespace. A function like this doesn't refer to input fields on the web page at all; it doesn't get or set anything. As another example, code that accesses encrypted fields, decrypts the data, and does nothing other than display the data is also safe.")
	(br2)
	(pr "Besides executing code you write yourself, you can also execute library functions built in the browser extension. These are functions that simplify writing web apps that use encryption. You can control from the server which functions to use but none of them contain logic that could harm data in the browser.")
	(br2)
	(pr "Here comes the dirty summary of what goes on behind the scenes. You write Arc code on the server, the code is encoded as JSON, and it's sent to the browser in a hidden form variable. The code is copied by a browser extension (currently for Chrome) and decoded from JSON back to an S-expression (a nested list data structure used in Lisp). It is parsed with ArcJS (a Javascript implementation of Arc), interpreted with a safe code verifier written in Arc that produces Javascript, and finally executed in Javascript, safely, in the browser.")
	(br2)
	(pr "This is certainly an ugly hack. That's an awful lot of hoops to jump through for a feature that should probably be offered by default. But it's not out of recklessness " site-name* " was designed to run code safely this way: browsers have yet to offer safe web form data encryption natively. Short of writing or modifying an existing browser, I don't see another option.")
	(br2)
	(tag (a id "evolution"))
	(prbold "Evolution")
	(br2)
	(pr "How did it get to this? Mostly as a result of natural evolution. You are pressed to ship features fast when there's demand and after you ship it's hard to update.")
	(br2)
	(pr "But what if shipping features fast isn't really a disadvantage? Maybe it's more important to write a dirty version 1, understand the problem better, and then go back and rewrite a cleaner version. If that's true, browsers were right to ship HTML and Javascript that fast as version 1. The big thing they are missing now isn't web form data encryption. It's that they haven't writtten version 2 in 20+ years.")
	(br2)
	(pr "It looks appalling to us now that Javascript was practically bolted on top of HTML but it was a natural thing to do when the web was taking off. You had to run code in the browser; Javascript was a fast way to do it. It's arguably because Javascript shipped fast that it ended up winning the web.")
	(br2)
	(pr "Web form data encryption is in a similar spot now. It looks appalling to me now to have bolted a code verifier to the browser as a plugin. But it's not unreasonable a hack like " site-name* " could someday make it into browsers. You have to encrypt web form data in a browser; a code verifier is a fast way to do it. It's arguably a mistake not to ship a code verifier fast.")
	(br2)
	(pr "Why haven't browsers shipped a code verifier already then?")
	(br2)
	(pr "It's probably not because they couldn't. The simplest reason browsers haven't shipped a code verifier is they aren't trying to ship a code verifier. Since web apps aren't asking for one, browsers aren't pressured to add one. But if browsers were pressured or really wanted to they could stitch together a verifier in a month. It took only ten days to write the first version of Javascript after it became evident the web would need a browser language, and with Microsoft closing in on the rearview mirror, Netscape had to release one to stay ahead.")
	(br2)
	(pr "A difference between adding Javascript and adding web form data encryption is that there was more pressure to do the former than there is to do the latter. But a messier truth is that the problem isn't only lack of demand. It also seems to be political. Suppose there suddenly was demand for a code verifier, the browser companies are unlikely to agree to add one and also agree which one to add.")
	(br2)
	(pr "Adding a new attribute to HTML could get consensus from W3C within a month. Adding a code verifier for a programming language is nowhere as easy. Not only is it a harder problem with bigger scope, there isn't consensus for what the best programming language to power the web should be. Just look at Javascript. Practically all web developers use it, and it's unquestionable you need a language in the browser. And yet browsers haven't shipped a better language to replace Javascript however loudly developers have been asking for one. Replacing only Javascript without a code verifier for encryption, which should be a more important and pressing problem, is hard enough of a problem even Google hasn't cracked considering the unfair ")
	(tag (a href "http://en.wikipedia.org/wiki/Dart_(programming_language)#Criticism")
	     (pr "criticism"))
	(pr " it received for its Dart VM, a compelling replacement for Javascript.")
	(br2)
	(pr "So don't expect browsers sprint to add a verifier on top of it all. Especially one that includes a new language, no matter what the language. It's simply too much to ask. A better question to ask is: how do you get off the ground some hard feature whose de facto suppliers can't justify writing yet?")
	(br2)
	(pr "To the extent features harder to build are also more powerful there is a serious, overlooked implication here. That the more powerful a feature you want to add in existing browsers, the more resistance the feature will face. Suppose you wanted servers to access any file on people's laptops, or save more than the current browser limit of 5MB, or some other potent hack you are brewing. Preposterous. You won't get people to agree on any of this. If you wanted some explosive feature and wanted it fast, waiting for the approval of browsers or wavering committees or companies is the last thing you should do.")
	(br2)
	(pr "The most powerful features aren't products of consensus.")
	(br2)
	(pr "(At its extreme, if you came up with a powerful new way for the web to work, you are better off bypassing the browser. You are better off going directly to users.)")
	(br2)
	(pr "This is a classic bootstrap problem. You must first build a dirty, small thing before you build a cleaner, bigger thing. If browsers don't ship a verifier, what options are there left? Aside from writing a new browser, the first version of the code verifier browsers will one day ship will have <i>no other way</i> of getting bootstrapped but through a browser plugin. And not even that would be enough for takeoff, since that doesn't in itself make the code verifier widespread enough to convince browsers to embed it.")
	(br2)
	(pr "Which inevitably brings us to the toughest part. To boostrap the plugin you'd have to get users. A code verifier can't expect to be merged in the browser unless it is first used by a massively popular web app. Just as the web taking off in 1995 was what shipped Javascript in ten days, something else must be taking off now to ship a code verifier in a month.")
	(br2)
	(pr "This shouldn't sound surprising. No matter what you build, it has to get users. Which is the same thing any language that hopes to replace Javascript will also have to do.")
	(br2)
	(pr "A big surprise would be if a browser caught all the way up to the most powerful language. Lisp. That would be something like reaching escape velocity, that point where you finally break free from the shackles of gravity and no further propulsion is needed. A different universe opens up when you reach limits like that.")
	(br2)
	(tag (a id "browser"))
	(prbold "Browser")
	(br2)
	(pr "Is this enough to say for sure it's time for a new browser? Who knows. It feels it's time for me to want one. I now want to write web apps and they are painful to write.")
	(br2)
	(pr "If a new browser took over the web it probably wouldn't look like a browser at first. It would probably look like a database or a desktop app or a release system. It remains a fruitful thought exercise though to imagine how the ideal browser would come about, and as a consequence how the ideal Internet would work.")
	(br2)
	(pr "Let's start by imagining the destination. If anything was possible, how would you want the Internet to work?")
	(br2)
	(pr "I have my own biased preferences. I want all code written in Lisp. I want API calls to not be URLs but Lisp functions too, to have access to source code where possible, and to be free to modify source (both manually and automatically) without stepping on any toes. I don't want to have to worry about code problems like scaling and downtime and ssh-ing between servers and talking about machine specs. Or worry about data problems, like backups and encrypting and syncing files. I don't want a $1000 computer to ever say \"Your startup disk is almost full\", and I want the development environment to take closer to 14MB than 14GB. I want web programming to be as easy as Visual Basic, and no matter which computer I use to have the same environment. Not having Internet access shouldn't make it impossible to use a web app I ran in the past. I want live updates without breaking what's running, function access through capabilities, on some kind of SASOS designed like Plan 9. In the ideal Internet all running programs and data are \"in the computer\" with a working copy on your laptop.")
	(br2)
	(pr "This wish list doesn't show how we get there from where we are now. But the fuel in the burner is need. There is a visible gap from the current Internet to a better one. A good way to discover the ideal Internet is to wait until you hit limits in the current one and we are already hitting limits.")
	(br2)
	(pr "I trip on limits everywhere. Web apps need built-in encryption. Scaling is done manually but would be easier if you could automatically transform the code with Lisp, catching and fixing bugs along with it. Backups and syncing are almost a solved problem minus the caveat filesystems still don't let you run code over data the way a parallel database does. I'm tired of installing software and libraries instead of them just being there. A big open problem with software is reusing it, because it's hard to familiarize yourself with a codebase fast or know ahead of time how well the sofware works, especially if there's no automated testing. I want to hold in my head the entire program I'm writing. But by the time I run through my head these problems I'm facing I'm back to problem number one: I'm not excited about writing web apps.")
	(br2)
	(pr "If only I were the only one. The best programmers I know dislike writing web apps. I had the impression good programmers tend to write backends, and that they dislike the UI/UX because it's easy for the UI flow to come out wrong. Talking to them suggests something different. They mostly seem allergic to Javascript and to mixing languages in web page templates. It's hard to change code that way. They prefer to issue a new request back to the server than write client-side JS. But although there is an appeal to this dirty kind of web programming good programmers avoid there is also a dread, in the same way programming in assembly is delight before it turns into torture.")
	(br2)
	(pr "Which leads to serious, overlooked implication number two. If the best software is written by the best programmers, and the best programmers dislike writing web apps, then the best web software hasn't been written yet. The best web software won't be written with the current web.")
	(br2)
	(pr "The entire web for all we know may be an informal version 1. So while I don't know if it's time for a new browser, I know there is room for one as long as these problems are open.")
	(br2)
	(pr "And as long as we are toying with limits, here's the most ludicrous plan that dawned on me for creating the browser of my dreams. My best guess is it will come from (a) a massively used app, (b) needing a feature browsers lack, (c) written in a Lisp, (d) shipped as a new browser or browser plugin, that (e) either replaces or at least doesn't need all of HTML, CSS, and Javascript, and (f) isn't a product of consensus. I'll break down the explanation again. It's near impossible to get consensus for a powerful new feature browsers lack. The new feature will be needed to write the app and so will start as a plugin or browser. The app will need to get users for the plugin or browser to get users. And browser and app will be programmable in Lisp because Lisp is the best there is. Now that's something that could get the best programmers writing web apps.")
	(br2)
	(pr "Build all that and it'd be enough to tip the scale from tolerating the old Internet to upgrading to a new one. If this gigantic conjecture turns true, it will have gigantic consequences: today's browser and a big chunk of the Internet will be replaced.")
	(br2)
	(pr "It may even be easy to start. The first popular browser ViolaWWW was written in just ")
	(tag (a href "http://en.wikipedia.org/wiki/ViolaWWW#History_of_ViolaWWW")
	     (pr "four days"))
	(pr ".")
	(br2)
	(pr "But to be realistic it's absurd to actually expect the Internet will be replaced. It's no less absurd than the odds of all 6 of these unlikely conditions to hold true. Even if each of them had a generous 50% chance of happening, the probability the Internet will be replaced would be a lowly 1.5%. Which only adds to the shock: some apps may be halfway there. There are massively used web apps out there that already meet at least 5 of these 6 conditions.")
	(br2)
	(pr "Like Dropbox. It syncs local files which browsers can't do, ships as a download, and doesn't need HTML or Javascript. There is no consensus on local file sharing in the form of an RFC. The horrific inconvenience of writing an app that can't use the browser has a flip side. Get users, and you are in position to replace the Internet.")
	(br2)
	(pr "What's just as alarming is how other massively used web apps aren't that close. Facebook isn't. It doesn't need features missing from browsers, you don't need a download, it depends on HTML and Javascript, and it doesn't use Lisp.")
	(br2)
	(pr "Which surprisingly means that if giants like Facebook never use a plugin like " site-name* ", but you write a web app that does, you are in better position to replace the Internet than some of today's Internet giants.")
	(br2)
	(pr "As for the easier problem of web form data encryption, it's hard to predict when, or even if, browsers will add a code verifier. Maybe they'll band together and ship a verifier next month. That'd be a good thing for the web. In the meantime you get three features with " site-name* ": safe encryption that runs automatically, with easy updates, and Lisp in the place of Javascript. Kill three birds with one stone and that'd be a great catch.")
	(br 8)
	(hline)
	))

(defop notes req
  (page (backlink "/") site-name* nil
	(prbold "Implementation Details")
	(br2)
	(pr "This page explains the kinds of things that aren't apparent just by looking at the examples. One of the biggest challenges is that the problem keeps changing the more you learn about it. ")
	(br2)
	(pr "No, it's not enough to sign and verify everything, you need to do it only for fields requested by the server.  No, verification messages aren't secure if they are formatted by the server. No, you can't just verify parts of Javascript, you have to verify all of it. Yes, that appears restrictive at first. Surprise, autocompleting contacts is more laborious than just encrypting. No, it's not enough to encrypt, you also need encrypted search to autocomplete contacts. Oops, payments need to send unencrypted data to foreign servers. Oh no, why is OpenPGP.js so slow?")
	(br2)
	(pr "Writing " site-name* " felt like fighting the mythical Lernaean Hydra, a monster that grew two more heads for each one cut off. You have to cauterize the neck stumps.")
	(br2)
	(pr "If I knew when I started what I know now I would have not written a browser plugin. I would have shot for a browser. Writing a full browser may be too much work but I don't see why one couldn't write a rough version 1 of a browser wrapper in two days.")
	(br2)

	(tag (a id "source-code"))
	(prbold "Source Code")
	(br2)
	(pr "The code that generates the user's private/public keypair is in options_generate.js_common.in.")
	(br2)
	(pr "The code that migrates existing, unencrypted data of a user to become encrypted, after a private/public keypair is generated, is in encrypt_data_page.js_post2.in.")
	(br2)
	(pr "The code that reads the server code from an HTML hidden variable, calls Arc to process it, and executes the Javascript produced by Arc is in add_encryption.js_post.in.")
	(br2)
	(pr "The code verifier is in safe_rex.arc. Control flow starts from the function (srex), and calls (srex-clean-list) that verifies none of the code sent by the server contains bad characters that may inject Javascript. It then calls (srex-codewalk) that verifies the logic in the server code doesn't attempt to steal data starting at (verify-go).")
	(br2)

	(tag (a id "remote-code-execution"))
	(prbold "Remote Code Execution")
	(br2)
	(pr "Remote code execution commands are issued with a (rex) call. The first parameter contains a list of predefined browser extension commands. The second parameter contains as a string custom source code defined at the server that will be executed inside the browser extension. This source code must be inside a single (do) or (withs) block, because ArcJS does not implement (readall) so " site-name* " could only use (read), and (read) only returns the first S-expression.")
	(br2)
	(pr "When you use a 'submit rex command, for a submit button, you must also use an 'encrypt command. That's because 'encrypt supplies the form name the submit button will use, even if the list of items to encrypt is empty.")
	(br2)
	(pr "You can have multiple submit buttons but data are encrypted in all forms when you press any of those buttons. Only data for the corresponding form are submitted, as they should, but encrypting everything kept the implementation simpler. It also makes a user confident no data were sent unencrypted to the server.")
	(br2)
	(pr "Pressing Enter in an input box typically submits the form unencrypted. To prevent this, the browser plugin overrides onkeypress on all input boxes to force the data to be automatically encrypted.")
	(br2)
	(pr "Besides encryption logic, an 'encrypt rex command also contains logic to decrypt. Don't supply a separate 'decrypt command if you supplied an 'encrypt, but do supply to 'encrypt the list of items to decrypt.")
	(br2)
	(pr "There are four hooks inside the browser extension that define what executes when:")
	(each i (list "(srex-after-decrypt) after a specific field has been decrypted."
		      "(srex-after-decrypt-all) after all fields have been decrypted. If no data will be decrypted but you want to run code after the page loads, this is the hook to use."
		      "(srex-before-encrypt) before a specific field will be encrypted."
		      "(srex-before-encrypt-all) before all fields will be encrypted. Use this hook if you want data sent to the server to be derived from other data. You may incorrectly take encrypted data as input otherwise if you used (srex-before-encrypt).")
	      (tag (ul)
		   (tag (li) (pr i))))
	(pr "Names of input fields that will be encrypted can't contain dashes, because Javascript variable names can't contain dashes. Same goes for function parameters. It's probably related that in the 'encrypt command, using dashes in the hash keys of the enc-itemlist leads to the wordsplit_forenc path not generated. So don't use dashes in either enc-itemlist or dec-itemlist.")
	(br2)
	(pr "The user can choose to not encrypt data after a page was sent from the server, by checking a 'publish' field. This can be useful for blogging. But the publish field can't be hidden and already checked when served. Otherwise the server could trick the user to submit unencrypted data.")
	(br2)
	(pr "The 'contact command lets the server ask the user to verify the fingerprint of the public-key of a contact. An fn field in the data structure defines which user function to run that returns the public key. It may seem inconsistent the name of this function is defined there instead of the (rex) code through a hook. The reason there's no hook is that no encrypted data need to be processed: public usernames and public keys are both public. The function implementation is still provided by the user though and must return a hash table that contains key-value pairs for the ids 'userid' and 'pubkey'.")
	(br2)
	(pr "Encrypted credentials like the password and initialization vector for encrypted search, or the verified mappings between key ids to contact ids, are not transmitted through the (rex) command but as hidden text values through HTML. It'd be insecure to transmit them in (rex). The browser plugin would need to call (srex-clean-item), which inspects all (rex) input and refuses to process commands containing whitespace. Since these credentials are encrypted with PGP, which produces ciphertext containing whitespaces (e.g. \"-----BEGIN PGP MESSAGE-----\"), (rex) would refuse to process them.")
	(br2)
	(pr "It doesn't work to base64 encode the credentials either. They need to be decoded eventually, so you still need to sanitize them with (srex-clean-item) before emitting them in Javascript.")
	(br2)
	(pr "The damage from this is server code increases. Every page on the server must transmit the credentials even if nothing is encrypted or decrypted.")
	(br2)

	(tag (a id "accessing-data"))
	(prbold "Accessing Data")
	(br2)
	(pr "To make it easier to determine if one should access the value of textboxes or the innerText of an HTML element, there's an (srex-get) function with a boolean parameter signifying if an element is a textbox. An (srex-set) function does something similar when setting values.")
	(br2)
	(pr "The code verifier generates and calls a Javascript function called srex_get_wtype() to dynamically determine if one is calling an object method, accessing an attribute, or accessing a string or list as an array.")
	(br2)
	(pr "It shouldn't be nececcary to call (srex-get-wtype) in Arc manually, but the code verifier does not automatically detect the cases where it should call it. For example, it should use it in (each) loops but doesn't.")
	(br2)
	(pr "The (len) Arc function is converted to (srex-length), which dynamically determines if its parameter is a string or an object to return its length.")
	(br2)
	(pr "The reason displaying decrypted data is slow is that Chrome's localStorage.getItem() is slow to read data. It takes too long to return a 3072-bit private key. It's not the execution of decryption code for a large key in Javascript that is slow.")
	(br2)
	(pr "Hashing user passwords turned out to conflict with the browser's password manager. The manager saved the hash instead of the password. That lead to double hashing when the user visited the login page again, which failed authenticating. To address this, the hashing logic does not hash if the password length is equal to the length of the SHA512 hash (128 characters.)")
	(br2)

	(tag (a id "arc-and-arcjs"))
	(prbold "Arc and ArcJS")
	(br2)
	(pr "ArcJS does not implement some of Arc's primitives that do IO because it would be hard to define all IO functions in Javascript. For example (pr) and (tostring) are missing. (prn) works though, because it's defined internally as console.log() which always appends a newline.")
	(br2)
	(pr "ArcJS also does not attempt to load most of Arc's libraries. Some useful ones, like string.arc, have been loaded by " site-name* ". Some functions in them, like (subst), have been redefined to not use (tostring). Some simple yet necessary library functions, like (color), have been manually added. html.arc is not part of " site-name* " because I didn't need it yet.")
	(br2)
	(pr "parsecomb0.arc and fromjson.arc, which are not part of Arc, have been loaded in " site-name* " to parse the JSON sent from the server.")
	(br2)
	(pr "The Arc code sent from the server contains functions Arc doesn't implement because it made it easier to implement the verifier. For example there's a (return) function, to force the generated Javascript to issue a return.")
	(br2)

	(tag (a id "jquery-tokeninput"))
	(prbold "jQuery Tokeninput")
	(br2)
	(pr "A change was applied to jQuery Tokeninput, a library that autocompletes text. An onSearch event was added, to allow encrypting a search query before submitting it to the server. This required changing the signature of run_search() to accept as an additional parameter the search term typed in before it was encrypted. The dropdown menu didn't show up otherwise.")
	(br2)

	(tag (a id "encrypting-with-autocomplete"))
	(prbold "Encrypting with Autocomplete")
	(br2)
	(pr "In the examples, there are two fields involved when you use autocomplete to pick results from an encrypted search query and submit them back to the server. A web application can use any name it chooses for these fields, but they do serve two distinct purposes, and an app has to use both when using autocomplete:")
	(tag (ul)
	     (tag (li)
		  (pr "autocomplete-all: a list that contains a hash table for each entry that was selected from the autocomplete control. For example, if what was selected is people's contact info, each entry in the list contains key-value pairs for an id (like a username), an encrypted public key, a name, or an email address."))
	     (br)
	     (tag (li)
		  (pr "encpass: a hash table that contains key-value pairs from an id (like a username) to a hash table that contains an entry for the public key. For each user that should be able to decrypt all encrypted fields in a page, an entry is added in encpass.")))
	(pr "Why have both? Because not everything that autocompletes must be encrypted. One should be free to autocomplete and submit back to the server only unencrypted text, while still encrypting other text fields.")
	(br2)
	(pr "The two fields are connected though. When text is about to be encrypted and submitted back to the server, each list entry from autocomplete-all that contains the two fields 'id' and 'pubkey' is used to generate an entry in encpass: it is used to encrypt for that user.")
	(br2)
	(pr "Note that if what is autocompleted should be encrypted, the 'pubkey' value supplied by the server should be in an encrypted state. The autocomplete logic will decrypt it and use it.")
	(br2)

	(tag (a id "cryptographic-libraries"))
	(prbold "Cryptographic Libraries")
	(br2)
	(pr "It isn't easy gluing together the bare minimum crypto needed: public-key cryptography, with signing and verification. The tough part is making one JS library work with another. Key features are missing and examples are incomplete. You could copy code from one library to the other but it gets confusing and it's not the most secure thing to do with libraries that lack a testsuite.")
	(br2)
	(pr "(Does anyone know of a self-contained JS crypto library that offers signing and verification <i>and nothing else</i>?)")
	(br2)
	(tag (a href "https://code.google.com/p/crypto-js/") (pr "CryptoJS"))
	(pr " for example is good at symmetric cryptography but doesn't support asymmetric (public-key) cryptography. ")
	(tag (a href "https://github.com/travist/jsencrypt") (pr "JSEncrypt"))
	(pr " does, but not signing and verification. ")
	(tag (a href "http://kjur.github.io/jsrsasign/") (pr "JsRSAsign")) (pr " says it does, but it doesn't read a private key generated with JSEncrypt. You need to do more to get it to work, like possibly generate a self-signed certificate, and there's no self-contained example that shows how. Imagine how pleasing it was to find ") (tag (a href "https://github.com/wwwtyro/cryptico") (pr "Cryptico")) (pr ", which supports public-key cryptography and signing and verification, and then realize it doesn't return the private key.")
	(br2)
	(pr "Three libraries got close to offering most of public-key cryptography. ")
	(tag (a href "https://github.com/keybase/kbpgp") (pr "kbpgp"))
	(pr " seems great but key generation doesn't finish when started in a Chrome browser plugin. ") 
	(tag (a href "https://github.com/openpgpjs/openpgpjs") (pr "OpenPGP.js"))
	(pr " can't be used in a loop without extra work because it relies on an idiom called Promises that constrain control flow. The root of the problem is lack of synchronization primitives in single threaded Javascript. ")
	(tag (a href "https://github.com/digitalbazaar/forge") (pr "Forge"))
	(pr " seemed powerful, heavyweight, and I didn't attempt to use it. ")
	(br2)
	(pr "It's strange CryptoJS uses the insecure Math.random() instead of window.crypto.getRandomValues() in component/core.js:random(). It's strange five out of six authors of crypto libraries I emailed didn't respond.")
	(br2)

	(tag (a id "pgp"))
	(prbold "PGP")
	(br2)
	(pr "Out of necessity I caved. " site-name* " uses OpenPGP.js. The good part is OpenPGP.js implements key pieces: public-key crypto, with signing and verification, in an open format. The bad part is, PGP is an old format.")
	(br2)

	(pr "Most people had no Internet when the first PGP spec came out in ")
	(tag (a href "https://tools.ietf.org/html/rfc1991")
	     (pr "1996"))
	(pr ". So it's no wonder the new spec that followed in ")
	(tag (a href "https://tools.ietf.org/html/rfc2440")
	     (pr "1998"))
	(pr " missed the mark. How could it have predicted user needs in a 5 year-old new medium called Internet, when even the makers of the leading Internet application back then, Internet search, couldn't predict the needs of their app and Google was barely two months old?")
	(br2)
	(pr "Although feasible, the web of trust proposed by the 1998 spec isn't ")
	(tag (a href "https://lists.torproject.org/pipermail/tor-talk/2013-September/030235.html")
	     (pr "perfect"))
	(pr ". Two years is too short to beta test managing keyrings, or issuing revocation certificates, or attending key-signing parties when web apps aren't using PGP yet and the Internet is exploding.")
	(br2)
	(pr "You also can't search encrypted messages. All you can do is decrypt and verify them. That's not a problem if you decrypt data and store them on a laptop but it becomes a problem when most of the applications you use store their data on the Internet.")
	(br2)
	;; HKP rfc: http://tools.ietf.org/html/draft-shaw-openpgp-hkp-00
	;; http://en.wikipedia.org/wiki/Key_server_%28cryptographic%29#Problems_with_keyservers
	(pr "That sums up the big blind spot in the PGP spec. Neither spec nor software made it easy to encrypt data for the dominant way Internet apps would be delivered: in the browser. It's a mistake to cement in a spec how people may work when you don't know for sure how people want to work. That's premature optimization. It's better to wait and see first.")
	(br2)
	(pr "I don't mean to be hard on PGP. It solves a specific problem well. The full scope of the privacy problem is wider than what PGP solves though, since it includes the browser, and any solution to the privacy problem will need to adapt for the browser. It seems suspect to stay tied to how people used to think about a problem two decades ago while the problem has been evolving into something else.")
	(br2)
	(pr "As for specifics, some design flaws are around presentation. OpenPGP.js doesn't decrypt ciphertext that isn't neatly formatted. Like ciphertext in an HTML element's .innerText property which returns text as a string that doesn't contain newlines. \"Unknown ASCII armor type\" says the error message. You could read a message with .innerHTML, but then you can't embed other elements inside the one you are decrypting, and server code shrinks more when you can do that (even in Lisp). You can format ciphertext with newlines using (topgp) on the server, but it doesn't work with text input fields and it's slow in Arc. A faster but less elegant workaround is to read ciphertext from a hidden textarea.")
	(br2)
	(pr "This is also a bug when encrypting. The default ASCII armored output of PGP can't be used with input fields because once again input automatically removes newlines and carriage return. You have to use a textarea.")
	(br2)
	(pr "Except in the cases where you can't. Like when the UI you are building shouldn't use a textarea because you need everything the user types to show up in a single line and because there's no way to disable the multiline feature of textarea to do that. So you have to manually write code that copies the unencrypted data from input into the hidden textarea. See what a mess ASCII armored output is with HTML?")
	(br2)
	(pr "Another flaw is you can't show ciphertext as a single string with no whitespace. I want to do that and I can't. The ciphertext is always formatted at a fixed length of 60 characters, and it always shows the '-- BEGIN PGP MESSAGE --' banner that users shouldn't be forced to look at. This wasn't a big deal in an ASCII terminal in 1996 but it affects aesthetics in a browser today.")
	(br2)
	(pr "I don't know if it's a bug, but encrypting multiple messages in parallel and then waiting for all Promises to be fulfilled leads to some messages failing to encrypt. No error message is raised either. The result is simply null. The workaround was to call directly the 6 lines of code found in openpgp.js:signAndEncryptMessage.execute.")
	(br2)
	(pr "Signing messages before encrypting them is noticeably slower. In a standalone HTML page with OpenPGP.js, it takes an additional time of only 70ms to sign, but the same signing source code inside the plugin takes at least 1250ms. It's tempting to point the finger at Chrome since kbpgp had trouble in Chrome too, but I doubt that's the whole story. If I extract the HTML from an example, and manually add the Javascript produced by the browser plugin, execution is still slow.")
	(br2)

	(pr "One reason the plugin is slow is it takes ")
	(tag (a href "https://github.com/openpgpjs/openpgpjs/issues/353")
	     (pr "400ms"))
	(pr " for the following decrypt function to finish, which is strange, since it typically takes a standalone OpenPGP.js only 30ms to run the same function:")
	(br2)
	(pr "msg.decrypt(decrypt_privkey_openpgp);")
	(br2)
	(pr "The end result is it takes ")
	(tag (a href "slow_js.png")
	     (pr "1.3 seconds"))
	(pr " to load a static HTML page served from localhost that decrypts content. It also takes 2.5 seconds from the time a form's encrypt function is entered until the submit is issued. Overall, OpenPGP.js isn't as ")
	(tag (a href "https://github.com/dcposch/scramble/issues/25")
	     (pr "fast"))
	(pr " as it could be.")
	(br2)

	(tag (a id "verifying-contacts"))
	(prbold "Verifying Contacts")
	(br2)
	(pr "When a contact is added, the plugin saves a mapping between the key id of the contact's public key to the contact id (like a username). This is necessary because you can't reliably get the contact id otherwise. Encrypted PGP messages include ")
	(tag (a href "http://security.stackexchange.com/questions/25170/what-information-is-leaked-from-an-openpgp-encrypted-file")
	     (pr "key ids"))
	(pr ", not contact ids.")
	(br2)
	(pr "It would be nice if the problem of mapping key ids to contact ids went away. ")
	(tag (a href "http://keybase.io")
	     (pr "Keybase"))
	(pr " is working in that direction.")
	(br2)
	(pr "Mapping a key id to a contact id is not ideal because key ids are ")
	(tag (a href "https://tools.ietf.org/html/rfc4880#page-9")
	     (pr "short"))
	(pr ", which means they are prone to collisions and can be forged. But that's a problem that must be dealt with anyway, because even full fingerprints can be ")
	(tag (a href "http://seclists.org/fulldisclosure/2011/Sep/223")
	     (pr "forged"))
	(pr ", and the danger is greater after a contact has been verified. The user could be tricked to switch to a forged key for a contact it had trusted, and messages encrypted for the contact will be read by the attacker.")
	(br2)
	(pr "As a first line of defense, the user is shown the full fingerprint of a public key before verifying the contact.")
	(br2)
	(pr "Then, an additional check in the plugin refuses to re-verify a previously verified contact, to protect from overwriting an existing short key id with a different public key. If a contact's public key changed, delete the contact and verify again.")
	(br2)
	(pr "A better mapping would have been from public key fingerprints to contact ids. The problem with that is there doesn't seem to be a way in OpenPGP.js to extract a fingerprint from an encrypted message's signature. There is a .getSigningKeyIds() function but no .getSigningFingerprints().")
	(br2)
	(pr "The public key of each contact is encrypted and saved on the server. The main reason is to keep the solution elegant. To save everything encrypted on the server (except the private key and encrypted password), which allows accessing contacts from multiple machines without need for synchronization. A secondary reason is the default localStorage limit of 5MB in browsers. A 4096-bit armored key consumes roughly 2404-2432 bytes (the range is due to the length of the optional comment), which means no more than 2155-2180 contacts can be stored.")
	(br2)
	(pr "For the same reasons, the mapping between key ids to contact ids is encrypted and stored on the server.")
	(br2)
	;; don't do this; it complicates verification
;; 	(pr "When displaying whether a contact is verified, the display message is positioned relatively to the field used as the user id. Else an attacker could display multiple contacts on a page and confuse the user about which contacts are verified and which aren't.")
;; 	(br2)

	(tag (a id "verifying-content"))
	(prbold "Verifying Content")
	(br2)
	(pr "With a desktop app that stores content locally, unless there are continual software updates, you always know content you wrote was written by you. Not so with a web app. You can't tell if the content was forged on the server unless you examine the signature.")
	(br2)
	(pr "For this reason the browser plugin always verifies the author of all content by looking at the content's signature.")
	(br2)
	(pr "If a checkbox that reads \"show authors\" is checked by the user, the browser plugin \"always\" displays the author's user id on the left-hand-side of the content.")
	(br2)
	(pr "With two exceptions. Number one, if all content in a page was written by you, no confirmation is shown. Showing author user ids is necessary when multiple authors share content but the confirmation isn't visually pleasing. So when you are the only author, the plugin doesn't show the confirmation message (but still verifies signatures).")
	(br2)
	(pr "The final decision to display author ids isn't left to the server, because a bad server that forged content could easily display your own user id to trick you into thinking the content you are looking at is yours, even though the content was slightly altered by someone else. Say, if it was written by one of your verified but forged contacts. This still isn't too strong of a protection. A malicious server could display author user ids on the right-hand-side of the content, while author ids are also displayed on the left-hand-side, and confuse the user.")
	(br2)
	(pr "But the user has at least some control for viewing author user ids. They can visually see what changes on the screen when they check \"show authors\". The user wouldn't have this option if it was fully controlled by the server.")
	(br2)
	(pr "Exception number two is an application wanting to visually conceal the identity of all authors, to provide visual anonymity. The browser plugin still verifies signatures but doesn't display author user ids. It only displays an alert that author information will not be shown. It's possible to extend this feature to also indicate if content was written by the current user or not, but I didn't think it'd be useful in practice.")
	;; well, I don't know really.
	;; Content viewed anonymously tends to exclude content written by the current user.
	(br2)

	(tag (a id "encrypted-search"))
	(prbold "Encrypted Search")
	(br2)
	(pr "There is a small presentation bug in the ")
	(tag (a href "http://www.cs.berkeley.edu/~dawnsong/papers/se.pdf")
	     (pr "paper"))
	(pr " that showed how encrypted search can be practical. In Figure 3, F_k_i(S_i) should be F_k_i(L_i). That would match the explanation in Section 4.4, paragraph 2, which is correct: \"Alice should generate k_i as k_i := f_k'(L_i)\".")
	(br2)
	(pr "There is no logic in " site-name* " that implements encrypted indexes. If all that users will need to encrypt for search is a list of contacts, that isn't a huge list, and encrypted search can be parallelized on the server. But there are fair odds users will need encrypted search for content as well. As the paper authors noted, efficiently updating an encrypted search index is an interesting research question.")
	(br2)

	(tag (a id "limitations"))
	(prbold "Limitations")
	(br2)
	(pr "Arc code sent to the browser plugin for execution currently can't define macros. I'm not sure it's wise to support them, because to do that the verifier must also become an evaluator, which increases the odds of introducing a bug that leads to a security breach. I didn't need macros anyway. You could generate macroexpanded source on the server and send that instead.")
	(br2)
	(pr "Chrome's autocomplete sets the background of a textbox to a light yellow, which overrides the red background set by the browser extension for fields that won't be encrypted.")
	(br2)
	(pr "I don't know why but attempting to disable double form submission didn't work.")
	(br2)
	(pr "The reason key pairs are 3072 bits and not 4096 or higher is that on my machine Safari is often unable to generate a 4096-bit key in under 5 minutes. You would think this wouldn't matter, since the browser plugin is written for Chrome. But it matters when reusing the key generation logic on a trusted server.")
	(br2)
	(pr "When running on a trusted server, the dialog that asks to verify the public-key fingerprint of a contact isn't displayed correctly in Firefox and Safari because these browsers don't ")
	(tag (a href "https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dialog")
	     (pr "support"))
	(pr " it yet. It's displayed at the bottom left of the screen and pressing on either 'Yes' or 'No' in that dialog doesn't work. Firefox doesn't give an error at all and Safari gives the error:")
	(br2)
	(pr "[Error] TypeError: undefined is not a function (evaluating 'dialog_gs12.close()')")
	(br2)
	(pr "Encrypted search words are too short. They are 32 bytes in length, since they are constructed from the output of CryptoJS.AES.encrypt() which is 256 bits. Can the length be increased? People who understand encryption well should better have a look at this.")
	(br 8)
	(hline)))

(defop faq req
  (page (backlink "/") site-name* nil
	(prbold "Frequently Asked Questions")
	(br 4)

	(prbold "Does the demo require installing a Chrome extension?")
	(br2)
	(pr "Yes.")
	(br2)

	(prbold "Will this work only on Chrome?")
	(br2)
	(pr "The approach is general that it could be ported to any browser. The current implementation works only on Chrome.")
	(br2)

	(prbold "Is this open-source?")
	(br2)
	(pr "Yes.")
	(br2)

	(prbold "Is this a browser extension or a browser?")
	(br2)
	(pr "This is a browser extension.")
	(br2)

	(prbold "What data does it encrypt?")
	(br2)
	(pr "It encrypts data from textboxes and input fields in web forms.")
	(br2)
	(pr "It decrypts data from any HTML element, like a div or a span, not only textboxes and input fields.")
	(br2)

	(prbold "What does the code verifier do?")
	(br2)
	(pr "It reads Arc code sent by the server and does all sorts of things with it internally, but the main two are:") (br)
	(tag (ul)
	     (tag (li)
		  (pr "it checks all data access operations. If the server sends code that would steal unencrypted or decrypted data, the code verifier forbids that code from running."))
	     (br)
	     (tag (li)
		  (pr "it automatically generates code that automatically decrypts data when a page loads. And it also automatically generates code that automatically encrypts data when a web form is submitted.")
		  (br2)
		  (pr "The user does no manual work to encrypt or decrypt.")))

	(prbold "How does using an extra language help?")
	(br2)
	(pr "It makes it easy to write the code verifier. Parsing Lisp is easier than parsing Javascript; you just call (read). And verifying a smaller language grammar is less work.")
	(br2)

	(prbold "Does the code verifier run on the client?")
	(br2)
	(pr "Yes. It runs inside the browser extension, on the client.")
	(br2)

	(prbold "How does the browser extension get updates?")
	(br2)
	(pr "There's nothing in " site-name* " that updates the browser extension. If you download the browser extension from the Chrome store, then naturally the browser extension is updated from the Chrome store. The goal is to not need to update the browser extension though.")
	(br2)
	(pr "When I say " site-name* " helps update web apps, I mean it's easy to change in the web app the fields that should be encrypted <i>without</i> needing the user to update the browser extension.")
	(br2)

	(prbold "Can the government \"persuade\" you to update the extension in a way that compromises the verifier?")
	(br2)
	(pr "Yes. This is the same problem browsers, operating systems, and generally both software and hardware have.")
	(br2)

	(prbold "If the server gets hacked can an attacker decrypt data?")
	(br2)
	(pr "Unless the browser extension itself has security flaws, no.")
	(br2)

	(prbold "If I enter data in a web form, I typically trust the server. Why would I want to give a server encrypted data if I'm the only one who can decrypt the data?")
	(br2)
	(pr "Because a server can get hacked even if you trust it.")
	(br2)
	(pr "You are also not the only one who may give a server encrypted data. Anyone who wants to send you a message may give a server encrypted data. A friend who doesn't want to trust the server can encrypt a message intended for you, store it on the server, and have you download and decrypt it. Like web-based encrypted email.")
	(br2)

	(prbold "The halting problem is going to prevent your verifier from being reliable. It will produce either false negatives or false positives (or it will hang).")
	(br2)
	(pr "Some of this is true and some of it seems unlikely. The code verifier is not an interpreter and therefore doesn't evaluate code. It neither needs to nor attempts to determine if a program will halt.")
	(br2)
	(pr "What the code verifier does is transform code. It determines if a program reads or writes data in forms, and generates new code for reading and writing in forms to happen safely. It also checks for unsafe data access operations during this transformation, but it never evaluates the code to do that.")
	(br2)
	(pr "Unless there is a design flaw or bug in the code verifier, there is no possibility for producing false negatives: you can't execute evil code that steals decrypted or unencrypted data. There is no theoretical limitation like the halting problem that makes this type of safe code execution impossible.")
	(br2)
	(pr "There is also ")
	(tag (a href "https://en.wikipedia.org/wiki/Proof-carrying_code")
	     (pr "proof"))
	(pr " more general safe code execution is possible. But the code verifier does nothing this complicated.")
	(br2)
	(pr "False positives are possible though, because the code verifier's checks are overprotective. This is a limitation in the implementation. Instead of keeping a separate symbol table per basic block or per function, a single symbol table is kept over the whole program. If the plugin turns out to be so overprotective in practice that it becomes unusable, then this limitation can be corrected.")
	(br2)

	(prbold "The halting problem is more general than that. A corollary of the halting problem is that it is not generally possible to determine a large variety of useful properties of a program. You should read ")
	(tag (a href "http://matt.might.net/articles/intro-static-analysis/")
	     (prbold "this"))
	(prbold ".")
	(br2)
	(pr "This doesn't affect " site-name* ".")
	(br2)
	(pr "The link above assumes a program only has a buffer overflow if and only if it reaches a buffer overflow when executed. This excludes programs that contain buffer overflow code but never reach that code because of, say, an infinite loop before the code.")
	(br2)
	(pr "This means the claim in the link \"Guaranteeing the safety of array bounds and deciding program termination are equivalent in difficulty\" is valid only under a narrower definition and phrased with broad language. The intended claim is \"Guaranteeing a buffer overflow will be reached and deciding program termination are equivalent in difficulty\". That's a valid claim.")
	(br2)
	(pr "You cannot guarantee a program will reach a buffer overflow.")
	(br2)
	(pr "You can guarantee a program will never reach a buffer overflow.")
	(br2)
	(pr "If we try to match the definition intended in the link above, " site-name* " can't guarantee that visiting a web page will lead to data in the page to be encrypted if the server did not flag the data to be encrypted or if the program hits something like an infinite loop. And " site-name* " can't guarantee the data will be decrypted if the server did not flag the data to be decrypted or if the program hits something like an infinite loop. The server has to cooperate.")
	(br2)
	(pr "But if the server doesn't cooperate, " site-name* " still alerts the user. If the server doesn't flag the data to be encrypted the input fields change to have a red background and an alert pops up on the page.")

	(br 8)
	(hline)))

(deftem profile
 user nil
 loginpassword nil
 encrypted nil
 stripe-cus-id nil
 pubkey nil           ; the public key
 searchmperc nil      ; the m value in encrypted search queries
 searchmaxwords nil   ; the number of words in the largest encrypted search document
 searchmaxnames nil   ; the number of words in the largest contact name
 encverifiedkeyids nil; the public key ids we approved as verified; encrypted
 encsearchinfo nil    ; the encrypted search info: salt, pass, iv
)

(def save-prof (prof) (save-table prof (str appdir* app-name* "/" prof!user "/id")))
(def prof (user) (errsafe:temload 'profile (str appdir* app-name* "/" user "/id")))
(def add-prof (user)
  (withs (newprof (inst 'profile 'user user 'searchmaxwords 0))
	 (ensure-dir (str appdir* app-name* "/" user))
	 (ensure-dir (str appdir* app-name* "/" user "/data"))
	 (ensure-dir (str appdir* app-name* "/" user "/contacts"))
	 (save-prof newprof)
	 user))

(deftem data
 user nil
 id nil
 kind nil
 encpass nil
 seconds nil
 data nil
 encperm nil ; the encrypted permutation of the words in the data
 encspace nil; the encrypted non-words (whitespace, punctuation) in the data
 enccase nil ; the encrypted letter casing in each word
)

(def save-data (data) (save-table data (str appdir* app-name* "/" data!user "/data/" data!id)))
(def data (user id) (errsafe:temload 'data (str appdir* app-name* "/" user "/data/" id)))
(def add-data(user kind encpass data encperm encspace enccase)
  (withs (id (rand-string 14)
	  newdata (inst 'data 'user user 'kind kind 'encpass encpass 'id id
			'data data 'seconds (seconds) 'encperm encperm
			'encspace encspace 'enccase enccase))
	 (save-data newdata)
	 id))
(def alldata (user)
  (withs (all (table)
	  d (str appdir* app-name* "/" user "/data"))
	 (each id (dir d)
	       (= (all id) (data user id)))
	 all))
(def sort-data (datatab)
  (sort (fn (x y)
	    (withs (one (cadr x) two (cadr y))
		   (> one!seconds two!seconds))) (tablist datatab)))

(deftem contact
 id nil     
 user nil     ; profile user's username
 username nil ; contact's username
 pubkey nil
 encpass nil
 kind nil        ; added this only because it allows us to reuse data-search
 data nil        ; the contact name
 encpermname nil ; the encrypted permutation of the words in the data
 encspacename nil; the encrypted non-words (whitespace, punctuation) in the data
 enccasename nil ; the encrypted letter casing in each word
)

(def save-contact (contact) (save-table contact (str appdir* app-name* "/" contact!user "/contacts/" contact!id)))
(def contact (user id) (errsafe:temload 'contact (str appdir* app-name* "/" user "/contacts/" id)))
(def add-contact (username user pubkey data encpermname encspacename enccasename)
  (withs (id (rand-string 8) newcontact nil)
	 (= newcontact (inst 'contact 'id id 'user user 'username username
			     'pubkey pubkey 'data data
			     'encpermname encpermname 'encspacename encspacename 'enccasename enccasename))
	 (save-contact newcontact)
	 id))
; This could be more efficient. Ideally there should be a mapping
; between contact ids and usernames
(def del-contact (user contact-username)
  (each id (dir (str appdir* app-name* "/" user "/contacts"))
	(withs (c (contact user id))
	       (if (is c!username contact-username)
		   (tostring (system (str "rm -f " appdir* app-name* "/" user "/contacts/" id)))))))
(def allcontacts (user)
  (withs (all (table)
	  d (str appdir* app-name* "/" user "/contacts"))
	 (each id (dir d)
	       (= (all id) (contact user id)))
	 all))

(def anon-user (req)
  (aif (get-user req) (logout-user it))
  (withs (user (rand-string 14)
	  cookie (cook-user user))
	 (= (logins* user) req!ip)
	 (prcookie cookie)
	 (prn)
	 user))

(defop-raw demo (str req)
  (w/stdout str
    (withs (olduser (get-user req)
	    newuser (anon-user req))
	   (add-prof newuser)
	   ;; add sample existing data; they will be encrypted when
	   ;; the plugin is installed, but won't be searchable
	   (add-data newuser "blog" nil "This is a blogging test. It uses two sentences." nil nil nil)
	   (add-data newuser "story" nil "The Tortoise and the Hare.

There once lived a very proud hare. He loved to stroll around the warren with his nose held high in the air. Everyone knew that the hare thought he was the best hare there ever was." nil nil nil)
	   ;; add data that will be encrypted in a way that they can
	   ;; be searched (it's a contact); this happens when the
	   ;; plugin is installed
	   ;; fixme: should retest encrypting contacts on plugin installation
	   ;;(add-contact newuser newuser nil "Oliver Twist" nil nil nil)
	   ;;(add-data newuser "contact" nil "Oliver Twist" nil nil nil)
	   (prn)
	   (demo-core newuser))))

(def demo-core (user)
  (page nil site-name* nil
	(pr "Because this is only a demo, the demo server uses HTTP instead of HTTPS. But make sure you use HTTPS when deploying the demo in production.")
	(br2)
	(pr "<i>(You may need to enable \"Developer mode\" in Chrome to run this.<br><br>Somewhere in the web application you can instruct the user to install this browser extension. You can say something like the following.)</i>")
	(br 4)
	(pr "Hello, you need to install this Chrome browser extension for the app to protect your data better.")
	(br2)
	(pr "This will make sure your data are encrypted safely in your browser, and can't be decrypted if someone gets a copy of your data from the servers: they won't be able to decrypt them. The encryption keys are sealed safe inside the browser extension and can't be stolen.")
	(br2)
	(pr "To install the browser extension please:")
	(tag (ol)
	     (tag (li)
		  (pr "Click ")
		  (tag (a href (str "/joan" version* "-browser-extension.crx"))
		       (pr "here"))
		  (pr " to download the browser extension.")
		  (br2))
	     (tag (li)
		  (pr "then find in Chrome the <b>Window</b> menu and click on <b>Extensions</b>.")
		  (br2))
	     (tag (li)
		  (pr "<b>drag-and-drop</b> the extension from your Downloads into the Extensions page in Chrome, and select <b>Add</b> in the window that pops up.")
		  (br2)
		  (pr "This will open a new page in Chrome. Follow the instructions in that new page, and when you see a message that says \"Success\" then you can close this page that you are reading now.")
		  ;;(br2)
		  ))
	(br 8)
	(pr "<i>(By the way, if you are developing " site-name* " and just restarted the webserver, refresh this page to store a new cookie in the browser.)</i>")
	(hline)
	))

(def node ()
  (if (is (os) "Darwin")
      "/usr/local/bin/node"
      "/usr/bin/nodejs"))

(def tool (name) (isnt (tostring (system (str "which " name))) ""))
(def main ()
  (ensure-dir (str appdir* app-name*))
  (each cmd (list "sed" "python" "base64" "xargs" "xxd"
		  (node) ;; needed only for an example: to simulate a
			 ;; man-in-the-middle attack in
			 ;; joan-example-failed-signature.arc
		  )
	(if (no (tool cmd))
	    (do (prn "Missing command " cmd ".")
		(quit))))
  (asv 1412))

;; This can be used to return all data elements to have them encrypted
(defop api-v1-items req
  (withs (user (get-user req) p (prof user))
	 (pr (api-items-core user p (is (arg req "encrypted") "yes")))))

(def api-items-core (user p encrypted)
  (withs (alldata (alldata user) counter 0 out "{ \"data\": [")
	 (maptable (fn (k v)
		       (if (or (and (no encrypted) (no v!encpass))
			       (and encrypted v!encpass))
			   (do (if (> counter 0)
				   (= out (str out ",
")))
			       (= out (str out (api-items-data v)))
			       (++ counter)))) alldata)
	 (= out (str out "]"))
	 (= out (str out ", \"id\": \"" p!user "\"}"))
	 out))

(def api-rex (incommands incode)
  (tojson (obj commands incommands code incode)))
(def api-items-data (data)
  (str "{
  \"user\": " (aif data!user (json-escape it) "null") ",
  \"id\": " (aif data!id (json-escape it) "null") ",
  \"kind\": " (aif data!kind (json-escape it) "null") ",
  \"encpass\": " (aif data!encpass (json-escape it) "null") ",
  \"data\": " (aif data!data (json-escape it) "null")
  (if (is data!kind "contact")
      ;; this is searchable
      (str ",
  \"searchable\": " (tojson-list (list "data"))))
  "}"))

;; This can be used to save on the server the data that were encrypted
;; in the browser extension
(defop api-v1-encrypt req
  (withs (user (get-user req))
	 (withs (kind (arg req "kind") id (arg req "id") data (data user id))
		;; do something with this, like save somewhere the
		;; data element of this kind with this id
		(if (and (is (arg req "encrypted") "yes") data)
		    (do (= data!data (arg req "data"))
			(= data!encpass (arg req "encpass"))
			(save-data data))
		    (and (is (arg req "encrypted") "no") data)
		    (do (= data!data (arg req "data"))
			(= data!encpass nil)
			(save-data data)))
		)))

;; This is used to mark a user profile as encrypted
(defop api-v1-encrypt-done req
  (withs (encrypted (arg req "encrypted") from (arg req "from")
          user (get-user req) p (prof user))
	 (if (is encrypted "no")
	     (do (= p!encrypted nil)
		 (save-prof p))
	     (is encrypted "yes")
	     (do (= p!encrypted t)
		 (save-prof p)))))

(def encrypt-data-page-submit-fn (user keybits saltbits passkeybits passiters searchsaltbits searchpassbits searchmperc searchivbits)
  (fn (_)
      (withs (pluginuniq (rand-string 14) ; a unique browser extension installation id
	      plugincont "continue-here"  ; name of a function where to continue execution
	     ;; lookup the profile of this user, save in it the
	     ;; keybits saltbits passkeybits passiters searchsaltbits
	     ;; searchpassbits searchmperc searchivbits, and also
	     ;; create a plugin id and a continuation id and save them
	     ;; in the profile
             newprof (prof user))
	     (= newprof!searchmperc (coerce searchmperc 'num))
	     (save-prof newprof)
	     (prn header*)
	     (prn)
	     (prn (str "{ \"pluginuniq\": \"" pluginuniq "\",
\"plugincont\": \"" plugincont "\" }"))
      )))

;; This is used to return an fnid that will be called after encryption
;; finishes. It assumes there is a cookie for this user and a profile
;; created for it
(defop api-v1-encrypt-data-page req
  (pr (str "{ \"fnid\": \"" (fnid (encrypt-data-page-submit-fn (get-user req) (arg req "keybits") (arg req "saltbits") (arg req "passkeybits") (arg req "passiters") (arg req "searchsaltbits") (arg req "searchpassbits") (arg req "searchmperc") (arg req "searchivbits"))) "\", \"id\": \"" (get-user req) "\"}")))

(defop continue-here req
  (page nil site-name* nil
	(pr "<i>(Your web application would continue executing from this page.)</i>")
	(br 4)
	(prbold "Success")
	(br2)
	(pr "The browser extension has been installed successfully. Click ")
	(tag (a href "/example-text")
	     (pr "here"))
	(pr " to see how it encrypts data when you type text.")	
	))



;; Need html properties Arc doesn't offer out of the box
(attribute input      id             opstring)
(attribute input      checked        opstring)
(attribute input      autofocus      opstring)
(attribute input      maxlength      opnum)
(attribute input      placeholder    opstring)
(attribute textarea   id             opstring)
(attribute textarea   autofocus      opstring)
(attribute textarea   readonly       opstring)
(attribute form       id             opstring)
(attribute form       name           opstring)
(attribute div        class          opstring)

(def nbsp ((o n 1)) (repeat n (pr "&nbsp;")))

(def checkbox (n (o c))
     (gentag input type 'checkbox id n name n checked c))

; Make a form that has a name
(mac sform-core (fnurl n f . body)
  (w/uniq ga
    `(tag (form method 'post action ,fnurl id ,n name ,n)
       (fnid-field (fnid (fn (,ga)
                           (prn)
                           (,f ,ga))))
       ,@body)))

(mac sform (n f . body)
     `(sform-core fnurl* ,n ,f ,@body))

(def rex (incommands (o incode nil))
  (gentag input type "hidden" id "rex" name "rex"
	  value (tojson (obj commands incommands code incode))))

(load "fromjson.arc")
(def sysjson (cmd) (fromjson (tostring (system cmd))))

(defop examples req
  (page nil site-name* nil
	(encverif (get-user req))
	(pr "Click the links to run examples that use the browser extension.")
	(br 4)
	(prbold "Safety")
	(br2)
	(tag (a href "/example-nojs")
	     (pr "Do NOT execute Javascript code sent from the server."))
	(br2)
	(tag (a href "/example-noencrypt-alert")
	     (pr "Show in red input fields that won't be encrypted."))
	(br2)
	(tag (a href "/example-noeval")
	     (pr "Do NOT execute Arc code containing eval()."))
	(br2)
	(tag (a href "/example-nodom")
	     (pr "Do NOT allow direct access to the DOM or global " site-name* " variables."))
	(br2)
	(tag (a href "/example-failed-signature")
	     (pr "Show in red decrypted data fields that fail signature verification."))
	(br2)
	(tag (a href "/example-noonclick")
	     (pr "Do NOT allow overriding the onclick event of a form."))
	
	(br 4)
	(prbold "Features")
	(br2)
	(tag (a href "/example-text")
	     (pr "Encrypt and sign in the browser before sending to the server."))
	(br2)
	(tag (a href "/example-code-from-server")
	     (pr "Modify data before encrypting."))
	(br2)
	(tag (a href "/example-compute-from-decrypted")
	     (pr "Modify data after decrypting."))
	(br2)
	(tag (a href "/example-nodecrypt")
	     (pr "Send unencrypted (new) data, but don't send decrypted (old) data."))
	(br2)
	(tag (a href "/example-date")
	     (pr "Create a new Javascript object, in Arc."))
	(br2)
	(tag (a href "/example-multiple-forms")
	     (pr "Have more than one form in a page."))
	(br2)
	(tag (a href "/example-payments")
	     (pr "Process payments with Stripe."))
	(br2)
	(tag (a href "/example-login")
	     (pr "Hash a user password for login."))
	(br2)
	(tag (a href "/example-cookies")
	     (pr "Delete a cookie."))
	(br2)
	(tag (a href "/example-autocomplete-server")
	     (pr "Autocomplete from server."))
	(br2)
	(tag (a href "/example-autocomplete-server-multiple")
	     (pr "Autocomplete multiple items."))
	(br2)
	(tag (a href "/example-encrypted-search")
	     (pr "Search encrypted data."))
	(br2)
	(tag (a href "/example-autocomplete-encrypted-search")
	     (pr "Autocomplete multiple items from encrypted data."))
	(br2)
;; not ready yet
;; 	(tag (a href "/example-read-email")
;; 	     (pr "Read email."))
;; 	(br2)
	(tag (a href "/example-contact-add")
	     (pr "Add a contact."))
	(br2)
	(tag (a href "/example-contact-delete")
	     (pr "Delete a contact."))
	(br2)
	(tag (a href "/example-encrypt-for-contact-with-autocomplete")
	     (pr "Encrypt for an encrypted contact, chosen using autocomplete."))
	(br 4)
	(pr "You can see all the data you saved so far by clicking ")
	(tag (a href "/data")
	     (pr "here"))
	(pr " and all contacts ")
	(tag (a href "/contacts")
	     (pr "here"))
	(pr ".")
	(br 8)
	(hline)
	))

(def click-source (file)
  (pr "Click ")
  (tag (a href (str "/" file))
       (pr "here"))
  (pr " to see the server source code for this example."))

; 'user could be some other field from the profile, like a public identifier
(mac encpass-write (oldencpass p) 
     (withs (temp (uniq))
	    `(withs (,temp ,oldencpass)
		    (= ,oldencpass (table))
		    (= (,oldencpass (,p 'user)) ,temp)
		    (= ,oldencpass (tojson ,oldencpass)))))
     
(mac encpass-read (newencpass p)     
     `(do (= ,newencpass (fromjson ,newencpass))
	  ;; this returns the encpass of only the current user; could improve
;; 	  (= ,newencpass (,newencpass (,p 'user)))
;; 	  (= ,newencpass (,newencpass "encpass"))
	  ))

;; (def encpass-val (encpass user)
;;   (if (and encpass (encpass user))
;;       (encpass user)))

(def encpass-val (encpass profuniq)
  (if (and encpass (encpass profuniq))
      ((encpass profuniq) "encpass")))

(def encverif (user)
  (withs (p (prof user) ids "")
	 ;; the if is here to list the /examples without installing the plugin
	 (if (and p p!encverifiedkeyids)
	     (= ids p!encverifiedkeyids))
	 (gentag input type "hidden" id "encverifiedkeyids" name "encverifiedkeyids" value ids)))

(def encsearchinfo (user)
  (withs (p (prof user))
	 (gentag input type "hidden" id "encsearchinfo" name "encsearchinfo" value p!encsearchinfo)))

(def showauthors (user state)
  (withs (p (prof user))
	 (if p!encrypted
	     (menu nil
		   (do (checkbox "showauthors" state)
		       (plgray " show authors"))))))
  
(def topgp (s) ;; make .innerText return the expected PGP format
  (withs (multi (strsplit s "\n") ret "" counter 0 numitems (len multi))
         (each item multi
               ;; must also get rid of carriage return, produced by the version string, e.g.:
               ;; Version: OpenPGP.js v0.10.1
               (withs (itemtouse ((strsplit item "\r") 0))
                      (if (> counter 0)
                          (= ret (str ret "<br>
")))
                      (= ret (str ret itemtouse))
                      (++ counter)))
         ret))

(load "joan-example-text.arc")

(def example-text-view-encrypted (user id)
  (page (backlink "/examples") site-name* nil
	(withs (data (data user id))
	       (pr "Here is what was submitted. Click ")
	       (w/link (example-text-view-decrypted user data) (pr "here"))
	       (pr " to see it decrypted.")
	       (br2)
	       (encverif user)
	       (tabwrap
		(pr (topgp data!data)))
	       (br 8)
	       (hline)
	       )))

(def example-text-view-decrypted (user data)
  (page (backlink "/examples") site-name* nil
	(withs (dec-itemlist nil)
	       ;; must provide the password and id of the item to decrypt
	       (push (obj encpass (encpass-val data!encpass user)
			  id (str "entry_1")) dec-itemlist)
	       (rex (list (list 'decrypt
				(obj kind "blog-entry"
				     dec-itemlist dec-itemlist))))
	       (pr "The decrypted text is:")
	       (br2)
	       (encverif user)
	       (tabwrap
		(tag (span id (str "entry_1"))
		     (pr (topgp data!data))))
	       (br 8)
	       (hline)
	       )))

(def contacts-show (user)
  (withs (verify-list nil dec-itemlist nil counter 0)
	 (maptable (fn (k v)
		       (push (obj id (str "keyid_" counter)
				  istextarea nil
				  showin (str "show_verified_" counter)
				  kind "word")
			     verify-list)
		       (if v!data
			   (do (push (obj encpass (encpass-val v!encpass user)
					  id (str "nameid_" counter)
					  fromsearch t) dec-itemlist)))
		       (++ counter)) (allcontacts user))
	 (rex (list (list 'verify
			  (obj verify-list verify-list))
		    (list 'decrypt
			  (obj pubuserid user
			       dec-itemlist dec-itemlist))
		    )
	      nil)
	 (pr "Your contacts are: ") (br2)
	 (= counter 0)
	 (tag (table)
	      (maptable (fn (k v)
			    (tag (tr)
				 (tag (td)
				      (tag (span id (str "keyid_" counter))
					   (pr v!username))
				      (br)
				      (if v!data
					  (tag (span id (str "nameid_" counter))
					       (pr (tojson (obj data v!data
								permutation v!encpermname
								space v!encspacename
								case v!enccasename))))))
				 (tag (td)
				      (tag (span id (str "show_verified_" counter)))))
			    (++ counter)
			    ) (allcontacts user)))
	 (br 8)
	 (hline)))

(defop contacts req
  (page (backlink "/examples") site-name* nil
	(encverif (get-user req))
	(encsearchinfo (get-user req))
	(contacts-show (get-user req))))

(def example-contacts-show (user)
  (page (backlink "/examples") site-name* nil
	(pr "Success.")
	(br2)
	(encverif user)
	(encsearchinfo user)
	(contacts-show user)))

;; using a file as a template, replace a bunch of variables in it and
;; then read it
(def freadrep (file replist)
  (withs (newfile (str tmpdir* (rand-string 8)) repcmds "" out nil)
	 (each r replist
	       (withs (item-find (r 0) item-replace (r 1))
		      (= repcmds (str repcmds "sed -i -e 's/" item-find "/" item-replace "/g' " newfile ";"))))
	 (system (str "cp " file " " newfile ";" repcmds))
	 (= out (fread newfile))
	 (system (str "rm " newfile))
	 out))

(def lower (s)
  (withs (out nil)
	 (each c s
	       (withs (cval (coerce c 'int))
		      (if (and (>= cval 65) (<= cval 90))
			  (= out (str out (coerce (+ cval 32) 'char)))
			  (= out (str out c)))))
	 out))

(defop data req
  (page (backlink "/examples") site-name* nil
	(withs (user (get-user req))
	       (pr "This page shows all the data saved by you so far. The latest data are shown first. You are user " user ".")
	       (br2)
	       (encverif (get-user req))
	       (tabwrap
		(tag (ol)
		     (each d (sort-data (alldata user))
			   (tag (li)
				(pr d))
			   (br)))))))

(def os ()
  (tostring (system "uname | xargs echo -n")))
;; who said Arc doesn't have libraries?
(def base64tohex (i)
  ;;(fwrite (str "/tmp/base64tohex_" (seconds) "_in") (tostring (pr i)))
  (withs (f (str tmpdir* (rand-string 8)) out nil
	  decode-param "-d")
	 (if (is (os) "Darwin")     ; base64 -d on Linux
	     (= decode-param "-D")) ; base64 -D on Mac
	 (fwrite f i)
	 ;; the -c 1000000 allows printing in a single large string
	 (= out (tostring (system (str "(cat " f " |base64 " decode-param " |xxd -p -c 1000000 |xargs printf); rm -f " f))))
	 ;;(fwrite (str "/tmp/base64tohex_" (seconds) "_out") (tostring (pr out)))
	 out))

(= hex2bin* (table) bin2hex* (table))
(def init-hex2bin-bin2hex () ;; disgusted? this is 49x faster than calling Python!
  (withs (vals (list (list "0" "0000")
		     (list "1" "0001")
		     (list "2" "0010")
		     (list "3" "0011")
		     (list "4" "0100")
		     (list "5" "0101")
		     (list "6" "0110")
		     (list "7" "0111")
		     (list "8" "1000")
		     (list "9" "1001")
		     (list "a" "1010")
		     (list "b" "1011")
		     (list "c" "1100")
		     (list "d" "1101")
		     (list "e" "1110")
		     (list "f" "1111")))
	 (each item vals
	       (withs (h (car item) b (cadr item))
		      (= (hex2bin* h) b)
		      (= (bin2hex* b) h)))))
(init-hex2bin-bin2hex)

(def xorhex-charbin (cb1 cb2 thelen)
  (withs (counter 0 ret "")
	 (while (< counter thelen)
		(if (is (cb1 counter) (cb2 counter))
		    (= ret (str ret "0"))
		    (= ret (str ret "1")))
		(++ counter))
	 ret))

(def xorhex (str1 str2)  
  (withs (l (len str1) counter 0 ret "")
	 (while (< counter l)
		(= ret (str ret (bin2hex*
				 (xorhex-charbin (hex2bin* (str (str1 counter)))
						 (hex2bin* (str (str2 counter)))
						 4))))
		(++ counter))
	 ret))

(def pickm (word mperc)
  (withs (mperc-touse mperc)
	 ;; the user profile is lost the way I restart the webserver
	 ;; when debugging, so mperc is nil; address this
	 ;; (forgive me, I was hacking)
	 (if (no mperc-touse)
	     (= mperc-touse 0.25))
	 ;; what is returned should be an int for cut to work
	 (coerce (* mperc-touse (len word)) 'int)))

;; perform sequential scan over data to process encrypted search queries
;; the query that comes in uses keys in double quotes, because that's how
;; it was submitted by the browser
(def data-search (kind query user searchmperc data)
  ;;(fwrite "/tmp/query" (tostring (pr query)))
  (withs (ret (table) itemcounter 0)
	 (maptable
	  (fn (k d)
	      (if (is d!kind kind)
		  (each item d!data
			(if (no (ret d!id)) ;; stop if there's a match already
			;;(fwrite "/tmp/inhex" (base64tohex item))
			;; first, get the data in hex and compute n and m
			(withs (item_inhex (base64tohex item)
				m (pickm item_inhex searchmperc)
				n (len item_inhex)
				counter 0)
			       ;;(fwrite "/tmp/m" (tostring (pr m)))
			       ;;(fwrite "/tmp/n" (tostring (pr n)))
			       (each w (query "words")
				     ;; second, xor the search word with the data,
				     ;; and get the last m bits
				     ;;(fwrite "/tmp/w" w)
				     ;;(fwrite "/tmp/keys" (tostring (pr (query "keys"))))
				     ;;(fwrite "/tmp/fkey" (tostring (pr ((query "keys") counter))))
				     (withs (keysforthisword (query "keys")
					     keyscounter 0
					     keys (keysforthisword counter))
					    (each f_key keys
						  ;;(fwrite (str "/tmp/key_" counter) f_key)
						  (withs (dexor (xorhex item_inhex w)
							  dexor_lastm (cut dexor (- n m) n)
							  dexor_firstnm (cut dexor 0 (- n m))
							  preenc_firstnm (cut w 0 (- n m)))
							 ;; third, check if dexor_lastm matches the f_key
							 (if (is f_key dexor_lastm)
							     (= (ret d!id) d)))
						  (++ keyscounter))))
			       (++ counter)))))
	      (++ itemcounter))
	  data)
	 ;;(fwrite (str "/tmp/ret_" (seconds)) (tostring (pr ret)))
	 ret))

(def contact-search (kind query user searchmperc data)
  (data-search kind query user searchmperc data))

(def showsign (show checked)
  (tag (span style (if show
		       "display: inline-block"
		       "display:none"))
       (checkbox "sign" checked)
       (pr " sign")))

;; safety
(load "joan-example-nojs.arc")
(load "joan-example-noencrypt-alert.arc")
(load "joan-example-noeval.arc")
(load "joan-example-nodom.arc")
(load "joan-example-failed-signature.arc")
(load "joan-example-noonclick.arc")

;; features
(load "joan-example-code-from-server.arc")
(load "joan-example-compute-from-decrypted.arc")
(load "joan-example-nodecrypt.arc")
(load "joan-example-date.arc")
(load "joan-example-multiple-forms.arc")
(load "joan-example-payments.arc")
(load "joan-example-login.arc")
(load "joan-example-cookies.arc")
(load "joan-example-autocomplete-server.arc")
(load "joan-example-autocomplete-server-multiple.arc")
(load "joan-example-encrypted-search.arc")
(load "joan-example-autocomplete-encrypted-search.arc")
(load "joan-example-contact-add.arc")
(load "joan-example-contact-delete.arc")
(load "joan-example-encrypt-for-contact-with-autocomplete.arc")
