(defop example-autocomplete-encrypted-search req
  (page (backlink "/examples") site-name* nil
	;; define what the browser extension should do
	(rex (list (list 'encrypt
			 (obj formbuttons (list (obj form "ENTER_TEXT"
						     btn 'example-autocomplete-encrypted-search))
			      enc-itemlist (list (obj id "body"
						      ;; add ability to search inside this field
						      tosearch t
						      tosearchout "bodyout"))))))
	(pr "This example shows how you can autocomplete multiple results of an encrypted search query.")
	(br 3)
	(pr "There are two steps in this example:")
	(br)
	(tag (ul)
	     (tag (li)
		  (pr "In this page you enter some text, the text is encrypted in a special way that allows searching in it, and then it is saved on the server.")
		  (br2))
	     (tag (li)
		  (pr "In the next page, you can search the encrypted data with autocompletion.")))
	(br2)
	(click-source "joan-example-autocomplete-encrypted-search.arc")
	(br2)
	(sform "ENTER_TEXT"
	       (fn (_)
		   (withs (p (prof (get-user _))
			   bodyout (arg _ "bodyout")
			   ;; encpass is not used for encrypted words
			   encpass nil
			   dataid nil
			   numwords nil)
			  ;; bodyout contains JSON: a list of encrypted words
			  (= bodyout (fromjson bodyout))
			  (= dataid (add-data (get-user _) "searchable-blog-entry" encpass (bodyout "data") (bodyout "permutation") (bodyout "space") (bodyout "case")))
			  ;; the server is expected to keep track of
			  ;; the maximum words in a document, to tell
			  ;; the client to supply at least as many
			  ;; keys (one per word in a document)
			  (= numwords (len (bodyout "data")))
			  (if (or (no p!searchmaxwords) (> numwords p!searchmaxwords))
			      (do (= p!searchmaxwords numwords)
				  (save-prof p)))
			  (example-autocomplete-encrypted-search-page2 _ nil nil nil)))
	       ;; store the output of encryption that is searchable in bodyout, because
	       ;; its content isn't visually appealing (it's JSON)
	       (gentag input type "hidden" id "bodyout" name "bodyout" value "")
	       (tag (textarea id "body" name "body" rows 4 cols 53 autofocus "")
		    (pr ""))
	       (encverif (get-user req))
	       (encsearchinfo (get-user req))
	       (br 3)
	       (tag (a id "example-autocomplete-encrypted-search")
		    (pr "submit")))
	(br 8)
	(hline)))

(def example-autocomplete-encrypted-search-page2 (req results showres query)
  (page (backlink "/examples") site-name* nil
	(withs (user (get-user req) p (prof user)
	        dec-itemlist nil data-counter 0)
	       (rex (list (list 'encrypt
				(obj formbuttons (list (obj form "ENTER_TEXT"
							    btn 'example-autocomplete-encrypted-search
							    encpassid "encpass"))
				     autocomp-itemlist (list (obj url "/example-autocomplete-encrypted-search-contacts"
								  enc-fieldlist (list "name")
								  encpassid "encpass"
								  searchin "autocomplete-target"
								  searchout "autocomplete-out"
								  toretrievekeys (aif p!searchmaxwords it 0)
								  ))
				     )))
		    nil)
	       (sform "ENTER_TEXT"
		      (fn (_)
			  (withs (all (arg _ "autocomplete-out"))
				 ;; not doing anything with encpass in this example
				 (if (no (is all ""))
				     (= all (fromjson all)))
				 (example-autocomplete-encrypted-search-show all (get-user req))))
		      (pr "Enter a word to search in the text you saved.")
		      (pr " Or click ")
		      (tag (a href "/example-autocomplete-encrypted-search")
			   (pr "here"))
		      (pr " to add more text to encrypt.")
		      (br 3)
		      (gentag input type "text" id "autocomplete-target" name "autocomplete-target" value "" autofocus "")
		      (gentag input type "hidden" id "autocomplete-out" name "autocomplete-out" value "")
		      (gentag input type "hidden" id "encpass" name "encpass" value "")
		      (encverif (get-user req))
		      (br 3)
		      (tag (a id "example-autocomplete-encrypted-search")
			   (pr "submit"))
		      (br 8)
		      (hline)
		      ))))

(def example-autocomplete-encrypted-search-show (all user)
  (page (backlink "/examples") site-name* nil
        (pr "You selected the encrypted entries with id:")
        (br2)
	(encverif user)
        (tabwrap
         (tag (ol)
              (each d all
                    (tag (li)
                         (pr (d "id")))
                    (br))))
	(br 3)
	(pr "Notice how no decrypted data were submitted back to the server because that would be insecure. The complete data submitted are:")
	(br2)
	(tag (ul)
	     (tag (li)
		  (pr all)))
        (br 8)
        (hline)))

(defop example-autocomplete-encrypted-search-contacts req
  (withs (out "" term (arg req "q") counter 0
	  results nil p (prof (get-user req)))
	 ;;(fwrite "/tmp/term" term)
	 (= term (fromjson term))
	 (= results (data-search "searchable-blog-entry" term (get-user req) p!searchmperc (allcontacts (get-user req))))
	 (= out (str out "["))
	 (maptable (fn (k data)
		       (if (> counter 0)
			   (= out (str out ",")))
		       (= out (str out "{ \"name\": "
				   (tojson (obj data data!data
						permutation data!encperm
						space data!encspace
						case data!enccase))
				   ", \"id\": \"" data!id "\""
				   ", \"some_plaintext\": \"this was not encrypted to begin with\"}"))
		       (++ counter)
		       ) results)
	 (= out (str out "]"))
	 ;;(fwrite (str "/tmp/out_" (seconds)) out)
	 (pr out)))
