(defop example-encrypted-search req
  (page (backlink "/examples") site-name* nil
	;; define what the browser extension should do
	(rex (list (list 'encrypt
			 (obj formbuttons (list (obj form "ENTER_TEXT"
						     btn 'example-encrypted-search))
			      enc-itemlist (list (obj id "body"
						      ;; add ability to search inside this field
						      tosearch t
						      tosearchout "bodyout"))))))
	(pr "There is a different way to encrypt data, using symmetric encryption instead of asymmetric (public-key) encryption, that offers the advantage of securely searching the encrypted data.")
	(br2)
	(pr "Symmetric encryption is generally weaker than asymmetric, because both parties must have access to a shared key. But it's not weaker in a specific case. If there is only one party, the shared key is not exchanged, and everything else (like the length of the shared key and the encryption algorithm) is equal. For example, when encrypting data that will be searched only by you.")
	(br 3)
	(pr "The server cannot search without user authorization, cannot decrypt search results, and cannot decrypt the search query. It learns nothing more than the encrypted query and its encrypted results. The order of encrypted words is scrambled, and the frequency of a word in a document is not exposed.")
	(br2)
	(pr "Some drawbacks are inevitable with this feature:")
	(br)
	(tag (ul)
	     (tag (li)
		  (pr "You can't search for punctuation symbols, like . and @, because they are treated as word separators. This helps search for an email address without typing the full address. That's what people do now anyway, so this isn't a problem.")
		  (br2))
	     (tag (li)
		  (pr "A real drawback is you can't search by typing only part of a word. You have to type words in their entirety.")
		  (br2))
	     (tag (li)
		  (pr "You can't search both encrypted and decrypted data with the same query because you would have to reveal the plaintext query to do that. This hasn't been a problem so far.")
		  (br)))
	(br)
	(pr "There are two steps in this example. In this page you enter some text, the text is encrypted in a special way that allows searching in it, and then it is saved on the server.")
	(br2)
	(pr "In the next page, you can search over the encrypted data.")
	(br 3)
	(click-source "joan-example-encrypted-search.arc")
	(br2)
	(sform "ENTER_TEXT"
	       (fn (_)
		   (withs (p (prof (get-user _))
			   bodyout (arg _ "bodyout")
			   ;; encpass is not used for searchable encryption
			   encpass nil
			   dataid nil
			   numwords nil)
			  ;; bodyout contains JSON: a list of encrypted words
			  (= bodyout (fromjson bodyout))
			  (= dataid (add-data (get-user _) "searchable-blog-entry" encpass (bodyout "data") (bodyout "permutation") (bodyout "space") (bodyout "case")))
			  ;; the server is expected to keep track of
			  ;; the maximum words in a document, to tell
			  ;; the client to supply at least as many
			  ;; keys (one per word in a document)
			  (= numwords (len (bodyout "data")))
			  (if (or (no p!searchmaxwords) (> numwords p!searchmaxwords))
			      (do (= p!searchmaxwords numwords)
				  (save-prof p)))
			  (example-encrypted-search-page2 _ nil nil nil)))
	       ;; store the output of encryption that is searchable in bodyout, because
	       ;; its content isn't visually appealing (it's JSON)
	       (gentag input type "hidden" id "bodyout" name "bodyout" value "")
	       (encverif (get-user req))
	       (encsearchinfo (get-user req))
	       (tag (textarea id "body" name "body" rows 4 cols 53 autofocus "")
		    (pr ""))
	       (br 3)
	       (tag (a id "example-encrypted-search")
		    (pr "submit")))
	(br 8)
	(hline)))

(def example-encrypted-search-page2 (req results showres query)
  (page (backlink "/examples") site-name* nil
	(withs (user (get-user req) p (prof user)
	        dec-itemlist nil data-counter 0)
	       ;; must provide the password and id of the item to decrypt
	       (if showres
		   (maptable (fn (k data)
				 (push (obj id (str "entry_" data-counter)
					    fromsearch t
					    fromsearchin (str "entry_" data-counter "_in")
					    fromsearchtextarea t) dec-itemlist)
				 (++ data-counter))
			     results))
	       (rex (list (list 'encrypt
				(obj formbuttons (list (obj form "ENTER_TEXT"
							    btn 'example-encrypted-search))
				     enc-itemlist (list (obj id "body"
							     ;; perform a search query
							     toretrieve t
							     toretrievekeys p!searchmaxwords
							     toretrieveout "bodyout"))
				     dec-itemlist dec-itemlist))))

	       (if showres
		   (do (pr "The encrypted search query was:")
		       (br2)
		       (tabwrap
			(pr query))
		       (br 3)
		       ;; --------- start encrypted results --------
		       ;; I'm showing both encrypted and encrypted
		       ;; results, so you can see that encrypted
		       ;; results can visually pollute the user
		       ;; interface
		       (pr "The encrypted search results are:")
		       (br2)
		       (tabwrap
			(tag (ol)
			     (= data-counter 0)
			     (maptable (fn (k data)
					   (tag (li)
						(tag (span id (str "entry_" data-counter "_visible"))
						     (pr (tojson (obj data data!data
								      permutation data!encperm
								      space data!encspace
								      case data!enccase))))
						(br))
					   (++ data-counter))
				       results)))
		       ;; --------- end encrypted results --------
		       (br2)
		       (pr "The decrypted search results are:")
		       (br2)
		       (tabwrap
			(tag (ol)
			     (= data-counter 0)
			     (maptable (fn (k data)
					   (tag (li)
						(gentag input type "hidden"
							id (str "entry_" data-counter "_in")
							name (str "entry_" data-counter "_in")
							value (tojson (obj data data!data
									   permutation data!encperm
									   space data!encspace
									   case data!enccase)))
						(tag (span id (str "entry_" data-counter)))
						(br))
					   (++ data-counter))
				       results)))
		       (br2)))

	       (sform "ENTER_TEXT"
		      (fn (_)
			  (withs (p (prof (get-user _))
				  body (arg _ "bodyout")
				  thedata nil)
				 (= body (fromjson body))
				 (= thedata (data-search "searchable-blog-entry" body (get-user _) p!searchmperc (alldata (get-user _))))
				 ;; do the search, and fill thedata with the matches
				 (example-encrypted-search-page2 _ thedata t (arg _ "bodyout"))))
		      (pr "Enter a word to search in the text you saved.")
		      (if showres
			  (do (pr " Or click ")
			      (tag (a href "/example-encrypted-search")
				   (pr "here"))
			      (pr " to add more text to encrypt.")))
		      (br 3)
		      ;; store the encrypted search query in bodyout, because
		      ;; its content isn't visually appealing (it's JSON)
		      (gentag input type "hidden" id "bodyout" name "bodyout" value "")
		      (gentag input type "text" id "body" name "body" value "" style "width: 300px;" autofocus "")
		      (br 3)
		      (encverif (get-user req))
		      (encsearchinfo (get-user req))
		      (tag (a id "example-encrypted-search")
			   (pr "submit"))
		      (br 8)
		      (hline)
		      ))))
