#!/bin/bash

# package Joan
VERSION=`cat chrome-extension/prep.sh|grep VERSION| head -1| sed -e 's/.*=//' | xargs echo -n`


# Step 1: Create a new .crx file called chrome-extension.crx, by
# running something like:
#
# $ cd chrome-extension; ./prep.sh  http://104.236.73.78:1412 JoanDemo
#
# and then go to Chrome->Window->Extensions->Pack extension and                            
# follow the instructions
#
#
# Step 2: run this script
if [ "$1" == "clean" ]; then
    cd chrome-extension
    ./clean.sh
    cd -

    rm -rf joan-server > /dev/null
fi
mkdir -p joan-server/static
rm -f joan-server/static/joan*.tar.gz

for file in as.scm ac.scm brackets.scm libs.arc strings.arc pprint.arc code.arc html.arc srv.arc app.arc prompt.arc arc.arc news.arc blog.arc how-to-run-news copyright static/arc.png static/grayarrow.gif static/graydown.gif static/s.gif parsecomb0.arc fromjson.arc; do
    cp $file joan-server/$file
done

cp joan.arc joan-server/.
for file in joan-example-text.arc \
            joan-example-noencrypt-alert.arc \
            joan-example-nojs.arc \
            joan-example-code-from-server.arc \
            joan-example-code-from-server-sent.arc \
            joan-example-compute-from-decrypted.arc \
            joan-example-compute-from-decrypted-sent.arc \
            joan-example-nodecrypt.arc \
            joan-example-nodecrypt-sent-good.arc \
            joan-example-nodecrypt-sent-bad-static.arc \
            joan-example-nodecrypt-sent-bad-dynamic.arc \
            joan-example-noeval.arc \
            joan-example-noeval-sent.arc \
            joan-example-date.arc \
            joan-example-date-sent.arc \
            joan-example-multiple-forms.arc \
            joan-example-payments.arc \
            joan-example-payments-sent.arc \
            joan-example-login.arc \
            joan-example-cookies.arc \
            joan-example-cookies-sent.arc \
            joan-example-cookies-reset-sent.arc \
            joan-example-nodom.arc \
            joan-example-nodom-direct-sent.arc \
            joan-example-nodom-srex-get-wtype-sent.arc \
            joan-example-nodom-access-textarealist-sent.arc \
            joan-example-nodom-access-encverifiedkeyids-sent.arc \
            joan-example-autocomplete-server.arc \
            joan-example-autocomplete-server-sent.arc \
            joan-example-autocomplete-server-multiple.arc \
            joan-example-autocomplete-server-multiple-sent.arc \
            joan-example-encrypted-search.arc \
            joan-example-autocomplete-encrypted-search.arc \
            joan-example-failed-signature.arc \
            joan-example-contact-add.arc \
            joan-example-contact-add-somefakepubkey \
            joan-example-contact-add-sent.arc \
            joan-example-contact-delete.arc \
            joan-example-noonclick.arc \
            joan-example-noonclick-sent.arc \
            joan-example-encrypt-for-contact-with-autocomplete.arc \
    ; do
    cp $file joan-server/.
    cp $file joan-server/static/.
done
cp static/arc.png joan-server/static/favicon.png
cp static/slow_js.png joan-server/static/.
cp joan.sh joan-server/.
cp package_joan.sh joan-server/.
cp chrome-extension.crx joan-server/static/joan$VERSION-browser-extension.crx
cp chrome-extension/share_with_plugin.arc joan-server/.
cp chrome-extension/share_with_plugin_driver.arc joan-server/.
cp -Rp chrome-extension/cryptojs joan-server/static/.
echo "This is the server portion of the Joan demo. It is implemented in Arc but
you don't have to use an Arc server if there is something more convenient.

Run ./joan.sh to start, visit http://127.0.0.1:1412, and follow the
instructions.

The compiled extension .crx file is in:
joan-server/static/joan$VERSION-browser-extension.crx

To compile a new extension for Chrome type on the command-line:
$ cd chrome-extension
$ ./prep.sh http://some.ip.address.com MyWebSiteName

...and then go to Chrome->Window->Extensions->Pack extension and
follow the instructions.
" > joan-server/README
echo "This software is copyright (c) Kristis Makris.  Permission to use it is
granted under the Perl Foundations's Artistic License 2.0." > joan-server/copyright.joan
echo > joan-server/robots.txt
echo "(= version* $VERSION)" > joan-server/joan-version.arc

rm -rf joan$VERSION
mkdir joan$VERSION
cp -Rp joan-server joan$VERSION/. 
cp -Rp chrome-extension joan$VERSION/.
cp package_joan.sh joan$VERSION/.
tar -czf joan$VERSION.tar.gz joan$VERSION

# Put a copy of the source code in the demo
cp joan$VERSION.tar.gz joan-server/static
